<?php
/**
* ConfigurationRequest.php - Request file
*
* This file is part of the Configuration component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Configuration\Requests;

use Illuminate\Support\Arr;
use Illuminate\Http\Request;
use App\Yantrana\Base\BaseRequest;

class ConfigurationRequest extends BaseRequest
{
    /**
     * Loosely sanitize fields.
     *------------------------------------------------------------------------ */
    protected $looseSanitizationFields = [
        'terms_and_conditions' => true,
        'privacy_policy' => true,
        'cookie_policy' => true
    ];

    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     *-----------------------------------------------------------------------*/
    public function authorize()
    {
        return true;
    }

    protected $localAttributes = [];

    public function attributes()
    {
        return $this->localAttributes;
    }


    public function updateAttributes($attributeItems)
    {
        return $this->localAttributes = array_merge($this->localAttributes, $attributeItems);
    }


    /**
     * Get the validation rules that apply to the user register request.
     *
     * @return bool
     *-----------------------------------------------------------------------*/
    public function rules()
    {
        $requestData = new Request();
        $inputData = $this->all();

        $pageType = request()->pageType;

        $rules = [];
        // check validation via page type
        switch ($pageType) {
            case 'general':
                $rules = [
                    'name' => 'required',
                    'business_email' => 'required|email',
                    'contact_email' => 'required|email',
                ];
                break;

            case 'credit-package':
                $formType = $requestData->input('form_type');
                if ($formType == 'currency_form') {
                    $rules = [
                        'currency' => 'required',
                        'currency_symbol' => 'required',
                        'currency_value' => 'required',
                        'round_zero_decimal_currency' => 'required',
                    ];
                } else {
                    $uid = $inputData['uid'];
                    foreach ($inputData['credit_packages']['package_data'][$uid] as $key => $value) {
                        $rules['credit_packages.package_data.' . $uid . '.' . $key] = 'required';
                    }
                }

                break;

            case 'payment':
                $enablePaypal = $inputData['enable_paypal'];
                $enableStripe = $inputData['enable_stripe'];
                $enableRazorpay = $inputData['enable_razorpay'];

                // check if paypal checkout is enable
                if ($enablePaypal) {
                    $paypalTestMode = $inputData['use_test_paypal_checkout'];
                    // Check if paypal test mode is enable
                    if ($paypalTestMode and !$inputData['paypal_test_keys_exist']) {
                        $rules = [
                            'paypal_checkout_testing_client_id' => 'required',
                            'paypal_checkout_testing_secret_key' => 'required',
                        ];
                    } elseif (!$paypalTestMode and !array_get($inputData, 'paypal_live_keys_exist')) {
                        $rules = [
                            'paypal_checkout_live_client_id' => 'required',
                            'paypal_checkout_live_secret_key' => 'required',
                        ];
                    }
                }

                // check if stripe payment enable
                if ($enableStripe) {
                    $stripeTestMode = $inputData['use_test_stripe'];
                    // Check if stripe test mode is enable
                    if ($stripeTestMode and !$inputData['stripe_test_keys_exist']) {
                        $rules = [
                            'stripe_testing_secret_key' => 'required',
                            'stripe_testing_publishable_key' => 'required',
                        ];
                    } elseif (!$stripeTestMode and !array_get($inputData, 'stripe_live_keys_exist')) {
                        $rules = [
                            'stripe_live_secret_key' => 'required',
                            'stripe_live_publishable_key' => 'required',
                        ];
                    }
                }

                // check if razorpay payment enable
                if ($enableRazorpay) {
                    $razorpayTestMode = $inputData['use_test_razorpay'];
                    // Check if stripe test mode is enable
                    if ($razorpayTestMode and !$inputData['razorpay_test_keys_exist']) {
                        $rules = [
                            'razorpay_testing_key' => 'required',
                            'razorpay_testing_secret_key' => 'required',
                        ];
                    } elseif (!$razorpayTestMode and !array_get($inputData, 'razorpay_live_keys_exist')) {
                        $rules = [
                            'razorpay_live_key' => 'required',
                            'razorpay_live_secret_key' => 'required',
                        ];
                    }
                }

                break;

            case 'social-login':
                $allFacebookLogin = $inputData['allow_facebook_login'];
                $allGoogleLogin = $inputData['allow_google_login'];
                //if facebook login are enable then add required validation
                if ($allFacebookLogin and !$inputData['facebook_keys_exist']) {
                    $rules = [
                        'facebook_client_id' => 'required',
                        'facebook_client_secret' => 'required',
                    ];
                }
                //if google login are enable then add required validation
                if ($allGoogleLogin and !$inputData['google_keys_exist']) {
                    $rules = [
                        'google_client_id' => 'required',
                        'google_client_secret' => 'required',
                    ];
                }
                break;

            case 'integration':
                $allowRecaptcha = $inputData['allow_recaptcha'];
                //if recaptcha are enable then add required validation
                if ($allowRecaptcha and !$inputData['recaptcha_site_key_exist']) {
                    $rules = [
                        'recaptcha_site_key' => 'required',
                        'recaptcha_secret_key' => 'required',
                    ];
                }
                break;
            case 'ai-provider':
                $allowOpenAi = $inputData['enable_open_ai'];
                $allowStableDiffusion = $inputData['allow_stable_diffusion_ai'];
                $rules = [];
                if($allowOpenAi and !$inputData['open_ai_key_exist']) {
                    $rules = array_merge($rules, [
                        'open_ai_keys' => 'required',
                    ]);
                }

                if($allowStableDiffusion and !$inputData['stable_diffusion_key_exist']) {
                    $rules = array_merge($rules, [
                        'stable_diffusion_key' => 'required',
                    ]);
                }
                if(Arr::get($inputData, 'enable_image_animation')
                    and (!Arr::get($inputData, 'is_leiapix_close_for_update'))) {
                    $this->updateAttributes([
                        'leiapix_s3_access_key' => __tr('S3 Access Key'),
                        'leiapix_s3_access_secret' => __tr('S3 Access Secret'),
                        'leiapix_s3_region' => __tr('S3 Region'),
                        'leiapix_s3_bucket' => __tr('S3 Bucket'),
                        'leiapix_s3_url' => __tr('S3 URL'),
                        'leiapix_s3_endpoint' => __tr('S3 ENDPOINT URL'),
                    ]);
                    $rules = array_merge($rules, [
                        'leiapix_client_id' => 'required',
                        'leiapix_client_secret' => 'required',
                    ]);
                    if(!Arr::get($inputData, 'is_leiapix_s3_provider_close_for_update')) {
                        $rules = array_merge($rules, [
                            // s3 configurations
                            'leiapix_s3_access_key' => 'required',
                            'leiapix_s3_access_secret' => 'required',
                            'leiapix_s3_region' => 'required',
                            'leiapix_s3_bucket' => 'required',
                            'leiapix_s3_url' => 'required|url',
                            'leiapix_s3_endpoint' => 'required|url',
                        ]);
                    }
                }
                break;
            case 'email':

                //driver specific rules
                $driverRules = [];

                if ($inputData['use_env_default_email_settings'] != '1') {
                    //default rules
                    $rules = [
                        'mail_from_address' => 'required|email',
                        'mail_from_name' => 'required',
                        'mail_driver' => 'required',
                    ];

                    //for driver specific rules
                    switch ($inputData['mail_driver']) {
                        case 'smtp':
                            $driverRules = [
                                'smtp_mail_host' => 'required',
                                'smtp_mail_port' => 'required',
                                'smtp_mail_encryption' => 'required',
                                'smtp_mail_username' => 'required',
                                'smtp_mail_password_or_apikey' => 'required',
                            ];
                            break;

                        case 'sparkpost':
                            $driverRules = [
                                'sparkpost_mail_password_or_apikey' => 'required',
                            ];
                            break;

                        case 'mailgun':
                            $driverRules = [
                                'mailgun_domain' => 'required',
                            ];
                            break;

                        default:
                            $driverRules = [];
                            break;
                    }
                }
                if($inputData['use_enable_sms_settings']) {

                    switch ($inputData['sms_driver']) {
                        case 'twilio':
                            $driverRules = [
                                'sms_twilio_sid' => 'required',
                                'sms_twilio_token' => 'required',
                                'sms_twilio_from' => 'required',
                            ];
                            break;

                        case 'textlocal':
                            $driverRules = [
                                'sms_textlocal_hash' => 'required',
                                'sms_textlocal_from' => 'required',
                                'sms_textlocal_username' => 'required',
                                'sms_textlocal_url' => 'required',
                            ];
                            break;

                        case 'sms77':
                            $driverRules = [
                                'sms_sms77_apiKey' => 'required',
                                'sms_sms77_flash' => 'required',
                                'sms_sms77_from' => 'required',
                            ];
                            break;

                        default:
                            $driverRules = [];
                            break;
                    }
                }

                $rules = array_merge($rules, $driverRules);
                break;

            case 'usage-pricing':
                $rules = [
                    'speech_to_text_pricing' => 'required|integer|min:1',
                    'text_to_speech_pricing' => 'required|integer|min:1',
                    'image_animation_pricing' => 'required|integer|min:1',
                    'per_word_pricing' => 'required|integer|min:1',
                    'image_pricing.512x512' => 'required|integer|min:1',
                    'image_pricing.256x256' => 'required|integer|min:1',
                    'image_pricing.1024x1024' => 'required|integer|min:1',
                ];
                break;
            case 'terms-and-policies':
            default:
                $rules = [];
                break;
        }

        return $rules;
    }
}
