<?php

/**
 * DashboardEngine.php - Main component file
 *
 * This file is part of the Dashboard component.
 *-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Dashboard;

use App\Yantrana\Base\BaseEngine;
use Carbon\Carbon;
use App\Yantrana\Components\Dashboard\Interfaces\DashboardEngineInterface;
use App\Yantrana\Components\Dashboard\Repositories\DashboardRepository;
use App\Yantrana\Components\Prompt\Repositories\PromptRepository;

class DashboardEngine extends BaseEngine implements DashboardEngineInterface
{
    /**
     * @var  DashboardRepository - Dashboard Repository
     */
    protected $dashboardRepository;

    /**
     * @var  PromptRepository $promptRepository - Prompt Repository
     */
    protected $promptRepository;

    /**
     * Constructor
     *
     * @param  DashboardRepository  $dashboardRepository - Dashboard Repository
     * @return  void
     *-----------------------------------------------------------------------*/
    public function __construct(DashboardRepository $dashboardRepository, PromptRepository $promptRepository)
    {
        $this->dashboardRepository = $dashboardRepository;
        $this->promptRepository = $promptRepository;
    }

    /**
     * Prepare Dashboard
     *
     * @return  void
     *-----------------------------------------------------------------------*/
    public function prepareDashboard($startDate = null, $endDate = null)
    {
        if ($startDate == null) {
            $startDate = Carbon::now()->firstOfMonth();
        }
        if ($endDate == null) {
            $endDate = Carbon::now()->lastOfMonth();
        }
        //active users
        $activeUsers = $this->dashboardRepository->fetchUsers(['status' => 1])->count();
        //inactive users
        $inactiveUsers = $this->dashboardRepository->fetchUsers(['status' => 2])->count();
        //blocked users
        $blockedUsers = $this->dashboardRepository->fetchUsers(['status' => 3])->count();
        //all text generation count
        $textGeneration = $this->promptRepository->fetchCountGenerationAdminDashboard('text', $startDate, $endDate);
        //all image generation count
        $imageGeneration = $this->promptRepository->fetchCountGenerationAdminDashboard('image', $startDate, $endDate);

        //all code generation count
        $codeGeneration = $this->promptRepository->fetchCountGenerationAdminDashboard('code', $startDate, $endDate);

        //all speech-to-text generation count
        $speechToTextGeneration = $this->promptRepository->fetchCountGenerationAdminDashboard('speech-to-text', $startDate, $endDate);

        //all text translation count
        $textTranslation = $this->promptRepository->fetchCountGenerationAdminDashboard('translation', $startDate, $endDate);

        //all image animation count
        $imageAnimation = $this->promptRepository->fetchCountGenerationAdminDashboard('image-animation', $startDate, $endDate);

        //all text-to-speech generation count
        $textToSpeechGeneration = $this->promptRepository->fetchCountGenerationAdminDashboard('text-to-speech', $startDate, $endDate);

        // all transactions graph
        $transactions = $this->dashboardRepository->currentYearFinancialTransactions(2)->toArray();

        //monthly transactions graph
        $monthlyTransactions = collect($transactions)->groupBy('month');

        //all earnings count
        $adminsEarnings = number_format((float) $this->dashboardRepository->adminsTotalEarning($startDate, $endDate), 2);

        // all users registered in current year
        $currentYearRegistrations = $this->dashboardRepository->currentYearRegistrations()->toArray();
        //month wise collection
        $monthWiseUsers = collect($currentYearRegistrations)->groupBy(['gender', 'month']);

        $datasets = [
            1 => [
                'label' => 'Male',
                'backgroundColor' => 'blue',
                'data' => array_fill(0, 12, 0),
            ],
            2 => [
                'label' => 'Female',
                'backgroundColor' => 'pink',
                'borderColor' => 'pink',
                'borderWidth' => 1,
                'data' => array_fill(0, 12, 0),
            ],
            3 => [
                'label' => 'Secret',
                'backgroundColor' => 'grey',
                'borderColor' => 'grey',
                'borderWidth' => 1,
                'data' => array_fill(0, 12, 0),
            ],
        ];
        //show all months graph
        $allTheMonths = collect(range(11, 0))->map(function ($i) {
            $dt = today()->startOfMonth()->subMonth($i);
            return $dt->month;
        });

        //dashboard counts
        $dashboardCounts = [
            'current_months_income' => 0,
            'month_wise_income' => array_fill(0, 12, 0),
            'currency' => getCurrencySymbol(),
            'current_year_registrations' => $datasets,
            'labels' => [],
        ];

        $monthWiseUsersArr = [];
        //manage month wise users data for graph
        for ($i = 0; $i < count($allTheMonths); $i++) {
            for ($j = 0; $j < count($datasets); $j++) {
                $monthWiseUsersArr[$j][$allTheMonths[$i]] = 0;
            }
        }
        //check if month wise users not empty
        if (!__isEmpty($monthWiseUsers)) {
            foreach ($monthWiseUsers as $key => $monthWiseUser) {
                foreach ($monthWiseUser as $key2 => $mtUsers) {
                    $monthWiseUsersArr[$key - 1][$key2] = $mtUsers->count();
                }
            }
        }
        //calculate current year registrations of users for graph
        foreach ($monthWiseUsersArr as $monthWiseUsersArrKey => $monthWiseUsersArrValue) {
            $j = 0;
            foreach ($monthWiseUsersArrValue as $key => $value) {
                $dashboardCounts['current_year_registrations'][$monthWiseUsersArrKey + 1]['data'][$j] = $value;
                $j++;
            }
        }

        //manage label of months for graph
        $allTheMonths = collect(range(11, 0))->map(function ($i) {
            $dt = today()->startOfMonth()->subMonth($i);
            return [
                'month_number' => $dt->month,
                'month_name' => $dt->translatedFormat('M') . ' ' . $dt->translatedFormat('y'),
            ];
        });
        //print month wise data for graph
        $months = $last12MonthsLabels = [];
        for ($i = 0; $i < count($allTheMonths); $i++) {
            $dashboardCounts['labels'][$i] = $allTheMonths[$i]['month_name'];
            $months[$allTheMonths[$i]['month_number']] = 0;
        }

        $currentMonth = (int) date('m');
        // calculate earnings of filtered date
        if (!__isEmpty($monthlyTransactions)) {
            foreach ($monthlyTransactions as $key => $trans) {
                if (!__isEmpty($trans)) {
                    $amount = array_sum($trans->pluck('amount')->toArray());
                    // total earnings
                    $months[$key] = $amount;
                }
            }

        }

        //month wise income
        $i = 0;
        foreach ($months as $monthKey => $month) {
            $dashboardCounts['month_wise_income'][$i] = $months[$monthKey];
            $i++;
        }
        //returns all count values
        return [
            'dashboardData' => $dashboardCounts,
            'firstOfMonth' => Carbon::now()->firstOfMonth(),
            'lastOfMonth' => Carbon::now()->lastOfMonth(),
            'totalEarnings' => $dashboardCounts['currency'] . $adminsEarnings,//earningCounts,
            'allActiveUsersCounts' => $activeUsers,
            'allInactiveUsersCounts' => $inactiveUsers,
            'allblockedUsersCounts' => $blockedUsers,
            'textGeneratedCounts' => $textGeneration, //text generated count
            'imageGeneratedCounts' => $imageGeneration, //image generated count
            'codeGeneratedCounts' => $codeGeneration, //code generated count
            'speechToTextGeneratedCounts' => $speechToTextGeneration, //speech to text generated count
            'textTranslationCount' => $textTranslation, //translation count
            'imageAnimationCounts' => $imageAnimation, //image animated
            'textToSpeechGeneratedCounts' => $textToSpeechGeneration, //speech to text generated count

        ];
    }
    /**
     * Prepare admin Dashboard filtered Data
     *
     * @param   Array  $inputData
     *
     * @return  Array
     */
    public function prepareAdminsDashboardData($inputData)
    {
        //get filtered duration
        $duration = $inputData['daterange'];
        $duration = explode(' - ', $duration);
        $startDate = $endDate = null;
        //check if duration set
        if (isset($duration[0]) and isset($duration[1])) {
            $startDate = Carbon::parse($duration[0] . ' 00:00:00');
            $endDate = Carbon::parse($duration[1] . ' 23:59:59');
        }

        // Calculate the difference between the two dates in days
        $daysDifference = $startDate->diffInDays($endDate);
        //check days difference not exceeds 1yr
        if ($daysDifference > 365) {
            return $this->engineReaction(2, ['show_message' => true], __tr('Duration exceeds more than 1 year.'));
        }
        //fetch filtered dashboard data
        $adminDashboardData = $this->prepareDashboard($startDate, $endDate);


        return $this->engineReaction(1, $adminDashboardData);
    }
}
