<?php
/**
* HomeController.php - Controller file
*
* This file is part of the Home component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Home\Controllers;

use Illuminate\Support\Str;
use App\Yantrana\Base\BaseController;
use App\Yantrana\Components\Home\HomeEngine;
use App\Yantrana\Components\Pages\ManagePagesEngine;
use App\Yantrana\Support\CommonUnsecuredPostRequest;
use App\Yantrana\Components\UserSetting\UserSettingEngine;
use App\Yantrana\Components\CreditPackage\CreditPackageEngine;

class HomeController extends BaseController
{
    /**
     * @var  HomeEngine - Home Engine
     */
    protected $homeEngine;

    /**
     * @var  ManagePagesEngine - Manage Pages Engine
     */
    protected $managePageEngine;

    /**
     * @var  UserSettingEngine - UserSetting Engine
     */
    protected $userSettingEngine;

    /**
     * @var  CreditPackageEngine - Credit Packages engine
     */
    protected $creditPackageEngine;

    /**
     * Constructor
     *
     * @param  HomeEngine  $homeEngine - Home Engine
     * @param  ManagePagesEngine  $managePageEngine - Manage Pages Engine
     * @return  void
     *-----------------------------------------------------------------------*/
    public function __construct(
        HomeEngine $homeEngine,
        ManagePagesEngine $managePageEngine,
        UserSettingEngine $userSettingEngine,
        CreditPackageEngine $creditPackageEngine,
    ) {
        $this->homeEngine = $homeEngine;
        $this->managePageEngine = $managePageEngine;
        $this->userSettingEngine = $userSettingEngine;
        $this->creditPackageEngine = $creditPackageEngine;
    }

    /**
     * View Home Page
     *---------------------------------------------------------------- */
    public function homePage()
    {

        $data = $this->homeEngine->getDashboardData();
        return $this->loadPublicView('home', $data);
    }

    /**
     * ChangeLocale - It also managed from index.php.
     *---------------------------------------------------------------- */
    protected function changeLocale(CommonUnsecuredPostRequest $request, $localeId = null)
    {
        if (is_string($localeId)) {
            changeAppLocale($localeId);
            if (isLoggedIn()) {
                $this->userSettingEngine->processUserSettingStore('site_defaults', [
                    'selected_locale' => $localeId,
                ]);
            }
        }
        if ($request->has('redirectTo')) {
            $redirectUrl = base64_decode($request->get('redirectTo'));
            if(Str::startsWith($redirectUrl, url(''))) {
                header("Location: $redirectUrl");
                exit();
            }
        }

        return __tr('Invalid Request');
    }

    /**
     * preview page
     *---------------------------------------------------------------- */
    public function previewPage($pageUid, $title)
    {
        $processReaction = $this->managePageEngine->previewPage($pageUid);

        return $this->loadView('pages.preview', $processReaction['data'], [
            'compress_page' => false,
        ]);
    }

    /**
     * Pricing page
     *---------------------------------------------------------------- */
    public function pricingPage()
    {
        $processReaction = $this->creditPackageEngine->prepareCreditPackageList();
        return $this->loadView('pricing', [
          'creditPackageData' => $processReaction->data('creditPackageData')
        ]);
    }

    /**
     * preview landing page
     *---------------------------------------------------------------- */
    public function landingPage()
    {
        $getAllTemplatesData =  getPromptTemplates(null, null, [
            'with_total_count' => true
        ]);
        $getAllTemplates = $getAllTemplatesData['templates'];
        return $this->loadView('outer-home',[
            'categoriesList' => $getAllTemplates,
            'totalTemplateCount' => $getAllTemplatesData['totalTemplateCount'],
        ]);
    }

    /**
     * Get Dashboard User Data
     *
     * @param   CommonUnsecuredPostRequest  $request
     *
     * @return  Array
     */
    public function getDashboardUserData(CommonUnsecuredPostRequest $request)
    {
        $request->validate([
            'daterange' => [
                'required'
            ]
        ]);

        $prepareUserDashboardData = $this->homeEngine->prepareUserDashboardData($request->all());
        // Update client side
        updateClientModels($prepareUserDashboardData['data']);

        return $this->responseAction(
            $this->processResponse($prepareUserDashboardData, [], [], true)
        );
    }



}
