<?php

/**
 * HomeEngine.php - Main component file
 *
 * This file is part of the Home component.
 *-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Home;

use Carbon\Carbon;
use App\Yantrana\Base\BaseEngine;
use App\Yantrana\Components\Home\Interfaces\HomeEngineInterface;
use App\Yantrana\Components\Prompt\Repositories\PromptRepository;
use App\Yantrana\Components\User\Repositories\ReferralRepository;
use App\Yantrana\Components\FinancialTransaction\Repositories\FinancialTransactionRepository;

class HomeEngine extends BaseEngine implements HomeEngineInterface
{
    /**
     * @var  PromptRepository $promptRepository - Prompt Repository
     */
    protected $promptRepository;

    /**
     * @var  ReferralRepository $referralRepository - Referral Repository
     */
    protected $referralRepository;

    /**
     * @var  FinancialTransactionRepository - FinancialTransaction Repository
     */
    protected $financialTransactionRepository;

    /**
     * Constructor
     *
     * @param  PromptRepository $promptRepository - Prompt Repository
     * @param  FinancialTransactionRepository  $financialTransactionRepository - FinancialTransaction Repository
     *
     * @return  void
     *-----------------------------------------------------------------------*/
    public function __construct(PromptRepository $promptRepository, ReferralRepository $referralRepository, FinancialTransactionRepository $financialTransactionRepository)
    {
        $this->promptRepository = $promptRepository;
        $this->referralRepository = $referralRepository;
        $this->financialTransactionRepository = $financialTransactionRepository;
    }


    /**
     * Get Dashboard Data
     *
     * @param   Date  $startDate
     * @param   Date  $endDate
     *
     * @return  Array
     */
    public function getDashboardData($startDate = null, $endDate = null)
    {
        //default date for filter
        if($startDate == null) {
            $startDate = Carbon::now()->firstOfMonth();
        }
        if($endDate == null) {
            $endDate = Carbon::now()->lastOfMonth();
        }
        //fetch credit transaction data for graph
        $prepareGenerationGraphData = $this->promptRepository->prepareGenerationGraph($startDate, $endDate);
        $userTransactionTypes = configItem('user_transaction_type'); //fetch integer value of transaction type
        // items not needed for graph
        $skipItemsForChart = [
            1, // purchase
            6, // Welcome Bonus
            9, // Referral Bonus
            10 // Referral Earning
        ];
        $userTransactionTypesData = [];
        //set initial graph value 0
        foreach($userTransactionTypes as $userTransactionTypeKey => $userTransactionTypeValue) {
            // if it needs to show in graph & assign default values
            if(!in_array($userTransactionTypeKey, $skipItemsForChart)) {
                $userTransactionTypesData[$userTransactionTypeValue] = 0;
            }
        }

        $collectionData = [];
        //bar graph data manage
        $datasets = [
            [
                //graph indication label
                'label' => __tr("My Credits Usages"),
                //fetch data for bar graph
                'data' => $userTransactionTypesData,
                //show background color for bar graph
                'backgroundColor' => [
                    'rgba(255, 99, 132, 0.2)',
                    'rgba(255, 159, 64, 0.2)',
                    'rgba(255, 205, 86, 0.2)',
                    'rgba(75, 192, 192, 0.2)',
                    'rgba(54, 162, 235, 0.2)',
                    'rgba(153, 102, 255, 0.2)',
                    'rgba(153, 102, 255, 0.2)',
                ],
                //show border color for bar graph
                'borderColor' => [
                    'rgb(255, 99, 132)',
                    'rgb(255, 159, 64)',
                    'rgb(255, 205, 86)',
                    'rgb(75, 192, 192)',
                    'rgb(54, 162, 235)',
                    'rgb(153, 102, 255)',
                    'rgb(54, 162, 235)',
                ],
                //set border width for bar graph
                'borderWidth' => 1
            ]
        ];
        // loop through the result received
        foreach($prepareGenerationGraphData as $transactionKey => $transaction) {
            // if it needs to show in graph & credit type has value
            if(!in_array($transaction->credit_type, $skipItemsForChart) and $transaction->credit_type) {
                // assign positive value for credits
                $datasets[0]['data'][$userTransactionTypes[$transaction->credit_type]] = abs($transaction->total_credits_used);

            }
        }
        // assign dataset to data
        $collectionData['data'] = $datasets;
        //all image generation count
        $imageGeneration = $this->promptRepository->fetchCountGeneration('image', $startDate, $endDate);

        //all code generation count
        $codeGeneration = $this->promptRepository->fetchCountGeneration('code', $startDate, $endDate);

        //all speech-to-text generation count
        $speechToTextGeneration = $this->promptRepository->fetchCountGeneration('speech-to-text', $startDate, $endDate);

        //all text generation count
        $textGeneration = $this->promptRepository->fetchCountGeneration('text', $startDate, $endDate);
        //all text translation count
        $textTranslation = $this->promptRepository->fetchCountGeneration('translation', $startDate, $endDate);
        //all image animation count
        $imageAnimation = $this->promptRepository->fetchCountGeneration('image-animation', $startDate, $endDate);
        //all text-to-speech generation count
        $textToSpeechGeneration = $this->promptRepository->fetchCountGeneration('text-to-speech', $startDate, $endDate);


        return [
            //return all count values
            'textGenerationCount' => __tr($textGeneration),
            'speechToTextGenerationCount' => __tr($speechToTextGeneration),
            'textToSpeechGenerationCount' => __tr($textToSpeechGeneration),
            'codeGenerationCount' => __tr($codeGeneration),
            'imageGenerationCount' => __tr($imageGeneration),
            'firstOfMonth' => Carbon::now()->firstOfMonth(),
            'lastOfMonth' => Carbon::now()->lastOfMonth(),
            'dashboardCounts' => $collectionData,
            'creditsEarnedReferral' => __tr($this->referralRepository->getCreditsEarnedReferral($startDate, $endDate)),
            'totalTranslations' => __tr($textTranslation),
            'totalImageAnimation' => __tr($imageAnimation),
        ];
    }

    /**
     * Prepare User Dashboard Data for date filter
     *
     * @param   Array  $inputData
     *
     * @return  Array
     */
    public function prepareUserDashboardData($inputData)
    {
        //daterange for date filter
        $duration = $inputData['daterange'];
        $duration = explode(' - ', $duration);
        $startDate = $endDate = null;

        if(isset($duration[0]) and isset($duration[1])) {
            //start date for filter
            $startDate = Carbon::parse($duration[0] . ' 00:00:00');
            //end date for filter
            $endDate = Carbon::parse($duration[1] . ' 23:59:59');
        }

        // Calculate the difference between the two dates in days
        $daysDifference = $startDate->diffInDays($endDate);
        //check day difference is not exceeds 1 year
        if($daysDifference > 365) {
            return $this->engineReaction(2, ['show_message' => true], __tr('Duration exceeds more than 1 year.'));
        }
        //fetch cards count data according to date filter
        $userDashboardData = $this->getDashboardData($startDate, $endDate);

        return $this->engineReaction(1, $userDashboardData);
    }

}
