<?php

/**
 * PromptController.php - Controller file
 *
 * This file is part of the Prompt component.
 *-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Prompt\Controllers;

use Illuminate\Support\Arr;
use Illuminate\Validation\Rule;

use App\Yantrana\Base\BaseRequest;
use App\Yantrana\Base\BaseController;
use App\Yantrana\Components\Media\MediaEngine;
use App\Yantrana\Components\Prompt\PromptEngine;
use App\Yantrana\Support\CommonUnsecuredPostRequest;

class PromptController extends BaseController
{
    /**
     * @var  PromptEngine $promptEngine - Prompt Engine
     */
    protected $promptEngine;

    /**
     * @var  MediaEngine - Media Engine
     */
    protected $mediaEngine;

    /**
     * Constructor
     *
     * @param  PromptEngine $promptEngine - Prompt Engine
     * @param  MediaEngine  $mediaEngine - Media Engine
     *
     * @return  void
     *-----------------------------------------------------------------------*/
    public function __construct(PromptEngine $promptEngine, MediaEngine $mediaEngine)
    {
        $this->promptEngine = $promptEngine;
        $this->mediaEngine = $mediaEngine;
    }

    /**
     * Get Template Data
     *
     * @param   CommonUnsecuredPostRequest  $request
     * @param   String                      $templates
     *
     * @return json object
     */
    public function getTemplateData(CommonUnsecuredPostRequest $request, $category = null)
    {
        // If the setting is off then the page will not show.
        if(getAppSettings('features_availability_users_settings', 'use_templates_text_generation') == 0) {
            return redirect()->route('dashboard_page');
        }
        $getAllTemplatesData =  getPromptTemplates(null, null, [
            'with_total_count' => true
        ]);
        $getAllTemplates = $getAllTemplatesData['templates'];
        if($category) {
            // if category does not exit
            abortIf(!array_key_exists($category, $getAllTemplates));
        }
        return $this->loadPublicView('template.list', [
            'categoriesList' => $getAllTemplates,
            'selectedCategoryKey' => $category,
            'totalTemplateCount' => $getAllTemplatesData['totalTemplateCount'],
        ]);
    }

    /**
     * Get All Prompt Templates
     *
     * @return  Array
     */
    private function getAllPromptTemplates()
    {
        $allTemplates = [];

        foreach (getPromptTemplates() as $value) {
            $allTemplates = array_merge($allTemplates, $value['templates']);
        }

        return ['templates' => $allTemplates];
    }

    /**
     * Array Merge Option
     *
     * @return  mixed
     */
    private function arrayMergeOption()
    {
        $getPromptTemplates = getPromptTemplates();
        $getPromptTemplates = array_merge([
          'all' => ['title' => __tr('All')]
        ], $getPromptTemplates);

        return $getPromptTemplates;
    }


    /**
     * Text Generation Page View
     *
     * @param   String  $promptKey
     *
     * @return json object
     */
    public function textGenerationPageView($promptKey)
    {
        // If the setting is off then the page will not show.
        if(getAppSettings('features_availability_users_settings', 'use_templates_text_generation') == 0) {
            return redirect()->route('dashboard_page');
        }

        $templates = $this->promptEngine->prepareTemplate($promptKey);
        return $this->loadPublicView('template.text', ['promptTemplate' => $templates]);
    }

    /**
     * Text Generator View
     */
    public function show($contentType = 'text')
    {
        // If the setting is off then the page will not show.
        if(getAppSettings('features_availability_users_settings', "use_{$contentType}_generation") == 0) {
            return redirect()->route('dashboard_page');
        }
        return $this->loadPublicView("user.generator.$contentType");
    }

    /**
     * Image Generator View
     *
     */
    public function imageGeneratorView()
    {
        // If the setting is off then the page will not show.
        if(getAppSettings('features_availability_users_settings', 'use_image_generation') == 0) {
            return redirect()->route('dashboard_page');
        }
        return $this->loadPublicView('user.generator.image');
    }


    /**
     * Image Generator Process
     *
     * @param   CommonUnsecuredPostRequest  $request
     *
     * @return Array
     */
    public function imageGeneratorProcess(CommonUnsecuredPostRequest $request)
    {
        $request->validate([
          'query' => 'required|min:2',
        ]);

        $processReaction = $this->promptEngine->processAiImageGenerate($request->all());
        return $this->processResponse($processReaction, [], [], true);
    }

    /**
     * Speech Generator View
     *
     */
    public function speechGeneratorView()
    {
        // If the setting is off then the page will not show.
        if(getAppSettings('features_availability_users_settings', 'use_speech_generation') == 0) {
            return redirect()->route('dashboard_page');
        }
        return $this->loadPublicView('user.generator.speech');
    }


    /**
     * Image Generator Process
     *
     * @param   CommonUnsecuredPostRequest  $request
     *
     * @return Array
     */
    public function speechGeneratorProcess(CommonUnsecuredPostRequest $request)
    {
        $request->validate([
          'query' => 'required|min:2|max:4096',
          'speech_speed' => 'required|numeric|min:0.25|max:4.0',
        ]);

        $processReaction = $this->promptEngine->processAiSpeechGenerate($request->all());
        return $this->processResponse($processReaction, [], [], true);
    }

    /**
     * Image Animation view
     *
     */
    public function imageAnimationView()
    {
        // If the setting is off then the page will not show.
        if(getAppSettings('features_availability_users_settings', 'use_image_to_animation_conversion') == 0) {
            return redirect()->route('dashboard_page');
        }
        return $this->loadPublicView('user.generator.image-animation');
    }

    /**
     * Upload Source Animation Image
     *
     * @param   CommonUnsecuredPostRequest  $request
     *
     * @return  Array
     */
    public function uploadSourceAnimationImage(CommonUnsecuredPostRequest $request)
    {
        $basicRequirements = $this->promptEngine->checkBasicRequirements($request->all(), 'image-animation');
        // if failed to satisfy basic requirements
        if($basicRequirements->failed()) {
            return $this->processResponse($basicRequirements, [], [], true);
        }
        $processReaction = $this->mediaEngine->processUploadSourceAnimationImage($request->all(), 'image_animation');
        return $this->processResponse($processReaction, [], [], true);
    }

    /**
     * Process source image animation
     *
     * @param   CommonUnsecuredPostRequest  $request
     *
     * @return Array
     */
    public function processSourceAnimationImage(CommonUnsecuredPostRequest $request)
    {
        $request->validate([
          'temp_image_uploaded' => 'required|min:2',
          'temp_image_file_name' => 'required|min:2',
        ]);

        $processReaction = $this->promptEngine->processSourceAnimationImage($request->all());
        updateClientModels([
            'requestFailedOrComplete' => true
        ]);
        return $this->processResponse($processReaction, [], [
            'show_message' => true
        ], true);
    }


    /**
     * History View
     *
     */
    public function historyView($historyType)
    {
        return $this->loadPublicView('user.generator.history');
    }

    /**
     * Get User History List
     *
     * @return json object
     */
    public function getUserHistoryList($historyType)
    {
        return $this->promptEngine->prepareUserHistoryList($historyType);
    }

    /**
     * Manage Users History View
     */
    public function manageUsersHistoryView()
    {
        return $this->loadManageView('user.generator.all-users-history');
    }

    /**
     * Manage All Users History List
     *
     * @return json object
     */
    public function manageAllUsersHistoryList($historyType)
    {
        return $this->promptEngine->prepareAllUsersHistoryList($historyType);
    }

    /**
     * Speech To Text View
     *
     */
    public function speechToTextView()
    {
        // If the setting is off then the page will not show.
        if(getAppSettings('features_availability_users_settings')['use_speech_to_text_generation'] == 0) {
            return redirect()->route('dashboard_page');
        }

        return $this->loadPublicView('user.generator.speech-to-text');
    }

    /**
     * Upload Speech Audio
     *
     * @param   CommonUnsecuredPostRequest  $request
     *
     * @return  Array
     */
    public function uploadSpeechAudio(CommonUnsecuredPostRequest $request)
    {
        $processReaction = $this->mediaEngine->processUploadSpeechAudio($request->all(), 'speech_audio');

        if($processReaction['reaction_code'] == 2) {
            return $this->processResponse($processReaction, [], [], true);
        }

        $prepareText = $this->promptEngine->prepareTextFromAudio($processReaction['data']);

        return $this->processResponse($prepareText, [], [], true);
    }

    /**
     * Code Generation View
     */
    public function codeGenerationView()
    {
        // If the setting is off then the page will not show.
        if(getAppSettings('features_availability_users_settings')['use_code_generation'] == 0) {
            return redirect()->route('dashboard_page');
        }
        //code generation view
        return $this->loadPublicView('user.generator.code');
    }

    /**
     * Text Generation Process
     *
     * @param   CommonUnsecuredPostRequest  $request
     *
     * @return  Array
     */
    public function textGeneratorProcess(CommonUnsecuredPostRequest $request)
    {
        $validations = [
            // 'language' => 'required|min:2',
            'type' => [
                'required',
                Rule::in([
                        'text',
                        'code',
                        'translation',
                    ]),
                ],
            ];
        if($request->subtype == 'template') {
            $templateInputData = getPromptTemplates(null, $request->get('prompt_key') . '.input_fields');
            foreach ($templateInputData as $templateInputItemKey => $templateInputItemValue) {
                $validations[$templateInputItemKey] = Arr::get($templateInputItemValue, 'validations', ['required']);
            }
        } else {
            $validations['prompt'] = 'required|min:2';
        }
        $request->validate($validations);
        // send request to process
        $processReaction = $this->promptEngine->processTextGeneration($request->all());
        return $this->processResponse($processReaction, [], [], true);
    }

    /**
     * Process Delete History
     *
     * @param   mix  $historyUId
     *
     * @return  Array
     */
    public function processDeleteHistory($historyUId)
    {

        $processReaction = $this->promptEngine->processDeleteHistory($historyUId);

        return $this->responseAction(
            $this->processResponse($processReaction, [], [], true)
        );
    }

    /**
     * Process Delete History
     *
     * @param   mix  $recordUid
     *
     * @return  download file
     */
    public function downloadResultFile($recordUid, $filename = null)
    {
        return $this->promptEngine->processDownloadRequest($recordUid, $filename);
    }
}
