<?php

/**
 * PromptTemplateController.php - Controller file
 *
 * This file is part of the Prompt component.
 *-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Prompt\Controllers;

use App\Yantrana\Base\BaseRequest;
use App\Yantrana\Base\BaseController;

use App\Yantrana\Support\CommonUnsecuredPostRequest;
use App\Yantrana\Components\Prompt\PromptTemplateEngine;
use SebastianBergmann\Template\Template;

class PromptTemplateController extends BaseController
{
    /**
     * @var  PromptTemplateEngine $promptTemplateEngine - PromptTemplate Engine
     */
    protected $promptTemplateEngine;

    /**
     * Constructor
     *
     * @param  PromptTemplateEngine $promptTemplateEngine - PromptTemplate Engine
     *
     * @return  void
     *-----------------------------------------------------------------------*/
    public function __construct(PromptTemplateEngine $promptTemplateEngine)
    {
        $this->promptTemplateEngine = $promptTemplateEngine;
    }


    /**
     * Reset the template collection file
     *
     */
    public function resetToDefaultTemplates(CommonUnsecuredPostRequest $request)
    {
        return $this->processResponse(
            $this->promptTemplateEngine->resetToDefaultTemplatesProcess(
                $request->all()
            ),
            [],
            [],
            true
        );
    }

    /**
     * Merge with the template collection file
     *
     */
    public function mergeDefaultTemplates(CommonUnsecuredPostRequest $request)
    {
        return $this->processResponse(
            $this->promptTemplateEngine->mergeDefaultTemplatesProcess(
                $request->all()
            ),
            [],
            [],
            true
        );
    }

    /**
     * Download current templates collection file
     *
     */
    public function downloadCurrentTemplateFile(CommonUnsecuredPostRequest $request)
    {
        return $this->promptTemplateEngine->downloadCurrentTemplateFileProcess(
            $request->all()
        );
    }

    /**
     * Categories List View
     *
     */
    public function categoriesListView()
    {
        return $this->loadManageView('template.category');
    }

    /**
     * Create New Category
     *
     * @param   CommonUnsecuredPostRequest  $request
     *
     * @return  array
     */
    public function createNewCategory(CommonUnsecuredPostRequest $request)
    {
        $request->validate([
          'title' => 'required',
          'slug' => 'required|alpha_dash:ascii|lowercase',
          'icon' => 'required',
          'description' => 'required',
        ]);

        $processReaction = $this->promptTemplateEngine->writeToExistingFile($request->all());

        if ($processReaction['reaction_code'] === 1) {
            return $this->responseAction(
                $this->processResponse($processReaction, [], [], true),
                $this->replaceView('template.table', [], "#categoryList")
            );
        } else {
            return $this->responseAction(
                $this->processResponse($processReaction, [], [], true)
            );
        }
    }

    /**
     * Get Category Template
     *
     * @param   string  $category
     *
     */
    public function getCategoryTemplate($category)
    {
        return $this->loadManageView('template.template-list');
    }

    /**
     * Update Category
     *
     * @param   CommonUnsecuredPostRequest  $request
     *
     * @return  array
     */
    public function updateCategory(CommonUnsecuredPostRequest $request)
    {
        $request->validate([
          'slug' => 'required',
          'title' => 'required',
        //   'icon' => 'required',
          'description' => 'required',
        ]);

        $processReaction = $this->promptTemplateEngine->processUpdateCategory($request->all());

        if ($processReaction['reaction_code'] === 1) {
            return $this->responseAction(
                $this->processResponse($processReaction, [], [], true),
                $this->replaceView('template.table', [], "#categoryList")
            );
        } else {
            return $this->responseAction(
                $this->processResponse($processReaction, [], [], true)
            );
        }
    }

    /**
     * Delete Category
     *
     * @param   int  $categoryId
     *
     * @return json object
     */
    public function deleteCategory($categoryId)
    {
        $processReaction = $this->promptTemplateEngine->processDeleteCategory($categoryId);
        return $this->processResponse($processReaction, [], [], true);
    }

    /**
     * Create Template
     *
     * @param   CommonUnsecuredPostRequest  $request  [$request description]
     *
     * @return  array
     */
    public function createTemplate(CommonUnsecuredPostRequest $request)
    {
        $validationRules = [
          'title' => 'required',
          'slug' => 'required|alpha_dash:ascii|lowercase',
          'icon' => 'required',
          'description' => 'required',
        ];
        $inputFields = $request->get('input_fields');

        // validation messages
        $validationMessages = [];
        // validation attributes names
        $validationAttributeNames = [];

        if (!empty($inputFields)) {
            foreach ($inputFields as $inputFieldKey => $inputField) {
                if (isset($inputField['values'])) {
                    foreach ($inputField['values'] as $key => $value) {
                        if($key == 'validations') {
                            continue;
                        }
                        if($key == 'default') {
                            continue;
                        }
                        // value validations
                        $validationRules["input_fields.$inputFieldKey.values.$key.value"] = 'required';
                        $validationRules["input_fields.$inputFieldKey.values.$key.value.required"] = __tr('At least 1 value is required for this option.');
                        // custom option name validation attribute name
                        $validationAttributeNames["input_fields.$inputFieldKey.values.$key.value"] = $value['key'];
                    }
                }
            }
        }
        $request->validate($validationRules, $validationMessages, $validationAttributeNames);


        $processReaction = $this->promptTemplateEngine->processCreateNewTemplate($request->all());
        if ($processReaction['reaction_code'] === 1) {
            return $this->responseAction(
                $this->processResponse($processReaction, [], [], true),
                $this->replaceView('template.prompt-table', [], "#templatesList")
            );
        } else {
            return $this->responseAction(
                $this->processResponse($processReaction, [], [], true)
            );
        }
    }

    /**
     * Edit Template
     *
     * @param   string  $templateId
     * @param   string  $categoryId
     *
     * @return json object
     */
    public function editTemplate($templateId, $categoryId)
    {
        $processReaction = $this->promptTemplateEngine->prepareEditTemplateData($templateId, $categoryId);
        return $this->responseAction(
            $this->processResponse($processReaction, [], [], true)
        );
    }

    /**
     * Update Template
     *
     * @param   CommonUnsecuredPostRequest  $request  [$request description]
     *
     * @return  array
     */
    public function updateTemplate(CommonUnsecuredPostRequest $request)
    {
        $validationRules = [
          'templateId' => 'required',
          'categoryId' => 'required',
          'title' => 'required',
        //   'icon' => 'required',
          'description' => 'required',
        ];
        $inputFields = $request->get('input_fields');

        // validation messages
        $validationMessages = [];
        // validation attributes names
        $validationAttributeNames = [];

        if (!empty($inputFields)) {
            foreach ($inputFields as $inputFieldKey => $inputField) {
                if (isset($inputField['values'])) {
                    foreach ($inputField['values'] as $key => $value) {
                        if($key == 'validations') {
                            continue;
                        }
                        if($key == 'default') {
                            continue;
                        }
                        // value validations
                        $validationRules["input_fields.$inputFieldKey.values.$key.value"] = 'required';
                        $validationRules["input_fields.$inputFieldKey.values.$key.value.required"] = __tr('At least 1 value is required for this option.');
                        // custom option name validation attribute name
                        $validationAttributeNames["input_fields.$inputFieldKey.values.$key.value"] = $value['key'];
                    }
                }
            }
        }
        $request->validate($validationRules, $validationMessages, $validationAttributeNames);

        $processReaction = $this->promptTemplateEngine->updateTemplateData($request->all());

        if ($processReaction['reaction_code'] === 1) {
            return $this->responseAction(
                $this->processResponse($processReaction, [], [], true),
                $this->replaceView('template.prompt-table', [], "#templatesList")
            );
        } else {
            return $this->responseAction(
                $this->processResponse($processReaction, [], [], true)
            );
        }
    }

    /**
     * Delete Template
     *
     * @param   string  $categoryId
     * @param   string  $templateId
     *
     * @return json object
     */
    public function deleteTemplate($categoryId, $templateId)
    {
        $processReaction = $this->promptTemplateEngine->processDeleteTemplateData($categoryId, $templateId);
        return $this->processResponse($processReaction, [], [], true);
    }
}
