<?php

/**
 * PromptEngine.php - Main component file
 *
 * This file is part of the Prompt component.
 *-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Prompt;

use getID3;
use CURLFile;
use Exception;
use ErrorException;
use getID3_writetags;
use Illuminate\Support\Arr;
use Illuminate\Support\Str;
use App\Yantrana\Base\BaseEngine;
use Illuminate\Support\Facades\Http;
use Intervention\Image\Facades\Image;
use Illuminate\Support\Facades\Storage;
use Illuminate\Http\Client\RequestException;
use GrahamCampbell\Markdown\Facades\Markdown;
use App\Yantrana\Components\Media\MediaEngine;
use App\Yantrana\Components\User\CreditWalletEngine;
use App\Yantrana\Components\UserSetting\UserSettingEngine;
use App\Yantrana\Components\Prompt\Repositories\PromptRepository;
use App\Yantrana\Components\Prompt\Interfaces\PromptEngineInterface;

class PromptEngine extends BaseEngine implements PromptEngineInterface
{
    /**
     * @var  PromptRepository $promptRepository - Prompt Repository
     */
    protected $promptRepository;

    /**
     * @var  UserSettingEngine - UserSetting Engine
     */
    protected $userSettingEngine;

    /**
     * @var  MediaEngine - Media Engine
     */
    protected $mediaEngine;

    /**
     * @var  CreditWalletEngine - CreditWallet Engine
     */
    protected $creditWalletEngine;

    //Open Ai key
    protected $openAiKey;

    /**
     * Constructor
     *
     * @param  PromptRepository $promptRepository - Prompt Repository
     * @param  MediaEngine $mediaEngine - Media Engine
     * @param  CreditWalletEngine  $creditWalletEngine - CreditWallet Engine
     *
     * @return  void
     *-----------------------------------------------------------------------*/

    public function __construct(PromptRepository $promptRepository, UserSettingEngine $userSettingEngine, MediaEngine $mediaEngine, CreditWalletEngine $creditWalletEngine)
    {
        $this->promptRepository = $promptRepository;
        $this->userSettingEngine = $userSettingEngine;
        $this->mediaEngine = $mediaEngine;
        $this->openAiKey = getAppSettings('open_ai_keys');
        $this->creditWalletEngine = $creditWalletEngine;
    }

    /**
     * Prepare Template
     *
     * @param   String  $searchTeam
     *
     * @return  array
     */
    public function prepareTemplate($searchTeam)
    {
        //create array for template
        $templates = [];
        $isFound = false;
        //prepare template
        foreach (getPromptTemplates() as $value) {
            if(!isset($value['templates'])) {
                continue;
            }
            if (Arr::has($value['templates'], $searchTeam)) {
                $templates = Arr::get($value['templates'], $searchTeam);
                $isFound = true;
                break;
            }
        }
        abortIf(!$isFound);
        return $templates;
    }

    /**
     * Process Http Request
     *
     * @param   array  $parameters
     * @param   String  $apiEndPointUrl
     * @param   array  $headers
     *
     * @return  mix
     */
    public function processHttpRequest($parameters, $apiEndPointUrl, $headers = [])
    {
        if ($headers == null) {
            $headers = [
              "Content-Type" => "application/json",
              'Authorization' => "Bearer $this->openAiKey"
            ];
        }

        $httpProcessRequest = Http::withHeaders($headers)->post($apiEndPointUrl, $parameters);
        // Throw an exception if a client or server error occurred...
        $httpProcessRequest->throw();
        return $httpProcessRequest;
    }



    /**
     * Store Prompt Response
     *
     * @param   array  $response
     * @param   array  $inputData
     * @param   string  $preparedTemplate
     * @param   boolean  $useTemplate
     *
     * @return  object
     */
    public function storePromptResponse($response, $inputData, $preparedTemplate, $useTemplate)
    {
        //get words count
        $wordCount = $this->countWords(Arr::get($response, 'choices.0.message.content'));

        //process credits calculations and store result
        $creditWalletId = $this->processCreditsCalculation($inputData, 'text', $response);

        if ($creditWalletId->failed()) {
            return $this->engineFailedResponse($creditWalletId['data'], $creditWalletId['message']);
        }

        if ($useTemplate == true) {
            $prompt = $inputData['prompt_key'];
            $promptCategoryId = $this->getCategoryByPromptTemplates($prompt);
            $preparedTemplate = Arr::get($inputData, 'input_fields.' . Arr::get($inputData, 'prompt_input_field'), '');
        } else {
            $prompt = null;
            $promptCategoryId = null;
        }
        //store prompt response data
        $promptResponseData = [
          'prompt' => $preparedTemplate,
          'prompt_response' => Arr::get($response, 'choices.0.message.content'),
          'language_code' => $inputData['language'],
          'response_word_count' => $wordCount,
          'response_char_count' => null, //$response['usage']['prompt_tokens'],
          'type' => 'text',
          'prompt_template_id' => $prompt,
          'prompt_category_id' => $promptCategoryId,
          '__data' => [
            'raw_response_data' => $response
          ],
          'credit_wallet_transactions__id' => $creditWalletId->data('creditWalletId'),
          'users__id' => getUserID(),
        ];
        //check if result generated
        if (!$result = $this->promptRepository->storePromptResponse($promptResponseData)) {
            return $this->engineFailedResponse(['show_message' => true], __tr('Something went wrong.'));
        }
        return $result;
    }

    /**
     * Get Category By Prompt Templates
     *
     * @param   String  $template
     *
     * @return  String
     */
    public function getCategoryByPromptTemplates($template)
    {
        foreach (getPromptTemplates() as $key => $array) {
            if (isset($array['templates'][$template])) {
                return $key;
            }
        }
    }

    /**
     * Count Words
     *
     * @param   string  $text
     *
     * @return  int
     */
    public function countWords($text)
    {
        // str_word_count does not count words in other utf etc language like Marathi
        return count(explode(' ', $text));
    }

    /**
     * Prepare Text Generate Template
     *
     * @param   array  $inputData
     *
     * @return  String
     */
    public function prepareTextGenerateTemplate($inputData)
    {
        //get prompt template to generate text
        $templateData = getPromptTemplates(null, $inputData['prompt_key']);
        if (!__isEmpty($templateData)) {
            return strtr($templateData['prompt'], $inputData);
        }
        return "";
    }

    /**
     * Validate Fields
     * @param   object  $request
     */
    public function validateFields($request)
    {
        $template = $this->prepareTemplate($request->get('prompt_key'));
        $items = $template['input_fields'];

        $rules = $messages = [];

        foreach ($items as $key => $value) {

            $validationOption = "";
            foreach ($value['validations'] as $validation) {
                $validationOption .= $validation . '|';

                $rules['input_fields.' . $key] = $validationOption;
                $messages['input_fields.' . $key . '.' . $validation] = 'The ' . $value['title'] . ' field is required.';
            }
        }

        $request->validate($rules, $messages);
    }


    /**
     * Process Ai Image Generate
     *
     * @param   array  $inputData
     *
     * @return  array
     */
    public function processAiImageGenerate($inputData)
    {
        // check if allow_stable_diffusion_ai enabled
        if (getAppSettings('allow_stable_diffusion_ai')
            and getAppSettings('stable_diffusion_key')
            and array_key_exists(getAppSettings('ai_image_generation_model'), configItem('stable_diffusion_ai.models.text_to_image_generation'))) {
            return $this->stableDiffusionImageGeneration($inputData);
        }

        // check if openai enabled
        if (getAppSettings('enable_open_ai')
             and $this->openAiKey
             and array_key_exists(getAppSettings('ai_image_generation_model'), configItem('open_ai.models.dall_e'))) {
            return $this->openAiImageGeneration($inputData);
        }

        return $this->engineFailedResponse(['show_message' => true], __tr('AI Image Generation provider not available.'));
    }

    /**
     * Open Ai Image Generation
     *
     * @param   array  $inputData
     *
     * @return  array
     */
    public function openAiImageGeneration($inputData)
    {
        //progress text load while generating image
        updateProgressTextModel(
            __tr('preparing request ...')
        );
        $prompt = $inputData['query'] . ' Image should be in ' . $inputData['style_preset'] . " style.";
        // check if system and user fulfils basic requirements
        $basicRequirements = $this->checkBasicRequirements($inputData, 'image-open-ai');
        // if failed to satisfy basic requirements
        if($basicRequirements->failed()) {
            updateClientModels([
                'showImageLoader' => false,
                'aiGeneratedResponse' => [],
              ]);
            return $basicRequirements;
        }
        //progress text load while generating image
        updateProgressTextModel(
            __tr('processing request ...')
        );
        // response from image generator
        $apiEndPointUrl = 'https://api.openai.com/v1/images/generations';

        $parameters = [
          "prompt" => $prompt,
          "n" => (int)$inputData['no_of_images'],
          "size" => $inputData['open_ai_image_size']
        ];
        // uncomment
        $res = $this->processHttpRequest($parameters, $apiEndPointUrl);

        // response decode
        $response = json_decode($res->getBody()->getContents(), true);

        if (isset($response['error']['message'])) {
            //exceptional error message
            updateClientModels([
              'aiGeneratedResponseError' => $response['error']['message'],
              'aiGeneratedResponse' => [],
            ]);

            return $this->engineFailedResponse([], $response['error']['message']);
        }
        //progress text load while generating image
        updateProgressTextModel(
            __tr('image preparation ...')
        );
        //download images
        $images = $this->mediaEngine->downloadAndStoreFile($response['data']);
        $response = [];
        //store responses
        $storeImageResponse = $this->storeImageGenerationResponse($response, $inputData, $images, 'image-open-ai');
        if ($storeImageResponse['reaction_code'] == 2) {
            return $this->engineFailedResponse($storeImageResponse['data'], $storeImageResponse['message']);
        }
        $modifiedData = $this->getLatestTextGeneratedResponses($storeImageResponse);
        // update client info
        updateClientModels([
          'aiGeneratedResponse' => $storeImageResponse->toArray(),
          'aiGeneratedResponseError' => '',
        ]);
        totalUserCredits();
        return $this->engineSuccessResponse([
          'response' => $modifiedData,
        ]);
    }

    /**
     * Open Ai Speech Generation
     *
     * @param   array  $inputData
     *
     * @return  array
     */
    public function processAiSpeechGenerate($inputData)
    {
        //progress text load while generating speech
        updateProgressTextModel(
            __tr('preparing request ...')
        );
        $prompt = $inputData['query'];
        // check if system and user fulfils basic requirements
        $basicRequirements = $this->checkBasicRequirements($inputData, 'text-to-speech');
        // if failed to satisfy basic requirements
        if($basicRequirements->failed()) {
            updateClientModels([
                'showSpeechLoader' => false,
                'aiGeneratedResponse' => [],
              ]);
            return $basicRequirements;
        }
        //progress text load while generating speech
        updateProgressTextModel(
            __tr('processing request ...')
        );
        // https://platform.openai.com/docs/api-reference/audio/createSpeech
        // response from image generator
        $apiEndPointUrl = 'https://api.openai.com/v1/audio/speech';
        $parameters = [
          "input" => $prompt,
          "model" => getAppSettings('open_ai_text_to_speech_generation_model'),
          "voice" => $inputData['speech_voice'],
          "speed" => $inputData['speech_speed'],
        ];
        // uncomment
        $res = $this->processHttpRequest($parameters, $apiEndPointUrl);

        updateProgressTextModel(
            __tr('speech preparation ...')
        );
        //download images
        $speechFile = $this->mediaEngine->downloadAndStoreAudioFile($res);
        //store responses
        $storeSpeechResponse = $this->storeSpeechResponse($inputData, $speechFile, 'image-open-ai');

        if ($storeSpeechResponse->failed()) {
            return $this->engineFailedResponse($storeSpeechResponse['data'], $storeSpeechResponse['message']);
        }
        // update client info
        updateClientModels([
          'aiGeneratedResponseError' => '',
        ]);
        totalUserCredits();
        return $this->engineSuccessResponse([
          'audioFileUrl' => $speechFile->data('path'),
          'recordUid' => $storeSpeechResponse->data('promptStoredResponse._uid'),
        ]);
    }

    /**
     * Store Image Generation Response
     *
     * @param   mixed  $response
     * @param   array  $inputData
     * @param   array  $images
     *
     * @return  json object|array
     */
    public function storeImageGenerationResponse($response, $inputData, $images, $type)
    {

        $resultString = '';

        foreach ($images as $key => $image) {
            $resultString .= $image;

            if ($key < count($images) - 1) {
                $resultString .= ', ';
            }
        }
        //check if type is image-open-ai
        if($type == 'image-open-ai') {
            //process credits calculations and store result
            $creditWalletId = $this->processCreditsCalculation($inputData, 'image-open-ai');
        } else {
            //process credits calculations and store result
            $creditWalletId = $this->processCreditsCalculation($inputData, 'image-stable-diffusion');
        }

        if (is_array($creditWalletId) && $creditWalletId['reaction_code'] == 2) {
            return $this->engineFailedResponse($creditWalletId['data'], $creditWalletId['message']);
        }


        $data = [
          'prompt' => $inputData['query'],
          'prompt_response' => $resultString,
          'type' => 'image',
          '__data' => json_encode($response),
          'credit_wallet_transactions__id' => $creditWalletId->data('creditWalletId'),
          'users__id' => getUserID(),
        ];


        if (!$data = $this->promptRepository->storePromptResponse($data)) {
            return $this->engineFailedResponse(['show_message' => true], __tr('Something went wrong.'));
        }
        return $data;
    }

    /**
     * Store Speech Generation Response
     *
     * @param   array  $inputData
     * @param   EngineResponse  $speechFile
     *
     * @return EngineResponse
     */
    public function storeSpeechResponse($inputData, $speechFile, $type)
    {
        //process credits calculations and store result
        $creditWalletId = $this->processCreditsCalculation($inputData, 'text-to-speech');

        if ($creditWalletId->failed()) {
            return $this->engineFailedResponse($creditWalletId->data(), $creditWalletId->message());
        }

        $data = [
          'prompt' => $inputData['query'],
          'prompt_response' => $speechFile->data('fileName'),
          'type' => 'text-to-speech',
        //   '__data' => json_encode($response),
          'credit_wallet_transactions__id' => $creditWalletId->data('creditWalletId'),
          'users__id' => getUserID(),
        ];

        if (!$promptStoredResponse = $this->promptRepository->storePromptResponse($data)) {
            return $this->engineFailedResponse(['show_message' => true], __tr('Something went wrong.'));
        }
        return $this->engineSuccessResponse([
            'promptStoredResponse' => $promptStoredResponse
        ]);
    }

    /**
     * Process Credits Calculation And Store Results
     *
     * @param   array   $inputData
     * @param   String  $type
     * @param   array  $response
     *
     * @return  array||int
     */
    public function processCreditsCalculation($inputData, $type, $responseText = null)
    {
        $creditType = null;
        $tokenCalculation = null;
        if($type == 'image-animation') {
            $tokenCalculation = getAppSettings('image_animation_pricing');
            $creditType = 8;
        } elseif ($type == 'image-stable-diffusion') {
            //image generation model
            $imageGenerationModel = getAppSettings('ai_image_generation_model');
            //image generation model size
            $size = configItem("stable_diffusion_ai.models.text_to_image_generation.$imageGenerationModel.size");
            //token calculation on number of images
            $tokenCalculation = $inputData['no_of_images'] * getAppSettings('image_pricing')[$size . 'x' . $size];
            //credit type
            $creditType = 3;
        } elseif($type == "image-open-ai") {
            //token calculation on no. of images
            $tokenCalculation = $inputData['no_of_images'] * getAppSettings('image_pricing')[$inputData['open_ai_image_size']];
            //credit type
            $creditType = 3;
        }
        if(in_array($type, [
            'text',
            'code',
            'translation',
            'text-to-speech'
        ])) {
            if(in_array($type, [
                'text-to-speech'
            ])) {
                //no. of words in query
                $wordCount = $this->countWords($inputData['query']);
                //token calculation on words
                $tokenCalculation = getAppSettings('text_to_speech_pricing') * $wordCount;
            } else {
                //no. of words in response
                $wordCount = $this->countWords($responseText);
                //token calculation on words
                $tokenCalculation = getAppSettings('per_word_pricing') * $wordCount;
            }
        }

        if($type == "text") {
            //credit type
            $creditType = 2;
        } elseif($type == "speech-to-text") {
            //credit type
            $creditType = 5;
            $tokenCalculation = getAppSettings('speech_to_text_pricing') * $inputData['audio_duration'];
        } elseif($type == 'code') {
            //credit type
            $creditType = 4;
        } elseif($type == 'translation') {
            //credit type
            $creditType = 7;
        } elseif($type == 'image-animation') {
            //credit type
            $creditType = 8;
            $tokenCalculation = getAppSettings('image_animation_pricing');
        } elseif($type == "text-to-speech") {
            //credit type
            $creditType = 11;
        }
        //store credits
        if (!$creditWalletId = $this->creditWalletEngine->processDebitCredits($tokenCalculation, $creditType)) {
            //error response
            return $this->engineFailedResponse(['show_message' => true], __tr('Failed to charge the transaction.'));
        }

        return $this->engineSuccessResponse([
            'creditWalletId' => $creditWalletId
        ]);
    }
    /**
         * get duration of audio
         *
         * @param   array  $audioFilePath
         * @param   EngineResponse  $speechFile
         *
         *
         */
    public function getAudioDuration($audioFilePath)
    {
        $getID3 = new getID3();
        //get audio file info
        $fileInfo = $getID3->analyze($audioFilePath);
        if(Arr::get($fileInfo, 'error')) {
            return null;
        }
        //check audio time in seconds
        if (isset($fileInfo['playtime_seconds'])) {
            return $durationInSeconds = $fileInfo['playtime_seconds'];
        }

        return null; // Error or unsupported audio file
    }

    /**
     * get Previous History list.
     *
     * @return object
     *---------------------------------------------------------------- */
    public function prepareUserHistoryList($historyType)
    {
        //get history collection data
        $historyCollection = $this->promptRepository->fetchUserHistoryListDataTable($historyType);
        //get prompt language data
        $promptLanguages = getPromptLanguages();
        //get programming  language list data
        $programmingLanguages = configItem('programming_languages_list');
        $packageImageFolderPath = getPathByKey('image_animation', ['{_uid}' => getUserUID()]);

        $requireColumns = [
          '_id',
          '_uid',
          'created_at' => function ($historyData) {
              return formatDate($historyData['created_at'], 'l jS F Y h:i:s A');
          },
          'prompt' => function ($data) use ($packageImageFolderPath) {
              if ($data['type'] == 'image-animation') {
                  return getMediaUrl($packageImageFolderPath, $data['prompt']);
              } elseif (isset($data['prompt_template_id'])) {
                  $inputFieldsData = Arr::get($data, '__data.input_data', []);
                  $neInputDataFields = Arr::where($inputFieldsData, function ($value, string $key) {
                      if(str_starts_with($key, '__')) {
                          return $value;
                      }
                  });
                  return implode(', ', $neInputDataFields);
              } elseif(__isEmpty($data['prompt'])) {
                  return __tr("No prompt available");
              }
              return $data['prompt'];
          },
          'prompt_response' => function ($historyData) {
              if ($historyData['type'] == 'image') {
                  $generatedImages = [];
                  $packageImageFolderPath = getPathByKey('generated_photos', ['{_uid}' => getUserUID()]);
                  $imagesName = explode(', ', $historyData['prompt_response']);
                  foreach ($imagesName as $key => $image) {
                      $generatedImages[] = [
                          'filename' => $image,
                          'url' => getMediaUrl($packageImageFolderPath, $image),
                      ];
                  }
                  return $generatedImages;
              } elseif ($historyData['type'] == 'code') {
                  return $this->formattingResponse($historyData['prompt_response']);
              } else {
                  return $historyData['prompt_response'];
              }
          },
          'language_code' => function ($historyData) use (&$promptLanguages, &$programmingLanguages) {
              if ($historyData['type'] == 'code') {
                  return Arr::get($programmingLanguages, $historyData['language_code']);
              } else {
                  return Arr::get($promptLanguages, $historyData['language_code']);
              }
          },
          'response_word_count',
          'response_char_count',
          'type' => function ($historyData) {
              return $historyData['type'];
          },
          'prompt_template_id',
          'prompt_category_id',
          'template' => function ($historyData) {
              if(isset($historyData['prompt_template_id'])) {
                  $templateData = getPromptTemplates(null, $historyData['prompt_template_id']);
                  return Arr::get($templateData, 'title');
              }
              return __tr('N/A');
          },
        ];

        return $this->dataTableResponse($historyCollection, $requireColumns);
    }

    /**
     * Prepare All Users History List
     *
     * @return  object
     */
    public function prepareAllUsersHistoryList($historyType)
    {
        //get history list data
        $historyCollection = $this->promptRepository->fetchAllUsersHistoryListDataTable($historyType);
        //get prompt language data
        $promptLanguages = getPromptLanguages();
        //get programming  language list data
        $programmingLanguages = configItem('programming_languages_list');

        $requireColumns = [
          '_id',
          '_uid',
          'created_at' => function ($historyData) {
              return formatDate($historyData['created_at'], 'l jS F Y h:i:s A');
          },
          //prompt of image-animation
          'prompt' => function ($data) {
             $userUid =  Arr::get($data, 'user._uid');
              if ($data['type'] == 'image-animation') {
                  $packageImageFolderPath = getPathByKey('image_animation', ['{_uid}' => Arr::get($data, 'user._uid')]);
                  return getMediaUrl($packageImageFolderPath, $data['prompt']);
              } else if($data['type'] == 'speech-to-text') {
                return getMediaUrl(
                        getPathByKey('speech_audio', [
                            '{_uid}' => $userUid
                        ]),
                    $data['prompt']
                );
              } elseif (isset($data['prompt_template_id'])) {
                  $inputFieldsData = Arr::get($data, '__data.input_data', []);
                  $neInputDataFields = Arr::where($inputFieldsData, function ($value, $key) {
                      if(str_starts_with($key, '__')) {
                          return $value;
                      }
                  });
                  return implode(', ', $neInputDataFields);
              } elseif(__isEmpty($data['prompt'])) {
                  return __tr("No prompt available");
              }
              return $data['prompt'];
          },
          //prompt_response of image generation
          'prompt_response' => function ($historyData) {
              if ($historyData['type'] == 'image') {
                  $generatedImages = [];
                  $packageImageFolderPath = getPathByKey('generated_photos', ['{_uid}' => $historyData['userUid']]);
                  $imagesName = explode(', ', $historyData['prompt_response']);
                  foreach ($imagesName as $key => $image) {
                      $generatedImages[] = [
                          'filename' => $image,
                          'url' => getMediaUrl($packageImageFolderPath, $image),
                      ];
                  }
                  return $generatedImages;
              } elseif ($historyData['type'] == 'text-to-speech') {
                return getMediaUrl(
                        getPathByKey('generated_speech', [
                            '{_uid}' => $historyData['userUid']
                        ]),
                    $historyData['prompt_response']
                );
              } elseif ($historyData['type'] == 'image-animation') {
                return getMediaUrl(
                        getPathByKey('image_animation', [
                            '{_uid}' => $historyData['userUid']
                        ]),
                    $historyData['prompt_response']
                );
            } elseif ($historyData['type'] == 'code') {    //data of code generation
                  return $this->formattingResponse($historyData['prompt_response']);
              } else {
                  return $historyData['prompt_response'];
              }
          },
          //data of other type
          'type' => function ($historyData) {
              return $historyData['type'];
          },
          'full_name',
          'user',
          'template' => function ($historyData) {
              if(isset($historyData['prompt_template_id'])) {
                  $templateData = getPromptTemplates(null, $historyData['prompt_template_id']);
                  return Arr::get($templateData, 'title');
              }
              return "N/A";
          },
           //data of code generation
            'language_code' => function ($historyData) use (&$promptLanguages, &$programmingLanguages) {
                if ($historyData['type'] == 'code') {
                    return Arr::get($programmingLanguages, $historyData['language_code']);
                } else {
                    return Arr::get($promptLanguages, $historyData['language_code']);
                }
            },
        ];

        return $this->dataTableResponse($historyCollection, $requireColumns);
    }

    /**
     * Stable Diffusion Image Generation
     *
     * @param   array  $inputData
     *
     * @return  array
     */
    public function stableDiffusionImageGeneration($inputData)
    {
        updateProgressTextModel(
            __tr('preparing request ...')
        );
        //Stable Diffusion api keys
        $apiKeys = getAppSettings('stable_diffusion_key');
        //Stable Diffusion Image Generation Model
        $imageGenerationModel = getAppSettings('ai_image_generation_model');
        //Stable Diffusion Image Generation size
        $size = configItem("stable_diffusion_ai.models.text_to_image_generation.$imageGenerationModel.size");
        // check if system and user fulfils basic requirements
        $basicRequirements = $this->checkBasicRequirements($inputData, 'image-stable-diffusion');
        // if failed to satisfy basic requirements
        if($basicRequirements->failed()) {
            updateClientModels([
                'showImageLoader' => false,
                'aiGeneratedResponse' => [],
              ]);
            return $basicRequirements;
        }
        //check if credits are enough to proceed this request
        updateProgressTextModel(
            __tr('processing request ...')
        );

        $apiEndPointUrl = "https://api.stability.ai/v1/generation/$imageGenerationModel/text-to-image";
        //
        $headers = [
          "Content-Type" => "application/json;jpeg",
          "Authorization" => "Bearer $apiKeys"
        ];

        $body = [
          "text_prompts" => [
                [
                    "text" => $inputData['query'],
                    "weight" => 1
                ]
            ],
          "cfg_scale" => getAppSettings('stable_diffusion_cfg_scale'),
          "height" => (int)$size,
          "width" => (int)$size,
          "samples" => (int)$inputData['no_of_images'],
          "steps" => getAppSettings('stable_diffusion_steps'),
          "style_preset" => $inputData['style_preset'],
        ];

        $res = $this->processHttpRequest($body, $apiEndPointUrl, $headers);

        $response = json_decode($res->getBody()->getContents(), true);

        if ((isset($response['message']) and !__isEmpty($response['message'])) or !isset($response['artifacts'])) {
            //error response
            return $this->engineFailedResponse(['show_message' => true], $response['message']);
        }

        $images = $imagesName = [];

        foreach ($response['artifacts'] as $key => $value) {
            $imageResponse = $this->downloadImage($value['base64']);
            $images['url'][] = $imageResponse['url'];
            $images['generatedImagesName'][] = $imageResponse['imageName'];
            $imagesName[] = $imageResponse['imageName'];
        }
        //store responses
        $storeImageResponse = $this->storeImageGenerationResponse($response, $inputData, $images['generatedImagesName'], 'image-stable-diffusion');
        //error response
        if ($storeImageResponse['reaction_code'] == 2) {
            return $this->engineFailedResponse($storeImageResponse['data'], $storeImageResponse['message']);
        }
        $modifiedData = $this->getLatestTextGeneratedResponses($storeImageResponse);
        updateClientModels([
            'aiGeneratedResponse' =>  $modifiedData,
            'aiGeneratedResponseError' => '',
          ]);

        totalUserCredits();
        return $this->engineSuccessResponse([
            'response' =>  $modifiedData,
        ]);
    }

    /**
     * Stable Diffusion Download Image
     *
     * @param   string  $base64Data
     *
     * @return  array
     */
    public function downloadImage($base64Data)
    {
        // Determine the image format (e.g., JPEG, PNG)
        $imageFormat = configItem('ai_generate_image_type');

        // Set the appropriate data URL prefix based on the image format
        $dataUrlPrefix = 'data:image/' . $imageFormat . ';base64,';

        // Specify the directory where you want to save the image
        $savePath = getPathByKey('generated_photos', ['{_uid}' => authUID()]);

        if (!file_exists($savePath)) {
            mkdir($savePath, 0755, true);
        }
        // store the image
        $images = $this->mediaEngine->downloadAndStoreFile(['dummy_value'], file_get_contents($dataUrlPrefix . $base64Data));

        $data = [
          'imageName' => $images[0],
          'url' => getMediaUrl($savePath, $images[0]),
        //   'url' => $path,
        ];

        return $data;
    }

    /**
     * Prepare Text From Audio
     *
     * @param   array  $inputData
     *
     * @return  array
     */
    public function prepareTextFromAudio($inputData)
    {
        //check open_ai_speech_to_text_generation_model is checked
        $textFromAudioGenerationModel = getAppSettings('open_ai_speech_to_text_generation_model');
        if (!getAppSettings('enable_open_ai') || __isEmpty($this->openAiKey) || __isEmpty($textFromAudioGenerationModel)) {
            return $this->engineFailedResponse(
                [
                    'show_message' => true
                ],
                __tr('Speech to Text Generator is not available.')
            );
        }
        try {
            // get path to store file
            $speechAudioFolderPath = getPathByKey('speech_audio', ['{_uid}' => authUID()]);
            // move uploaded file
            $uploadedMedia = $this->mediaEngine->processMoveFile($speechAudioFolderPath, $inputData['fileName'], [], ['createLocalCopy' => true]);
            // get the real path of file
            $localPath = $uploadedMedia->data('localPath');
            // get the url of file
            $savedFileUrl = $uploadedMedia->data('path');
            // file name
            $savedFileName = $uploadedMedia->data('fileName');
            // determine audio duration
            $inputData['audio_duration'] = $this->getAudioDuration($localPath);
            // delete local file
            $this->mediaEngine->deleteLocalFile($uploadedMedia->data('realFullPath'));
            // if failed it must not be valid file
            if(!$inputData['audio_duration']) {
                return $this->engineFailedResponse(['show_message' => true], __tr('Failed to determine audio file'));
            }
            // check if system and user fulfils basic requirements
            $basicRequirements = $this->checkBasicRequirements($inputData, 'speech-to-text');
            // if failed to satisfy basic requirements
            if($basicRequirements->failed()) {
                return $basicRequirements;
            }
            // Make the HTTP POST request using Laravel's HTTP client
            $res = Http::withHeaders([
              'Authorization' => 'Bearer ' . $this->openAiKey,
            ])
              ->attach('file', file_get_contents($savedFileUrl), $savedFileName)
              ->post('https://api.openai.com/v1/audio/transcriptions', [
                'model' => $textFromAudioGenerationModel,
              ]);
            // Throw an exception if a client or server error occurred...
            $res->throw();
            $response = json_decode($res->getBody()->getContents(), true);
            // if there is error
            if (isset($response['error']['message'])) {
                return $this->engineFailedResponse(['show_message' => true], $response['error']['message']);
            }
            // assign audio duration
            $response['audio_duration'] = $inputData['audio_duration'];
            // store record into database
            $this->storeSpeechToTextResponse($response, $uploadedMedia->data('fileName'));
            // get and update the credits
            totalUserCredits();
            // Handle the response data here
            return $this->engineSuccessResponse([
                'response' => $response['text'],
                'creditsRemaining' => totalUserCredits(),
              ], __tr('Text has been generated successfully from your audio file.'));

        } catch (\Exception $e) {
            return $this->engineFailedResponse(['show_message' => true], __tr('Failed to process audio file.'));
        }
        return $this->engineFailedResponse(['show_message' => true], __tr('Failed to process audio file.'));
    }

    /**
     * Store Speech To Text Response
     *
     * @param   array  $response
     *
     * @return  void|array
     */
    public function storeSpeechToTextResponse($response, $animationVideoFileName)
    {
        //count no of words in the response
        $wordCount = $this->countWords(Arr::get($response, 'text'));

        //process credits calculations and store result
        $creditWalletId = $this->processCreditsCalculation($response, 'speech-to-text', $response['text']);

        if (is_array($creditWalletId) && $creditWalletId['reaction_code'] == 2) {
            return $this->engineFailedResponse($creditWalletId['data'], $creditWalletId['message']);
        }

        $promptResponseData = [
          'prompt' => $animationVideoFileName,
          'prompt_response' => Arr::get($response, 'text'),
          'response_word_count' => $wordCount,
        //   'response_char_count' => strlen(str_replace(" ", "", Arr::get($response, 'text'))),
          'response_char_count' => strlen(Arr::get($response, 'text')),
          'type' => 'speech-to-text',
          '__data' => json_encode($response),
          'credit_wallet_transactions__id' => $creditWalletId->data('creditWalletId'),
          'users__id' => getUserID(),
        ];

        if (!$this->promptRepository->storePromptResponse($promptResponseData)) {
            return $this->engineFailedResponse(['show_message' => true], __tr('Something went wrong.'));
        }
    }

    public function checkBasicRequirements($inputData, $type = 'text')
    {
        //check if credits are enough to proceed this request
        $isCreditsEnough = $this->isCreditsEnough($inputData, $type);
        if ($isCreditsEnough->failed()) {
            return $this->engineFailedResponse($isCreditsEnough->data(), $isCreditsEnough->message());
        }

        return $this->engineSuccessResponse(array_merge(
            $isCreditsEnough->data()
        ));
    }

    /**
     * Process Ai Code Generate
     *
     * @param   array  $inputData
     *
     * @return array
     */
    public function processTextGeneration($inputData)
    {
        if (!getAppSettings('enable_open_ai')) {
            return $this->engineFailedResponse(
                [
                    'show_message' => true
                ],
                __tr('AI Text Generator is not available')
            );
        }
        // check if system and user fulfils basic requirements
        $basicRequirements = $this->checkBasicRequirements($inputData);

        // if failed to satisfy basic requirements
        if($basicRequirements->failed()) {
            return $basicRequirements;
        }
        $contentRequestType = $inputData['type'];
        $systemPrompt = "";
        $userPrompt = '';
        $useTemplate = false;
        /*
        Programming Code Generation
        */

        if($contentRequestType == 'code') {
            $userPrompt = $inputData['prompt'];
            //store programming language
            $this->userSettingEngine->processUserSettingStore('ai_provider', [
                'programming_language' => $inputData['language'],
            ]);
            // system prompt
            $systemPrompt = "You are expertise in " . $inputData['language'] . ' programming language. Generate code in markdown language for user requested subject. Do not give any additional instructions or information. Include comments only if required for code.';
        } elseif($contentRequestType == 'text') {
            $this->userSettingEngine->processUserSettingStore('ai_provider', [
                'prompt_response_language' => getPromptLanguages($inputData['language']),
              ]);
            $useTemplate = Arr::get($inputData, 'subtype') === 'template';
            if ($useTemplate == true) {
                $userPrompt = $this->prepareTextGenerateTemplate($inputData);
                $inputData['prompt'] = $userPrompt;
                $systemPrompt = "You are Expert to answer given topic. Your task is to generate content in markdown on user suggested topic in " . getPromptLanguages($inputData['language']) . " language.";
            } else {
                $userPrompt = $inputData['prompt'];
                $systemPrompt = "You need to generate well formatted the content in markdown on user suggested topic in " . getPromptLanguages($inputData['language']) . " language.";
            }
        } elseif($contentRequestType == 'translation') {
            $inputData['destination_language'] = $inputData['language'];
            $userPrompt = $inputData['prompt'];
            $this->userSettingEngine->processUserSettingStore('ai_provider', [
                'prompt_response_destination_language' => getPromptLanguages($inputData['destination_language']),
              ]);
            // store source language
            $this->userSettingEngine->processUserSettingStore('ai_provider', [
                'prompt_response_source_language' => ($inputData['source_language'] ? getPromptLanguages($inputData['source_language']) : ''),
              ]);
            if($inputData['source_language']) {
                $systemPrompt = "You will be provided with a sentence in " . getPromptLanguages($inputData['source_language']) . " language, and ";
            }

            $systemPrompt .= "your task is to TRANSLATE given sentence into " . getPromptLanguages($inputData['destination_language']) . " language ONLY.";
            if(isset($inputData['pronunciation']) and $inputData['pronunciation']) {
                $systemPrompt .= " Generate pronunciation for translated sentence.";
            }
        }
        // send the request
        $prepareAndPostProviderRequestResponse = $this->prepareAndPostProviderRequest($systemPrompt, $userPrompt, [
            'maxTokenUses' => $basicRequirements->data('maxTokenUses'),
            // live stream based response
            'stream' => true
        ]);
        // check if its failed
        if(!$prepareAndPostProviderRequestResponse->success()) {
            return $prepareAndPostProviderRequestResponse;
        }
        // get the response content
        $rawText = $prepareAndPostProviderRequestResponse->data('responseContent');
        // client info update
        updateProgressTextModel(
            __tr('almost done ...')
        );
        $formattedResponse = $rawText;
        // client info update
        updateProgressTextModel(
            __tr('formatting completed')
        );
        $storedGenerationResponse = $this->storeGenerationTransaction($rawText, $inputData);
        if(!$storedGenerationResponse->success()) {
            return $storedGenerationResponse;
        }
        $totalUserCredits =  totalUserCredits();
        // response
        return $this->engineSuccessResponse([
          'codeResponse' => [],
        //   'codeResponse' => $this->getLatestTextGeneratedResponses($codeResponse),
          'responseTextData' =>  $formattedResponse,
          'language' => $inputData['language'],
          'creditsRemaining' => $totalUserCredits
        ]);
    }

    /**
     * Prepare and post the request to provider like open ai etc
     *
     * @param string $systemPrompt
     * @param string $userPrompt
     * @param array $options
     * @return EngineResponse
     */
    public function prepareAndPostProviderRequest($systemPrompt, $userPrompt, $options = [])
    {
        // default options
        $options = array_merge([
            'stream' => false,
            'maxTokenUses' => null,
        ], $options);
        // request headers
        $headers = [
            "Content-Type" => "application/json",
            'Authorization' => "Bearer $this->openAiKey"
          ];
        //code generation model
        $textGenerationModel = getAppSettings('open_ai_text_generation_model');
        //get items
        $getItem = Arr::get(configItem("open_ai.models.text_generation"), $textGenerationModel);
        $responseFullText = '';
        try {
            //response from chat GPT
            $parameters = [
              'messages' => [
                [
                    'role' => 'system',
                    'content' => $systemPrompt,
                ],
                  [
                      'role' => 'user',
                      'content' => $userPrompt,
                  ]
              ],
              'model' => $textGenerationModel,
              // 'max_tokens' => (int)$inputData['max_code_token'],
              'stream' => $options['stream'],
            ];

            if($options['maxTokenUses']) {
                $parameters['max_tokens'] = $options['maxTokenUses'];
            }
            // client info update
            updateProgressTextModel(
                __tr('Processing request ...')
            );
            $providerResponse = Http::withOptions([
                // 'debug' => config('app.debug'), // keep it off on debug mode the unexpected data may produced which may create problem for standard formatted data require for our js libs
                'stream' => $options['stream'],
            ])->withHeaders($headers)->post($getItem['endPoint'], $parameters);
            // Throw an exception if a client or server error occurred...
            $providerResponse->throw();
            // if streamed data
            // ref: https://github.com/openai-php/client/blob/main/src/Responses/StreamResponse.php
            if($options['stream']) {
                while (!$providerResponse->getBody()->eof()) {
                    $line = $this->readLine($providerResponse->getBody());
                    if (!str_starts_with($line, 'data:')) {
                        continue;
                    }
                    $streamData = trim(substr($line, strlen('data:')));
                    if ($streamData === '[DONE]') {
                        break;
                    }
                    /** @var array{error?: array{message: string|array<int, string>, type: string, code: string}} $response */
                    $streamResponse = json_decode($streamData, true, 512, JSON_THROW_ON_ERROR);
                    if (isset($streamResponse['error'])) {
                        throw new ErrorException(json_encode($streamResponse['error']));
                    }
                    // collect data
                    $responseFullText .= Arr::get($streamResponse, 'choices.0.delta.content');
                    // send data to the browser
                    dispatchStreamEventData('updateAiGenerationResponseText', [
                        'responseData' => Arr::get($streamResponse, 'choices.0.delta.content')
                    ]);
                }
            }
            // if streamed data ends
            //response decode
            $response = json_decode($providerResponse->getBody()->getContents(), true);
            if (isset($response['error']['message'])) {
                //error response
                return $this->engineFailedResponse(['show_message' => true], $response['error']['message']);
            }
            return $this->engineSuccessResponse([
                    'show_message' => true,
                    'responseContent' => $responseFullText, //Arr::get($response, 'choices.0.message.content')
         ]);
        } catch (RequestException $e) {
            // client info update
            updateProgressTextModel(__tr('error while generating the content'));
            // Handle Http errors here
            if ($e->response) {
                $errorMessage = Arr::get($e->response->json(), 'error.message'); // You can get the error message from the API response
            } else {
                $errorMessage = $e->getMessage();
            }
            return $this->engineFailedResponse(['show_message' => true], $errorMessage);
        } catch (\Exception $e) {
            // Handle other Http errors (e.g., timeout)
            return $this->engineFailedResponse(['show_message' => true], $e->getMessage());
        }
    }

    /**
     * Read a line from the stream.
     */
    private function readLine($stream): string
    {
        $buffer = '';

        while (!$stream->eof()) {
            if ('' === ($byte = $stream->read(1))) {
                return $buffer;
            }
            $buffer .= $byte;
            if ($byte === "\n") {
                break;
            }
        }

        return $buffer;
    }

    /**
     * Formatting Response
     *
     * @param   String  $inputText
     *
     * @return  String
     */
    public function formattingResponse($inputText)
    {
        if(!$inputText) {
            return '';
        }
        return Markdown::convert($inputText)->getContent();
    }

    /**
     * Store Code Generation Response
     *
     * @param   mixed  $response
     * @param   array  $inputData
     *
     * @return  object
     */
    public function storeGenerationTransaction($responseText, $inputData)
    {
        //count no of words in the response
        $wordCount = $this->countWords($responseText);

        //process credits calculations and store result
        $creditWalletId = $this->processCreditsCalculation($inputData, $inputData['type'], $responseText);

        if (!$creditWalletId->success()) {
            return $this->engineFailedResponse([], $creditWalletId->message());
        }
        $promptResponseData = [
          'prompt' => $inputData['prompt'],
          'prompt_response' => $responseText,
          'language_code' => $inputData['language'],
          'prompt_template_id' => Arr::get($inputData, 'prompt_key'),
          'response_word_count' => $wordCount,
          'response_char_count' => strlen($responseText),
          'type' => $inputData['type'],
          '__data' => [
            'input_data' => $inputData
          ],
          'credit_wallet_transactions__id' => $creditWalletId->data('creditWalletId'),
          'users__id' => getUserID(),
        ];


        // if failed store response
        if (!$data = $this->promptRepository->storePromptResponse($promptResponseData)) {
            return $this->engineFailedResponse(['show_message' => true], __tr('Failed to save response'));
        }
        // on success
        return $this->engineSuccessResponse([
            'storePromptResponseData' => $data
        ]);
    }

    /**
     * Get Latest image Generated Responses
     *
     * @return object
     */
    public function getLatestTextGeneratedResponses($response)
    {
        if ($response->type == 'image') {
            $generatedImages = [];
            $packageImageFolderPath = getPathByKey('generated_photos', ['{_uid}' => getUserUID()]);
            $imagesName = explode(', ', $response->prompt_response);
            foreach ($imagesName as $key => $image) {
                $generatedImages[] = [
                    'url' => getMediaUrl($packageImageFolderPath, $image),
                    'filename' => $image,
                ];
            }
            $response['_uid'] = $response->_uid;
            $response['generatedImages'] = $generatedImages;
        }

        $response['formattedCreatedAt'] = formatDate($response->created_at);

        return $response;
    }

    /**
     * Process Delete History
     *
     * @param   mixed  $historyUID
     *
     * @return  array
     */
    public function processDeleteHistory($historyUID)
    {
        $where = [
            '_uid' => $historyUID
        ];
        // if not admin user
        if(!isAdmin()) {
            $where['users__id'] = getUserID();
        }
        //get history data to delete
        $historyCollection = $this->promptRepository->fetchIt($where);

        //if is empty then show error message
        if (__isEmpty($historyCollection)) {
            return $this->engineSuccessResponse(['show_message' => true], __tr('history does not exist'));
        }

        //Check if history deleted
        if ($this->promptRepository->deleteIt($historyCollection)) {
            return $this->engineSuccessResponse(['show_message' => true], __tr('History deleted successfully.'));
        }

        //error response
        return $this->engineResponse(18, ['show_message' => true], __tr('History not deleted.'));
    }

    /**
         * Process check enough credits
         *
         * @param   mixed  $inputData
         *
         * @return  EngineResponse
         */
    public function isCreditsEnough($inputData, $type)
    {
        $maxTokenUses = null;
        //get total credits of user
        $totalUserCredits = totalUserCredits();
        updateClientModels([
            'lwProcessingError' => ''
        ]);
        $processErrorMessage = __tr("You does not have enough credits balance. Please purchase more credits.");
        if ($type == 'text' || $type == "code") {
            $textGenerationModel = getAppSettings('open_ai_text_generation_model');
            $getItem = Arr::get(configItem("open_ai.models.text_generation"), $textGenerationModel);
            $maxAllowedTokensForModel = $getItem['max_tokens'];
            $tokensPerWord  = 0.7; // as per the document we are assuming 1000 tokens for 750 words, so for safe side we are treating 1000 tokens for 700 words.
            $perWordPricing = (int) getAppSettings('per_word_pricing');
            $perWordPricing = $perWordPricing == 0 ? 1 : $perWordPricing;
            // check user credits
            if($totalUserCredits <= 0) {
                updateClientModels([
                    'lwProcessingError' => $processErrorMessage
                ]);
                // response back
                return $this->engineFailedResponse(
                    [
                        'show_message' => true,
                    ],
                    $processErrorMessage
                );
            }
            $wordsGenerationPossibilityCount = floor($totalUserCredits / $perWordPricing);
            // check user credits uses
            if($wordsGenerationPossibilityCount <= 0) {
                updateClientModels([
                    'lwProcessingError' => $processErrorMessage
                ]);
                // response back
                return $this->engineFailedResponse(
                    [
                        'show_message' => true,
                    ],
                    $processErrorMessage
                );
            }
            $maxTokenUses = floor($wordsGenerationPossibilityCount / $tokensPerWord);

            if ($maxTokenUses > $maxAllowedTokensForModel) {
                $maxTokenUses = null;
            }
        } elseif
        //Stable Diffusion Image Generation Model
        ($type == 'image-open-ai') {
            $requiredCredits = $inputData['no_of_images'] * getAppSettings('image_pricing')[$inputData['open_ai_image_size']];
            //check credits
            if ($requiredCredits > $totalUserCredits) {
                updateClientModels([
                    'lwProcessingError' => $processErrorMessage
                ]);
                return $this->engineFailedResponse(['show_message' => true], $processErrorMessage);
            }
        } elseif ($type == 'image-stable-diffusion') {
            //Stable Diffusion Image Generation Model
            $imageGenerationModel = getAppSettings('ai_image_generation_model');
            //Stable Diffusion Image Generation size
            $size = configItem("stable_diffusion_ai.models.text_to_image_generation.$imageGenerationModel.size");

            $requiredCredits = $inputData['no_of_images'] * getAppSettings('image_pricing')[$size . 'x' . $size];
            //check credits
            if ($requiredCredits > $totalUserCredits) {
                updateClientModels([
                    'lwProcessingError' => $processErrorMessage
                ]);
                return $this->engineFailedResponse(['show_message' => true], $processErrorMessage);
            }
        } elseif ($type == 'image-animation') {
            //check for image-animation
            $requiredCredits = (int) getAppSettings('image_animation_pricing');
            //check credits
            if ($requiredCredits > $totalUserCredits) {
                updateClientModels([
                    'lwProcessingError' => $processErrorMessage
                ]);
                return $this->engineFailedResponse(['show_message' => true], $processErrorMessage);
            }
        } elseif ($type == 'text-to-speech') {
            //no. of words in query
            $wordCount = $this->countWords($inputData['query']);
            $perWordPricing = (int) getAppSettings('text_to_speech_pricing');
            $requiredCredits = (int) ($wordCount * $perWordPricing);
            //check credits
            if ($requiredCredits > $totalUserCredits) {
                updateClientModels([
                    'lwProcessingError' => $processErrorMessage
                ]);
                return $this->engineFailedResponse(['show_message' => true], $processErrorMessage);
            }
        } elseif ($type == 'speech-to-text') {
            $requiredCredits = getAppSettings('speech_to_text_pricing') * $inputData['audio_duration'];
            //check credits
            if ($requiredCredits > $totalUserCredits) {
                updateClientModels([
                    'lwProcessingError' => $processErrorMessage
                ]);
                return $this->engineFailedResponse(['show_message' => true], $processErrorMessage);
            }
        }

        return $this->engineSuccessResponse([
            'totalUserCredits' => $totalUserCredits,
            'maxTokenUses' => $maxTokenUses
        ]);
    }

    public function processSourceAnimationImage($inputData)
    {
        try {
            $tempUploadedImage = $inputData['temp_image_uploaded'];
            $tempImageFileName = $inputData['temp_image_file_name'];
            $headers = [
            ];
            updateProgressTextModel(
                __tr('requesting ...')
            );

            $basicRequirements = $this->checkBasicRequirements($inputData, 'image-animation');
            // if failed to satisfy basic requirements
            if($basicRequirements->failed()) {
                return $basicRequirements;
            }
            // get auth token
            $accessTokenRequest =  Http::withHeaders($headers)->asForm()->post('https://auth.leialoft.com/auth/realms/leialoft/protocol/openid-connect/token', [
                'grant_type' => 'client_credentials',
                'client_id' => getAppSettings('leiapix_client_id'),
                'client_secret' => getAppSettings('leiapix_client_secret'),
            ]);
            // Throw an exception if a client or server error occurred...
            $accessTokenRequest->throw();
            $accessToken = Arr::get(json_decode($accessTokenRequest->getBody()->getContents(), true), 'access_token');
            if(!$accessToken) {
                return $this->engineFailedResponse([], __tr('Authentication failed.'));
            }
            updateProgressTextModel(
                __tr('processing request ...')
            );
            $leiapixS3ProviderDisk = Storage::build([
                'driver' => 's3',
                'key' => getAppSettings('leiapix_s3_access_key'),
                'secret' => getAppSettings('leiapix_s3_access_secret'),
                'region' => getAppSettings('leiapix_s3_region'),
                'bucket' => getAppSettings('leiapix_s3_bucket'),
                'url' => getAppSettings('leiapix_s3_url'),
                'endpoint' => getAppSettings('leiapix_s3_endpoint'),
                // 'use_path_style_endpoint' => env('AWS_USE_PATH_STYLE_ENDPOINT', false),
                'throw' => false,
                'visibility' => 'public',
            ]);
            $animationVideoFileName = str_random() . '.mp4';
            updateProgressTextModel(
                __tr('preparing upload url ...')
            );
            $presignedUrl = $leiapixS3ProviderDisk->temporaryUploadUrl($animationVideoFileName, now()->addMinutes(5))['url'];
            if(!$presignedUrl) {
                return $this->engineFailedResponse([], __tr('Failed to process request.'));
            }
            $headers = [
                'Authorization' => "Bearer $accessToken"
            ];
            updateProgressTextModel(
                __tr('image preparation ...')
            );
            $disparityRequest =  Http::withHeaders($headers)->asForm()->post('https://api.leiapix.com/api/v1/disparity', [
                'inputImageUrl'      => $tempUploadedImage,
                'resultPresignedUrl' => $presignedUrl,
            ]);
            // Throw an exception if a client or server error occurred...
            $disparityRequest->throw();
            $inputDisparityRequestUrl = Arr::get(json_decode($disparityRequest->getBody()->getContents(), true), 'resultPresignedUrl');
            if(!$inputDisparityRequestUrl) {
                return $this->engineFailedResponse([], __tr('Failed to prepare image.'));
            }
            updateProgressTextModel(
                __tr('image processed ...')
            );
            $inputDisparityUrl = $leiapixS3ProviderDisk->temporaryUrl(
                $animationVideoFileName,
                now()->addMinutes(5)
            );
            if(!$inputDisparityUrl) {
                return $this->engineFailedResponse([], __tr('Failed to process request.'));
            }
            updateProgressTextModel(
                __tr('creating animation ...')
            );
            $animationRequest =  Http::withHeaders($headers)->asForm()->post('https://api.leiapix.com/api/v1/animation', [
                'inputImageUrl'      => $tempUploadedImage,
                'inputDisparityUrl'  => $inputDisparityUrl,
                'resultPresignedUrl' => $presignedUrl,
            ]);
            // Throw an exception if a client or server error occurred...
            $animationRequest->throw();
            $resultPresignedUrl = json_decode($animationRequest->getBody()->getContents(), true);
            if(!$resultPresignedUrl) {
                return $this->engineFailedResponse([], __tr('Failed to create animation.'));
            }
            updateProgressTextModel(
                __tr('processing animation file ...')
            );
            // Get the file contents
            $content = $leiapixS3ProviderDisk->get($animationVideoFileName);
            $packageImageFolderPath = getPathByKey('image_animation', ['{_uid}' => authUID()]);
            $processedFile = $this->mediaEngine->createFileFromContent($packageImageFolderPath, $animationVideoFileName, $content);
            $processedTempFile = $this->mediaEngine->processMoveFile($packageImageFolderPath, $tempImageFileName);
            if(!$processedFile->data('path')) {
                return $this->engineFailedResponse([], __tr('Failed to save the file.'));
            }
            updateProgressTextModel(
                __tr('cleaning up ...')
            );
            $leiapixS3ProviderDisk->delete(
                $animationVideoFileName
            );
            updateClientModels([
                'isAnimatedImageReady' => true,
                'animationVideoPath' => $processedFile->data('path'),
            ]);
            $creditWalletId = $this->processCreditsCalculation($inputData, 'image-animation');
            $data = [
                'prompt' => $processedTempFile->data('fileName'),
                'prompt_response' => $animationVideoFileName,
                'type' => 'image-animation',
                'credit_wallet_transactions__id' => $creditWalletId->data('creditWalletId'),
                'users__id' => getUserID(),
              ];
            if (!$this->promptRepository->storePromptResponse($data)) {
                return $this->engineFailedResponse(['show_message' => true], __tr('Failed to store record.'));
            }
            totalUserCredits();
            return $this->engineSuccessResponse(
                [
                    'processedFile' => $processedFile->data(),
                    'show_message' => true,
                ],
                __tr('You animation for the requested image is ready.')
            );
        } catch (Exception $e) {
            __logDebug($e->getMessage());
            return $this->engineFailedResponse([], __tr('Failed to complete the request.'));
        }
    }

    /**
     * Process the Download file
     *
     * @param string $recordUid
     * @return file|EngineResponse
     */
    public function processDownloadRequest($recordUid, $filename = null)
    {
        //get prompt record
        $promptRecord = $this->promptRepository->fetchIt(['_uid' => $recordUid]);
        //if is empty then show error message
        if (__isEmpty($promptRecord)) {
            return $this->engineSuccessResponse(['show_message' => true], __tr('history does not exist'));
        }
        $pathTypes = [
            'image-animation' => 'image_animation',
            'image' => 'generated_photos',
            'text-to-speech' => 'generated_speech',
        ];
        $savedPath = getPathByKey($pathTypes[$promptRecord->type], ['{_uid}' => authUID()]);
        if(!$filename) {
            $filename = $promptRecord->prompt_response;
        }
        return $this->mediaEngine->downloadRequestedFile($savedPath . DIRECTORY_SEPARATOR . $filename, $filename);
    }
}
