<?php

/**
 * PromptTemplateEngine.php - Main component file
 *
 * This file is part of the Prompt component.
 *-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Prompt;

use Illuminate\Support\Arr;
use App\Yantrana\Base\BaseEngine;
use App\Yantrana\Components\Prompt\Interfaces\PromptTemplateEngineInterface;

class PromptTemplateEngine extends BaseEngine implements PromptTemplateEngineInterface
{
    protected $templateFile;

    protected $templateDefaultFile;

    /**
     * Constructor
     *
     * @return  void
     *-----------------------------------------------------------------------*/
    public function __construct()
    {
        $this->templateFile = app_path('Yantrana/Components/Prompt/Support/text-prompt-collections.php');
        $this->templateDefaultFile = app_path('Yantrana/Components/Prompt/Support/text-prompt-collections.php.bk.php');
    }

    /**
     * Check Slug Uniqueness
     *
     * @param   string  $slug
     *
     * @return  EngineResponse
     */
    public function checkUniqueSlug($slug)
    {
        // Path to the existing PHP file
        $getPromptTemplates = getPromptTemplates(null, null, ['gettext' => false]);

        foreach ($getPromptTemplates as $getPromptTemplateKey => $getPromptTemplate) {

            if(!isset($getPromptTemplate['slug'])) {
                $getPromptTemplate['slug'] = $getPromptTemplateKey;
            }

            if(Arr::get($getPromptTemplate, 'slug') == $slug) {
                return $this->engineFailedResponse(['show_message' => true], __tr('Slug is already exist.'));
            }
            if(isset($getPromptTemplate['templates'])) {
                foreach ($getPromptTemplate['templates'] as $key => $template) {
                    if(!isset($template['slug'])) {
                        $template['slug'] = $key;
                    }

                    if(Arr::get($template, 'slug') == $slug) {
                        return $this->engineFailedResponse(['show_message' => true], __tr('Slug is already exist.'));
                    }
                }
            }
        }
        return $this->engineSuccessResponse();
    }

    /**
     * write To Existing File
     *
     * @param   array  $inputData
     *
     * @return  EngineResponse
     */
    public function writeToExistingFile($inputData)
    {
        $isSlugExist = $this->checkUniqueSlug($inputData['slug']);
        if($isSlugExist->failed()) {
            return $isSlugExist;
        }
        $this->processUpdateCategory($inputData);
        return $this->engineSuccessResponse([
            'show_message' => true,
            'getPromptTemplates' => getPromptTemplates(null, null, ['gettext' => false])
        ], __tr('Category Created'));
    }

    /**
     * Process Update Category
     *
     * @param   array  $inputData
     *
     * @return  EngineResponse
     */
    public function processUpdateCategory($inputData)
    {
        $categorySlug = $inputData['slug'];
        unset($inputData['slug']);
        $this->processAndPrepareTemplateCollectionFile([
            $categorySlug =>  $inputData
        ]);
        //return response
        return $this->engineSuccessResponse(['show_message' => true], __tr('Category Updated.'));
    }

    /**
     * Process and prepare Templates array data
     *
     * @param array $mergedArray
     * @return string
     */
    protected function processAndPrepareTemplateCollectionFile($collection, $options = [])
    {
        $options = array_merge([
            // send the category slug to delete, if delete_template is present then category won't be deleted
            'delete_category' => null,
            // send the template id to delete
            'delete_template' => null,
        ], $options);
        // Path to the existing PHP file
        $getPromptTemplates = getPromptTemplates(null, null, ['gettext' => false]);
        if(!empty($collection)) {
            //merge new array
            $mergedArray = arrayExtend($getPromptTemplates, $collection);
        } else {
            $mergedArray = $getPromptTemplates;
        }

        $gettextReadyItems = [
            'title',
            'description'
        ];
        foreach ($mergedArray as $categoryKey => $categoryValue) {
            // if want to delete category
            if($options['delete_category'] and !$options['delete_template'] and ($options['delete_category'] == $categoryKey)) {
                unset($mergedArray[$categoryKey]);
                continue;
            }
            // gettext ready the some field mentioned in $gettextReadyItems array
            foreach ($gettextReadyItems as $gettextReadyItem) {
                $mergedArray[$categoryKey][$gettextReadyItem] = "__tr('$categoryValue[$gettextReadyItem]')__trEnd";
            }
            if(isset($categoryValue['templates'])) {
                foreach ($categoryValue['templates'] as $templateKey => $templateValue) {
                    // if want to delete template
                    if($options['delete_template'] and ($options['delete_template'] == $templateKey)) {
                        unset($mergedArray[$categoryKey]['templates'][$templateKey]);
                        continue;
                    }
                    // gettext ready the some field mentioned in $gettextReadyItems array
                    foreach ($gettextReadyItems as $gettextReadyItem) {
                        $mergedArray[$categoryKey]['templates'][$templateKey][$gettextReadyItem] = "__tr('$templateValue[$gettextReadyItem]')__trEnd";
                    }
                    if(isset($templateValue['input_fields'])) {
                        foreach ($templateValue['input_fields'] as $inputFieldKey => $inputFieldValue) {
                            $inputValueTitle = $inputFieldValue['title'];
                            $mergedArray[$categoryKey]['templates'][$templateKey]['input_fields'][$inputFieldKey]['title'] = "__tr('$inputValueTitle')__trEnd";
                        }
                    }
                }
            }
        }
        //export into string array
        $processedData = strtr(var_export($mergedArray, true), [
            "'__tr" => '__tr',
            "__trEnd'" => '',
            "\'" => "'",
            ")'," => "),",
        ]);
        // Read the existing PHP file
        $filePath = $this->templateFile;
        // Generate the PHP code to return the merged array
        $phpCode = "<?php\n\nreturn " . $processedData . ";\n";

        //put content into file
        return file_put_contents($filePath, $phpCode);
    }

    /**
     * Process Delete Category
     *
     * @param   int  $categoryId
     *
     * @return  EngineResponse
     */
    public function processDeleteCategory($categoryId)
    {
        $this->processAndPrepareTemplateCollectionFile([], [
            'delete_category' => $categoryId
        ]);
        return $this->engineResponse(21, [
            'show_message' => true,
            'reloadPage' => true,
            'messageType' => 'success',
        ], __tr('Category Deleted'));
    }

    /**
     * Process To Create New Template
     *
     * @param   array  $inputData
     *
     * @return  EngineResponse
     */
    public function processCreateNewTemplate($inputData)
    {
        // check slug uniqueness
        $isSlugExist = $this->checkUniqueSlug($inputData['slug']);
        if($isSlugExist->failed()) {
            return $isSlugExist;
        }
        // get prompt templates
        $getPromptTemplates = getPromptTemplates(null, null, ['gettext' => false]);
        // check if the category exists
        if(!isset($inputData['category']) or !array_key_exists($inputData['category'], $getPromptTemplates)) {
            return $this->engineFailedResponse(['show_message' => true], __tr('Category does not exists.'));
        }
        // add the required parameters
        $inputData['categoryId'] = $inputData['category'];
        $inputData['templateId'] = $inputData['slug'];
        // remove
        unset($inputData['category']);
        unset($inputData['slug']);
        $this->updateTemplateData($inputData);

        //return response
        return $this->engineSuccessResponse(['show_message' => true], __tr('Template Added'));
    }

    /**
     * Prepare Edit Template Data
     *
     * @param   string  $templateId
     * @param   string  $categoryId
     *
     * @return  EngineResponse
     */
    public function prepareEditTemplateData($templateId, $categoryId)
    {
        $getPromptTemplates = getPromptTemplates(null, null, ['gettext' => false]);
        if(!isset($getPromptTemplates[$categoryId]['templates'][$templateId])) {
            return $this->engineFailedResponse(['show_message' => true], __tr('Template or Category does not exist'));
        }
        $options = [];
        $inputData = $getPromptTemplates[$categoryId]['templates'][$templateId];
        if(isset($inputData['input_fields'])) {
            foreach ($inputData['input_fields'] as $key => $value) {
                $options[$key] = array_merge([
                    'title' => ' ',
                    'type' => ' ',
                    'default' => ' ',
                ], $value);
                // remove validation item from interface
                if(isset($options[$key]['validations'])) {
                    unset($options[$key]['validations']);
                }
            }
        }

        if(!isset($inputData['slug'])) {
            $inputData['slug'] = $templateId;
        }
        updateClientModels([
            'templatesData' => $inputData,
            'newInputFields' => $options,
        ]);

        return $this->engineSuccessResponse(['templatesData' => $options]);
    }

    /**
     * Update Template Data
     *
     * @param   array  $inputData
     *
     * @return  EngineResponse
     */
    public function updateTemplateData($inputData)
    {
        $getPromptTemplates = getPromptTemplates(null, null, ['gettext' => false]);

        $categoryCollection = Arr::get($getPromptTemplates, $inputData['categoryId']);
        if(!isset($categoryCollection['templates'])) {
            $categoryCollection['templates'] = [];
        }

        $collection = [];
        $categoryId = $inputData['categoryId'];
        $templateId = $inputData['templateId'];
        unset($inputData['categoryId']);
        unset($inputData['templateId']);
        $collection = [
            $categoryId => [
                'templates' =>  [
                    $templateId => $inputData
                ]
            ]
        ];
        $collection[$categoryId]['templates'][$templateId]['input_fields'] = [];
        $inputFieldsArray = [];
        if(isset($inputData['input_fields'])) {
            $inputFields = $inputData['input_fields'];
            foreach ($inputFields as $key => $inputFieldValues) {
                foreach ($inputFieldValues['values'] as $key => $value) {
                    $value = array_merge([
                        'value' => '',
                    ], $value);
                    $optionValue = $value['value'];
                    $inputFieldsArray[$inputFieldValues['option_name']][$value['key']] = $optionValue;
                }
            }
            $collection[$categoryId]['templates'][$templateId]['input_fields'] = $inputFieldsArray;
        }

        $this->processAndPrepareTemplateCollectionFile($collection);

        //return response
        return $this->engineSuccessResponse(['show_message' => true], __tr('Template Updated'));

    }

    /**
     * Process To Delete Template Data
     *
     * @param   string  $categoryId
     * @param   string  $templateId
     *
     * @return  EngineResponse
     */
    public function processDeleteTemplateData($categoryId, $templateId)
    {

        $this->processAndPrepareTemplateCollectionFile([], [
            'delete_category' => $categoryId,
            'delete_template' => $templateId,
        ]);
        return $this->engineResponse(21, [
            'show_message' => true,
            'reloadPage' => true,
            'messageType' => 'success',
        ], __tr('Template Deleted'));
    }

    /**
     * Reset the category and templates file to default
     *
     * @return EngineResponse
     */
    public function resetToDefaultTemplatesProcess()
    {
        try {
            // delete existing file
            unlink($this->templateFile);
            // copy from default
            copy($this->templateDefaultFile, $this->templateFile);
        } catch (\Exception $e) {
            return $this->engineFailedResponse(['show_message' => true], $e->getMessage());
        }
        return $this->engineResponse(21, [
            'show_message' => true,
            'reloadPage' => true,
            'messageType' => 'success',
        ], __tr('Templates and Categories restored to defaults.'));
    }

    /**
     * Merge templates file to default
     *
     * @return EngineResponse
     */
    public function mergeDefaultTemplatesProcess()
    {
        try {
            $this->processAndPrepareTemplateCollectionFile(require $this->templateDefaultFile);
            // copy from default
        } catch (\Exception $e) {
            return $this->engineFailedResponse(['show_message' => true], $e->getMessage());
        }
        return $this->engineResponse(21, [
            'show_message' => true,
            'reloadPage' => true,
            'messageType' => 'success',
        ], __tr('Merged with defaults.'));
    }

    /**
     * Download current template file
     *
     * @return download response
     */
    public function downloadCurrentTemplateFileProcess()
    {
        return response()->download($this->templateFile);
    }
}
