<?php

/**
 * PromptRepository.php - Repository file
 *
 * This file is part of the Prompt component.
 *-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Prompt\Repositories;

use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use App\Yantrana\Base\BaseRepository;
use App\Yantrana\Components\Prompt\Models\PromptModel;
use App\Yantrana\Components\Prompt\Interfaces\PromptRepositoryInterface;

class PromptRepository extends BaseRepository implements PromptRepositoryInterface
{
    /**
     * primary model instance
     *
     * @var  object
     */
    protected $primaryModel = PromptModel::class;

    /**
     * Store Prompt Response
     *
     * @param   Array  $inputData
     *
     * @return  mixed
     */
    public function storePromptResponse($inputData)
    {
        $keyValues = [
           'prompt',
           'prompt_response',
           'language_code',
           'response_word_count',
           'response_char_count',
           'type',
           'prompt_template_id',
           'prompt_category_id',
           '__data',
           'credit_wallet_transactions__id',
           'users__id',
        ];
        //data stored into prompt model
        $dataStored = new PromptModel();
        // check if user profile stored successfully
        if ($dataStored->assignInputsAndSave($inputData, $keyValues)) {
            return $dataStored;
        }

        return false;
    }

    /**
     * Fetch User History List DataTable
     *
     * @return Array
     */
    public function fetchUserHistoryListDataTable($historyType)
    {
        $dataTableConfig = [
           'searchable' => [
              'prompt',
              'prompt_response',
           ],
        ];

        return PromptModel::where(['users__id' => getUserID(), 'type' => $historyType])
           ->dataTables($dataTableConfig)
           ->toArray();
    }

    /**
     * Fetch All User History List DataTable
     *
     * @return  Array
     */
    public function fetchAllUsersHistoryListDataTable($historyType)
    {
        $dataTableConfig = [
           'searchable' => [
              'prompt',
              'prompt_response',
              'full_name' => DB::raw('CONCAT(users.first_name, " ", users.last_name)'),
           ],
        ];

        return PromptModel::join('users', 'prompts.users__id', '=', 'users._id')
           ->select(
               __nestedKeyValues([
                 'prompts.*',
                 'users' => [
                    '_uid as userUid',
                    DB::raw("CONCAT(COALESCE(first_name, ''), ' ', COALESCE(last_name, '')) AS full_name"),
                 ]
            ])
           )
           ->where('prompts.type', '=', $historyType)
           ->with('user')
           ->dataTables($dataTableConfig)
           ->toArray();
    }

    /**
     * Prepare Generation Graph
     *
     * @param   Date  $startDate
     * @param   Date  $endDate
     *
     * @return  eloquent collection object
     */
    public function prepareGenerationGraph($startDate = null, $endDate = null)
    {
        $query = PromptModel::join('credit_wallet_transactions', 'prompts.credit_wallet_transactions__id', '=', 'credit_wallet_transactions._id')
        ->whereBetween('prompts.created_at', [$startDate, $endDate])
           ->where(['prompts.users__id' => getUserID()])
           ->select(
               __nestedKeyValues([
                 'credit_wallet_transactions.credit_type',
                 'prompts.type',
                 'credit_wallet_transactions.credits as credits',
                 DB::raw('SUM(credit_wallet_transactions.credits) as total_credits_used')
            ])
           )
           ->groupBy('credit_wallet_transactions.credit_type')
           ->get();

        return $query;
    }

    /**
     * Fetch Count Generation
     *
     * @param   String  $type
     * @param   Date  $startDate
     * @param   Date  $endDate
     *
     * @return  int
     */
    public function fetchCountGeneration($type, $startDate = null, $endDate = null)
    {
        $query = PromptModel::where(['users__id' => getUserID(), 'type' => $type])
           ->whereBetween('created_at', [$startDate, $endDate])
           ->count();

        return $query;
    }
    /**
     * Fetch Count Generation
     *
     * @param   String  $type
     * @param   Date  $startDate
     * @param   Date  $endDate
     *
     * @return  int
     */
    public function fetchCountGenerationAdminDashboard($type, $startDate = null, $endDate = null)
    {
        $query = PromptModel::where(['type' => $type])
           ->whereBetween('created_at', [$startDate, $endDate])
           ->count();

        return $query;
    }
}
