<?php

/**
 * ManageUserController.php - Controller file
 *
 * This file is part of the User component.
 *-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\User\Controllers;

use Illuminate\Support\Facades\Auth;
use App\Yantrana\Base\BaseController;
use App\Yantrana\Components\User\ManageUserEngine;
use App\Yantrana\Components\User\Requests\UserAddRequest;
use App\Yantrana\Components\User\Requests\UserUpdateRequest;
use App\Yantrana\Components\User\Requests\PaypalTransactionRequest;

class ManageUserController extends BaseController
{
    /**
     * @var  ManageUserEngine - ManageUser Engine
     */
    protected $manageUserEngine;

    /**
     * Constructor
     *
     * @param  ManageUserEngine  $manageUserEngine - ManageUser Engine
     * @return  void
     *-----------------------------------------------------------------------*/
    public function __construct(ManageUserEngine $manageUserEngine)
    {
        $this->manageUserEngine = $manageUserEngine;
    }

    /**
     * Manage User List.
     *
     * @return json object
     *---------------------------------------------------------------- */
    public function userList($status)
    {
        return $this->loadManageView('user.manage.list');
    }

    /**
     * Manage User List.
     *
     * @return json object
     *---------------------------------------------------------------- */
    public function userDataTableList($status)
    {
        return $this->manageUserEngine->prepareUsersDataTableList($status);
    }

    /**
     * Add new user view.
     *
     * @return json object
     *---------------------------------------------------------------- */
    public function addNewUserView()
    {
        return $this->loadManageView('user.manage.add');
    }

    /**
     * Add new user view.
     *
     * @param object UserAddRequest $request
     * @return json object
     *---------------------------------------------------------------- */
    public function processAddNewUser(UserAddRequest $request)
    {
        $processReaction = $this->manageUserEngine->processAddUser($request->all());

        return $this->responseAction(
            $this->processResponse($processReaction, [], [], true),
            $this->redirectTo('manage.user.view_list', ['status' => 1])
        );
    }

    /**
     * Edit User.
     *
     * @return json object
     *---------------------------------------------------------------- */
    public function processUpdateUser($userUid, UserUpdateRequest $request)
    {
        $processReaction = $this->manageUserEngine->processUserUpdate($userUid, $request->all());

        if ($processReaction['reaction_code'] == 1) {
            return $this->responseAction(
                $this->processResponse($processReaction, [], [], true),
                $this->redirectTo('manage.user.view_list', ['status' => 1])
            );
        }

        return $this->processResponse($processReaction, [], [], true);
    }

    /**
     * Process Soft delete user.
     *
     * @return json object
     *---------------------------------------------------------------- */
    public function processUserSoftDelete($userUid)
    {
        $processReaction = $this->manageUserEngine->processSoftDeleteUser($userUid);

        return $this->responseAction(
            $this->processResponse($processReaction, [], [], true)
        );
    }

    /**
     * Process Permanent delete user.
     *
     * @return json object
     *---------------------------------------------------------------- */
    public function processUserPermanentDelete($userUid)
    {
        $processReaction = $this->manageUserEngine->processPermanentDeleteUser($userUid);

        return $this->responseAction(
            $this->processResponse($processReaction, [], [], true)
        );
    }

    /**
     * Process Restore user.
     *
     * @return json object
     *---------------------------------------------------------------- */
    public function processRestoreUser($userUid)
    {
        $processReaction = $this->manageUserEngine->processUserRestore($userUid);

        return $this->responseAction(
            $this->processResponse($processReaction, [], [], true)
        );
    }

    /**
     * Process Block user.
     *
     * @return json object
     *---------------------------------------------------------------- */
    public function processUserBlock($userUid)
    {
        $processReaction = $this->manageUserEngine->processBlockUser($userUid);

        return $this->responseAction(
            $this->processResponse($processReaction, [], [], true)
        );
    }

    /**
     * Process Unblock user.
     *
     * @return json object
     *---------------------------------------------------------------- */
    public function processUserUnblock($userUid)
    {
        $processReaction = $this->manageUserEngine->processUnblockUser($userUid);

        return $this->responseAction(
            $this->processResponse($processReaction, [], [], true)
        );
    }

    /**
     * Get User Details.
     *
     * @param  string  $userUid
     * @return json object
     *---------------------------------------------------------------- */
    public function getUserDetails($userUid)
    {
        $processReaction = $this->manageUserEngine->prepareUserDetails($userUid);

        return $this->loadManageView('user.manage.details', $processReaction['data']);
    }

    /**
     * Show user Transaction List.
     *
     *-----------------------------------------------------------------------*/
    public function manageUserTransactionList($userUid)
    {
        return $this->manageUserEngine->getUserTransactionList($userUid);
    }

    /**
     * Add User Allocate Credits
     *
     * @param   object PaypalTransactionRequest  $request
     * @return  json object
     */
    public function addUserAllocateCredits(PaypalTransactionRequest $request)
    {
        $request->validate(['allocate_credits' => 'required']);
        return $this->manageUserEngine->allocatedCreditsForUser($request->all());
    }
}
