<?php
/**
* CreditWalletEngine.php - Main component file
*
* This file is part of the Credit Wallet User component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\User;

use Auth;
use Carbon\Carbon;
use Razorpay\Api\Api;
use Illuminate\Support\Arr;
use App\Yantrana\Base\BaseEngine;
use Imdhemy\Purchases\Facades\Product as InAppProduct;
use App\Yantrana\Components\User\Models\loginLogsModel;
use App\Yantrana\Components\User\Repositories\UserRepository;
use App\Yantrana\Components\User\Models\CreditWalletTransaction;
use App\Yantrana\Components\User\Repositories\LoginLogsRepository;
use App\Yantrana\Components\User\Repositories\ManageUserRepository;
use App\Yantrana\Components\User\Repositories\CreditWalletRepository;
use App\Yantrana\Components\Configuration\Repositories\ConfigurationRepository;
use App\Yantrana\Components\CreditPackage\Repositories\CreditPackageRepository;

class CreditWalletEngine extends BaseEngine
{
    /**
     * @var  CreditWalletRepository - CreditWallet Repository
     */
    protected $creditWalletRepository;

    /**
     * @var ManageUserRepository - Manage User Repository
     */
    protected $manageUserRepository;

    /**
     * @var  ConfigurationRepository - Configuration Repository
     */
    protected $configurationRepository;

    /**
     * @var PaypalEngine - Paypal Engine
     */
    protected $paypalEngine;

    /**
     * @var StripeEngine - Stripe Engine
     */
    protected $stripeEngine;

    /**
     * @var  CreditPackageRepository - CreditPackage Repository
     */
    protected $creditPackageRepository;

    /**
     * @var RazorpayEngine - Razorpay Engine
     */
    protected $razorpayEngine;

    /**
     * @var coinGateEngine - coinGate Engine
     */
    protected $coinGateEngine;

    /**
     * @var razorpayAPI - razorpayAPI
     */
    protected $razorpayAPI;

    /**
     * @var UserRepository - User Repository
     */
    protected $userRepository;

    /**
     * @var LoginLogsRepository - loginLogs Repository
     */
    protected $loginLogsRepository;

    /**
     * Constructor
     *
     * @param  CreditWalletRepository  $creditWalletRepository - CreditWallet Repository
     * @param  ManageUserRepository  $manageUserRepository - Manage User Repository
     * @param  ConfigurationRepository  $configurationRepository - Configuration Repository
     * @param  PaypalEngine  $paypalEngine- Paypal Engine
     * @param  StripeEngine  $stripeEngine- Stripe Engine
     * @param  CreditPackageRepository  $creditPackageRepository - CreditPackage Repository
     * @param  RazorpayEngine  $razorpayEngine - Razorpay Repository
     * @param  CoinGateEngine  $coinGateEngine - coinGate Engine
     * @param  UserRepository  $userRepository - user Repository
     * @param  LoginLogsRepository  $loginLogsRepository - loginLogs Repository
     * @return  void
     *-----------------------------------------------------------------------*/
    public function __construct(
        CreditWalletRepository $creditWalletRepository,
        ManageUserRepository $manageUserRepository,
        ConfigurationRepository $configurationRepository,
        PaypalEngine $paypalEngine,
        StripeEngine $stripeEngine,
        CreditPackageRepository $creditPackageRepository,
        RazorpayEngine $razorpayEngine,
        CoinGateEngine $coinGateEngine,
        UserRepository $userRepository,
        LoginLogsRepository $loginLogsRepository,
    ) {
        $this->creditWalletRepository = $creditWalletRepository;
        $this->manageUserRepository = $manageUserRepository;
        $this->configurationRepository = $configurationRepository;
        $this->paypalEngine = $paypalEngine;
        $this->stripeEngine = $stripeEngine;
        $this->creditPackageRepository = $creditPackageRepository;
        $this->razorpayEngine = $razorpayEngine;
        $this->coinGateEngine = $coinGateEngine;
        $this->userRepository = $userRepository;
        $this->loginLogsRepository = $loginLogsRepository;
    }
    /**
    * Prepare razorpay.
    *
    *
    *---------------------------------------------------------------- */
    public function initSetup()
    {
        if (getAppSettings('use_test_razorpay')) {
            $razorpayKey = getAppSettings('razorpay_testing_key');
            $razorpaySecret = getAppSettings('razorpay_testing_secret_key');
        } else {
            $razorpayKey = getAppSettings('razorpay_live_key');
            $razorpaySecret = getAppSettings('razorpay_live_secret_key');
        }
        $this->razorpayAPI = new Api($razorpayKey, $razorpaySecret);
    }

    /**
     * Prepare Credit Wallet User Data.
     *
     *
     *---------------------------------------------------------------- */
    public function prepareCreditWalletUserData()
    {
        //get credit package data
        $packageCollection = $this->creditPackageRepository->fetchAllActiveCreditPackage();

        $creditPackages = [];
        // check if user collection exists
        if (!__isEmpty($packageCollection)) {
            foreach ($packageCollection as $key => $package) {
                $packageImageUrl = '';
                $packageImageFolderPath = getPathByKey('package_image', ['{_uid}' => $package->_uid]);
                $packageImageUrl = getMediaUrl($packageImageFolderPath, $package['image']);
                $creditPackages[] = [
                    '_id' => $package['_id'],
                    '_uid' => $package['_uid'],
                    'package_uid' => toggleProductId($package['_uid']),
                    'package_name' => $package['title'],
                    'credit' => $package['credits'],
                    'price' => intval($package['price']),
                    'packageImageUrl' => $packageImageUrl,
                ];
            }
        }

        return $this->engineReaction(1, [
            'creditWalletData' => [
                'creditPackages' => $creditPackages,
            ],
            'paymentData' => [
                'currencySymbol' => getAppSettings('currency_symbol'),
                'currency' => getAppSettings('currency'),
                'enablePaypalCheckout' => getAppSettings('enable_paypal'),
                'useTestPaypalCheckout' => getAppSettings('use_test_paypal_checkout'),
                'paypalTestingClientId' => getAppSettings('paypal_checkout_testing_client_id'),
                'paypalLiveClientId' => getAppSettings('paypal_checkout_live_client_id'),
                'userName' => getUserAuthInfo('profile.full_name'),
                'userEmail' => getUserAuthInfo('profile.email'),
                'enableRazorpay' => getAppSettings('enable_razorpay'),
                'useTestRazorpay' => getAppSettings('use_test_razorpay'),
                'razorpayTestKey' => getAppSettings('razorpay_testing_key'),
                'razorpayLiveKey' => getAppSettings('razorpay_live_key'),
                'enableStripe' => getAppSettings('enable_stripe'),
                'useTestStripe' => getAppSettings('use_test_stripe'),
                'stripeTestPublishableKey' => getAppSettings('stripe_testing_publishable_key'),
                'stripeLivePublishableKey' => getAppSettings('stripe_live_publishable_key'),
            ],
        ]);
    }

    /**
     * Prepare Credit wallet Information
     *
     * @return void
     */
    public function prepareCreditWalletInfo()
    {
        return $this->engineReaction(1, [
            'creditBalance' => totalUserCredits()
        ]);
    }

    /**
     * get user transaction list data.
     *
     *
     * @return object
     *---------------------------------------------------------------- */
    public function prepareUserWalletTransactionList()
    {
        //get user transaction data
        $transactionCollection = $this->creditWalletRepository->fetchUserWalletTransactionList();

        $requireColumns = [
            '_id',
            '_uid',
            'created_at' => function ($key) {
                return formatDate($key['created_at'], 'l jS F Y h:i:s A');
            },
            'credits' => function ($key) {
                return __tr($key['credits']);
            },
            'credit_type',
            'transactionType' => function ($key) {
                $type = null;
                if (!__isEmpty($key['get_user_financial_transaction'])) {
                    $type = 1;
                } elseif ($key['credit_type'] == 6) {
                    $type = 6;
                } elseif(($key['credit_type'] >= 2 && $key['credit_type'] <= 5) or ($key['credit_type'] == 7)) {
                    $type = $key['credit_type'];
                } elseif ($key['credit_type'] == 1) {
                    $type = 6;
                }
                return $type;
            },
            'formattedTransactionType' => function ($key) {
                $type = $key['credit_type'];
                if (!__isEmpty($key['get_user_financial_transaction'])) {
                    $type = 1;
                } elseif ($key['credit_type'] == 6) {
                    $type = 6;
                } elseif(($key['credit_type'] >= 2 && $key['credit_type'] <= 5) or ($key['credit_type'] == 7)) {
                    $type = $key['credit_type'];
                } elseif ($key['credit_type'] == 1) {
                    $type = 6;
                }

                return isset($type) ? configItem('user_transaction_type', $type) : null;
            },
            'financialTransactionDetail' => function ($key) {
                $financialTransaction = [];
                if (!__isEmpty($key['get_user_financial_transaction'])) {
                    $transactionData = $key['get_user_financial_transaction'];
                    $financialTransaction = [
                        '_id' => $transactionData['_id'],
                        '_uid' => $transactionData['_uid'],
                        'status' => configItem('payments.status_codes', $transactionData['status']),
                        'amount' => priceFormat($transactionData['amount'], true, false),
                        'created_at' => formatDate($transactionData['created_at']),
                        'currency_code' => $transactionData['currency_code'],
                        'payment_mode' => isset($transactionData['is_test']) ? configItem('payments.payment_checkout_modes', $transactionData['is_test']) : null,
                        'method' => $transactionData['method'],
                    ];
                }

                return $financialTransaction;
            },
        ];

        return $this->dataTableResponse($transactionCollection, $requireColumns);
    }

    /**
     * Process paypal complete transaction.
     *
     * @param  array  $inputData
     *---------------------------------------------------------------- */
    public function processPaypalTransaction($inputData, $packageUid)
    {
        if ($this->creditWalletRepository->isAlreadyProcessed($inputData['id'])) {
            return $this->engineReaction(2, null, __tr('already processed'));
        }
        // process card charge
        $paypalPaymentDetail = $this->paypalEngine->getOrder($inputData['id']);

        //check reaction code is 1 or not
        if ($paypalPaymentDetail['reaction_code'] == 1) {
            $paypalResponse = $paypalPaymentDetail['data']['transactionResponse'];

            //check transaction status is completed or not
            if ($paypalResponse['status'] == 'COMPLETED') {
                //store transaction data
                if ($this->storePaymentData($paypalResponse, $packageUid, 'paypalPayment')) {
                    return $this->engineReaction(1, null, __tr('Payment Complete'));
                }
            } else {
                //payment failed response
                return $this->engineReaction(2, null, __tr('Payment Failed'));
            }
        } else {
            //error response
            return $this->engineReaction(2, [
                'errorMessage' => __tr('Failed to process'),
            ], __tr('Payment Failed'));
        }
    }


    /**
     * Process Payment request
     *
     * @param  array  $inputData
     *---------------------------------------------------------------- */
    public function processPayment($inputData)
    {
        $paymentMethod = $inputData['select_payment_method'];
        $packageUid = $inputData['select_package'];

        //get package collection
        $packageCollection = $this->creditPackageRepository->fetch($packageUid);

        //if it is empty then throw error
        if (__isEmpty($packageCollection)) {
            //success function
            return $this->engineReaction(2, null, __tr('Package does not exist.'));
        }

        //check payment method and package data exists
        if ($paymentMethod == 'stripe') {
            $packageImageFolderPath = getPathByKey('package_image', ['{_uid}' => $packageCollection->_uid]);
            $packageImageUrl = getMediaUrl($packageImageFolderPath, $packageCollection['image']);

            $stripeRequestData = [
                'packageUid' => $packageUid,
                'package_name' => $packageCollection['title'],
                'amount' => $packageCollection['price'],
                'currency' => getAppSettings('currency'),
                'packageImageUrl' => $packageImageUrl,
            ];
            //check is mobile app request
            if (isMobileAppRequest()) {
                $stripeRequestData['redirectAppUrl'] = base64_encode($inputData['redirectAppUrl']);
            }

            //get stripe session ata
            $stripeSessionData = $this->stripeEngine->processStripeRequest($stripeRequestData);

            //if reaction code is 1 then success response
            if ($stripeSessionData['reaction_code'] == 1) {
                return $this->engineReaction(1, [
                    'stripeSessionData' => $stripeSessionData['data'],
                ]);
            } else {
                //stripe failure response
                return $this->engineReaction(2, [
                    'errorMessage' => $stripeSessionData['data']['errorMessage'],
                ], __tr('Failed'));
            }
        }

        //failure response
        return $this->engineReaction(2, null, __tr('Invalid payment method'));
    }

    /**
     * Process retrieve stripe payment data
     *
     * @param  array  $inputData
     *---------------------------------------------------------------- */
    public function prepareStripeRetrieveData($inputData)
    {
        //get stripe payment ata
        $stripePaymentData = $this->stripeEngine->retrieveStripeData($inputData['session_id']);

        //check reaction code is 1
        if ($stripePaymentData['reaction_code'] == 1) {
            $stripeData = $stripePaymentData['data']['paymentData'];
            if ($stripeData['status'] == 'succeeded') {

                if ($this->creditWalletRepository->isAlreadyProcessed($stripeData['id'])) {
                    return $this->engineReaction(2, null, __tr('Already been processed'));
                }

                //store transaction data
                if ($this->storeStripePaymentData($stripeData, $inputData['packageUid'])) {
                    return $this->engineReaction(1, null, __tr('Payment Complete'));
                } else {
                    //payment failed response
                    return $this->engineReaction(2, null, __tr('Payment failed'));
                }
            }

        }
        //failure response
        return $this->engineReaction(2, null, __tr('Payment failed.'));
    }

    /**
     * Process paypal complete transaction.
     *
     * @param  array  $inputData
     *---------------------------------------------------------------- */
    public function storeStripePaymentData($inputData, $packageUid)
    {
        //get package collection
        $packageCollection = $this->creditPackageRepository->fetch($packageUid);

        //if it is empty then throw error
        if (__isEmpty($packageCollection)) {
            //success function
            return $this->engineReaction(2, null, __tr('Package does not exist.'));
        }

        if (!__isEmpty($inputData)) {
            $isStripeTestMode = 1;
            if (!getAppSettings('use_test_stripe')) {
                $isStripeTestMode = 2;
            }

            $userId = getUserID();
            if(isset($inputData['metadata']['userId'])) {
                $userId = $inputData['metadata']['userId'];
            }

            //collect store data
            $storeData = [
                'status' => 2,
                'amount' => $inputData['amount'] / 100,
                'users__id' => $userId,
                'method' => configItem('payments.payment_methods', 2),
                'currency_code' => getAppSettings('currency'),
                'is_test' => $isStripeTestMode,
                'txn_id' => array_get($inputData, 'id'),
                '__data' => [
                    'rawPaymentData' => json_encode($inputData),
                    'packageName' => $packageCollection['title'],
                ],
            ];

            //store transaction process
            if ($this->creditWalletRepository->storeTransaction($storeData, $packageCollection, $userId)) {
                storeReferralWalletTransaction($userId);
                //success function
                return $this->engineReaction(1, null, __tr('Purchase successfully'));
            }
        }
        //error response
        return $this->engineReaction(2, null, __tr('Purchased failed'));
    }

    /**
     * Process paypal complete transaction.
     *
     * @param  array  $inputData
     *---------------------------------------------------------------- */
    public function processRazorpayCheckout($inputData)
    {
        // process card charge
        $razorpayChargeRequest = $this->razorpayEngine->capturePayment($inputData['razorpayPaymentId']);
        //check reaction code is 1 or not
        if ($razorpayChargeRequest['reaction_code'] == 1) {
            $razorpayResponse = $razorpayChargeRequest['data']['transactionDetail'];

            if ($this->creditWalletRepository->isAlreadyProcessed($razorpayResponse['id'])) {
                return $this->engineReaction(2, null, __tr('Already been processed'));
            }

            //check transaction status is completed or not
            if ($razorpayResponse['captured'] === true) {
                //store transaction data
                if ($this->storePaymentData($razorpayResponse, $inputData['packageUid'], 'razorpayPayment')) {
                    return $this->engineReaction(1, null, __tr('Payment Complete'));
                }
            } else {
                //payment failed response
                return $this->engineReaction(2, null, __tr('Payment Failed'));
            }
        } else {
            //error response
            return $this->engineReaction(2, [
                'errorMessage' => __tr('Failed to process'),
            ], __tr('Payment Failed'));
        }
    }

    /**
     * Process paypal complete transaction.
     *
     * @param  array  $inputData
     *---------------------------------------------------------------- */
    public function storePaymentData($inputData, $packageUid, $paymentMethod, $userId = null)
    {
        //get package collection
        $packageCollection = $this->creditPackageRepository->fetch($packageUid);

        //if it is empty then throw error
        if (__isEmpty($packageCollection)) {
            //success function
            return $this->engineReaction(2, null, __tr('Package does not exist.'));
        }

        // check if user collection exists
        if (!__isEmpty($inputData)) {
            $isTestMode = 1;
            $amount = $packageCollection['price'];
            $currency = getAppSettings('currency');
            $paymentType = null;
            //collect paypal payment data
            if ($paymentMethod == 'paypalPayment') {
                $paymentType = configItem('payments.payment_methods', 1);
                //check is live mode
                if (!getAppSettings('use_test_paypal_checkout')) {
                    $isTestMode = 2;
                }

            //collect razorpay payment data
            } elseif ($paymentMethod == 'razorpayPayment') {
                $paymentType = configItem('payments.payment_methods', 3);
                //check is live mode
                if (!getAppSettings('use_test_razorpay')) {
                    $isTestMode = 2;
                }
            } elseif ($paymentMethod == 'apiPaypalPayment') {
                $paymentType = configItem('payments.payment_methods', 4);
                //check is live mode
                if (!getAppSettings('use_test_paypal_checkout')) {
                    $isTestMode = 2;
                }
            }
            if($userId == null) {
                $userId = getUserID();
            }
            //collect store data
            $storeData = [
                'status' => 2, //completed
                'amount' => $amount,
                'users__id' => $userId,
                'method' => $paymentType,
                'currency_code' => $currency,
                'is_test' => $isTestMode,
                'txn_id' => array_get($inputData, 'id'),
                '__data' => [
                    'rawPaymentData' => json_encode($inputData),
                    'packageName' => $packageCollection['title'],
                ],
            ];

            //store transaction process
            if ($financialTransactionId = $this->creditWalletRepository->storeTransaction($storeData, $packageCollection, $userId)) {
                //fetch updated user total credits by helper function
                totalUserCredits();
                storeReferralWalletTransaction($userId);
                //success function
                return $this->engineReaction(1, null, __tr('Purchase successfully'));
            }
        }

        return $this->engineReaction(2, null, __tr('Purchased failed'));
    }

    /**
     * Prepare Credit Wallet Stripe Intent User Data.
     *
     *
     *---------------------------------------------------------------- */
    public function processCreateStripePaymentIntent($inputData)
    {
        //get package collection
        $packageCollection = $this->creditPackageRepository->fetch($inputData['packageUid']);

        //if it is empty then throw error
        if (__isEmpty($packageCollection)) {
            //success function
            return $this->engineReaction(2, null, __tr('Package does not exist.'));
        }

        //get stripe payment intent data
        $stripePaymentIntentData = $this->stripeEngine->createPaymentIntent($packageCollection, $inputData['paymentMethodId']);

        if ($stripePaymentIntentData['reaction_code'] == 1) {
            return $this->engineReaction(1, $stripePaymentIntentData);
        }

        return $this->engineReaction(2, $stripePaymentIntentData);
    }

    /**
     * Prepare Credit Wallet Stripe Intent User Data.
     *
     *
     *---------------------------------------------------------------- */
    public function retrieveStripePaymentIntent($inputData)
    {
        //get package collection
        $packageCollection = $this->creditPackageRepository->fetch($inputData['packageUid']);

        //if it is empty then throw error
        if (__isEmpty($packageCollection)) {
            //success function
            return $this->engineReaction(2, null, __tr('Package does not exist.'));
        }

        //get stripe payment intent data
        $retrievePaymentIntentData = $this->stripeEngine->retrievePaymentIntent($packageCollection, $inputData['paymentIntentId']);

        if ($retrievePaymentIntentData['reaction_code'] == 1) {
            return $this->engineReaction(1, $retrievePaymentIntentData);
        }

        return $this->engineReaction(2, $retrievePaymentIntentData);
    }
    /**
     * Prepare app purchased User Data.
     *
     *
     *---------------------------------------------------------------- */

    public function processInAppPurchase($request)
    {
        try {
            //code...
            $productReceipt = InAppProduct::googlePlay()->id($request->get('productId'))->token($request->get('purchaseToken'))->get();
            $purchaseState = $productReceipt->getPurchaseState();
            $orderId = $productReceipt->getOrderId();
            if($purchaseState != 0) {
                return $this->engineReaction(2, null, __tr('Purchase not complete'));
            }
            //get package collection
            $packageCollection = $this->creditPackageRepository->fetch(toggleProductId($request->get('productId')));

            if ($this->creditWalletRepository->isAlreadyProcessed($orderId)) {
                return $this->engineReaction(2, null, __tr('Already been processed'));
            }

            if (!__isEmpty($packageCollection)) {
                //collect store data
                $storeData = [
                    'status' => 2,
                    'amount' => $packageCollection->price,
                    'users__id' => getUserID(),
                    'method' => configItem('payments.payment_methods', 5),
                    'currency_code' => getAppSettings('currency'),
                    'is_test' => configItem('payments.in_app_test_mode'),
                    'txn_id' => $orderId,
                    ' __data' => [
                        'rawPaymentData' => json_encode($productReceipt->toArray()),
                        'packageName' => $packageCollection->title,
                    ],
                ];

                //store transaction process
                if ($this->creditWalletRepository->storeTransaction($storeData, $packageCollection)) {
                    //success function
                    return $this->engineReaction(1, null, __tr('Purchase successfully'));
                }
            }
            //error response
            return $this->engineReaction(2, null, __tr('Transaction not completed'));
        } catch (\Exception $e) {
            return $this->engineReaction(2, null, __tr('Transaction not completed'));
        }
    }
    /**
        * Prepare coingate Data.
        *
        *
        *---------------------------------------------------------------- */

    public function processCoinGateCheckout($inputData)
    {
        $packageCollection = $this->creditPackageRepository->fetch($inputData['packageUid']);

        if (__isEmpty($packageCollection)) {
            //success function
            return $this->engineReaction(2, null, __tr('Package does not exist.'));
        }

        $isCoingateTestMode = 1;
        if (!getAppSettings('use_test_coingate')) {
            $isCoingateTestMode = 2;
        }

        $storeData = [
            'status' => 4,
            'amount' => $inputData['amount'],
            'users__id' => getUserID(),
            'method' => configItem('payments.payment_methods', 6),
            'currency_code' => getAppSettings('currency'),
            'is_test' => $isCoingateTestMode,
            'txn_id' => null,
            '__data' => [
                'rawPaymentData' => json_encode($inputData),
                'packageName' => $inputData['packageName'],
            ],
        ];

        //made new function for getting order id
        if ($orderData = $this->coinGateEngine->storeTransaction($storeData, $packageCollection)) {
            //success function
            $coinGateResponse = $this->coinGateEngine->processCoinGateCheckout($inputData, $orderData);
            if ($coinGateResponse['reaction_code'] == 1) {
                return $this->engineReaction(1, $coinGateResponse);
            }
            return $this->engineReaction(2, null, __tr('Something went wrong'));
        }
        //error response
        return $this->engineReaction(2, null, __tr('Transaction not completed'));
    }

    /**
     * Coingate Callback Data
     *
     * @return  object
     */
    public function coingateCallbackData()
    {
        return $this->coinGateEngine->captureCoinGateData();
    }

    /**
     * Handle Order Payment Stripe Webhook
     *
     * @return  json message
     */
    public function handleOrderPaymentStripeWebhook()
    {
        $paymentWebhookData = $this->stripeEngine->paymentWebhook();
        if ($paymentWebhookData['reaction_code'] == 1) {
            //check reaction code is 1
            $stripeData = $paymentWebhookData['data']['paymentIntent'];
            if ($stripeData['status'] == 'succeeded') {

                if ($this->creditWalletRepository->isAlreadyProcessed($stripeData['id'])) {
                    return $this->engineReaction(2, null, __tr('Already been processed'));
                }

                //store transaction data
                if ($this->storeStripePaymentData($stripeData, $stripeData['metadata']['packageUid'])) {
                    return $this->engineReaction(1, null, __tr('Payment Complete'));
                } else {
                    //payment failed response
                    return $this->engineReaction(2, null, __tr('Payment Failed'));
                }
            }
            //failure response
            return $this->engineReaction(2, null, $stripeData['status']);
        }
        return $this->engineResponse(2, [], __tr('Payment Fail'));
    }

    /**
     * Handle Order Payment RazorPay Webhook
     *
     * @return
     */
    public function handleOrderPaymentRazorPayWebhook()
    {
        $paymentWebhookData = $this->razorpayEngine->paymentWebhook();

        if ($paymentWebhookData['reaction_code'] == 1) {

            $razorpayResponse = $paymentWebhookData['data']['paymentIntent'];
            $paymentWebhookRazorPayData = $razorpayResponse['payload']['payment']['entity'];
            $paymentIntentId = $paymentWebhookRazorPayData['id'];

            if ($this->creditWalletRepository->isAlreadyProcessed($paymentIntentId)) {
                return $this->engineReaction(2, null, __tr('Already been processed'), 200);
            }

            if ($paymentWebhookRazorPayData['captured'] === true) {
                //store transaction data
                if ($this->storePaymentData($paymentWebhookRazorPayData, $paymentWebhookRazorPayData['notes']['packageUid'], 'razorpayPayment', $paymentWebhookRazorPayData['notes']['userId'])) {
                    return $this->engineReaction(1, null, __tr('Payment Complete'), 200);
                }
            } else {
                //payment failed response
                return $this->engineReaction(2, null, __tr('Payment Failed'));
            }
        }
        return $this->engineResponse(2, [], __tr('Payment Fail'));
    }


    /**
     * Process To ShowCreditBonus
     *
     * @return json object
     */
    public function processToUpdateLog()
    {
        $loginLogs = loginLogsModel::where('user_id', getUserID())->first();
        $this->loginLogsRepository->updateLoginLogs($loginLogs, ['updated_at' => Carbon::now()]);

        return $this->engineResponse(1, null);
    }


    /**
     * Prepare Order Process
     *
     * @param   array  $inputData
     *
     * @return  json object
     */
    public function prepareOrderProcess($inputData)
    {
        $packageCollection = $this->creditPackageRepository->fetch($inputData['packageUid']);

        if (__isEmpty($packageCollection)) {
            //check package not empty
            return $this->engineReaction(2, null, __tr('Package does not exist.'));
        }

        $isPaypalCheckoutTestMode = 1;
        if (!getAppSettings('use_test_paypal_checkout')) {
            $isPaypalCheckoutTestMode = 2;
        }
        //store data
        $storeData = [
            'status' => 4,
            'amount' => $inputData['packagePrice'],
            'users__id' => getUserID(),
            'method' => configItem('payments.payment_methods', 4),
            'currency_code' => getAppSettings('currency'),
            'is_test' => $isPaypalCheckoutTestMode,
            'txn_id' => null,
            '__data' => [
                'rawPaymentData' => json_encode($inputData),
                'packageName' => $inputData['packageName'],
            ],
        ];

        //made new function for getting order id
        if ($orderData = $this->coinGateEngine->storeTransaction($storeData, $packageCollection)) {
            //success function
            $paypalResponse = $this->paypalEngine->paypalOrderCreate($inputData, $orderData);
            if ($paypalResponse['reaction_code'] == 1) {
                return $this->engineReaction(1, $paypalResponse['data']);
            }
            return $this->engineReaction(2, null, __tr('Something went wrong'));
        }
        //error response
        return $this->engineReaction(2, null, __tr('Transaction not store'));
    }

    /**
     * Process Capture Paypal Order
     *
     * @param   array  $inputData
     *
     * @return  json   object
     */
    public function processCapturePaypalOrder($inputData)
    {
        return $this->paypalEngine->paypalCaptureOrder($inputData);
    }

    /**
     * Process Debit Credits
     *
     * @param   int  $credits
     *
     * @return  int|bool
     */
    public function processDebitCredits($credits, $creditType)
    {
        // Prepare store credit wallet transaction data
        $storeCreditWalletTransaction = [
            'status' => 1,
            'users__id' => getUserID(),
            'credits' => '-' . $credits,
            'credit_type' => $creditType
        ];

        // check if credit wallet transaction is stored
        if (!$creditWalledId = $this->creditWalletRepository->storeWalletTransaction($storeCreditWalletTransaction)) {
            return false;
        }
        return $creditWalledId->_id;
    }

}
