<?php

/**
 * ManageUserEngine.php - Main component file
 *
 * This file is part of the User component.
 *-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\User;

use App\Yantrana\Base\BaseEngine;
use App\Yantrana\Support\CommonTrait;
use App\Yantrana\Components\Media\MediaEngine;
use App\Yantrana\Components\User\Repositories\UserRepository;
use App\Yantrana\Support\Country\Repositories\CountryRepository;
use App\Yantrana\Components\User\Repositories\ManageUserRepository;
use App\Yantrana\Components\User\Repositories\CreditWalletRepository;
use App\Yantrana\Components\CreditPackage\Repositories\CreditPackageRepository;

class ManageUserEngine extends BaseEngine
{
    /**
     * @var CommonTrait - Common Trait
     */
    use CommonTrait;

    /**
     * @var  ManageUserRepository - ManageUser Repository
     */
    protected $manageUserRepository;

    /**
     * @var  CountryRepository - Country Repository
     */
    protected $countryRepository;

    /**
     * @var  CreditWalletRepository - CreditWallet Repository
     */
    protected $creditWalletRepository;

    /**
     * @var  MediaEngine - MediaEngine
     */
    protected $mediaEngine;

    /**
     * @var  UserRepository - UserRepository
     */
    protected $userRepository;

    /**
     * @var  CreditPackageRepository - CreditPackage Repository
     */
    protected $creditPackageRepository;

    /**
     * Constructor
     *
     * @param  ManageUserRepository  $manageUserRepository - ManageUser Repository
     * @param  CountryRepository  $countryRepository - Country Repository
     * @param  MediaEngine  $mediaEngine - MediaEngine
     * @param  CreditWalletRepository  $creditWalletRepository - CreditWallet Repository
     * @param  UserRepository  $userRepository - User Repository
     * @param  CreditPackageRepository  $creditPackageRepository - CreditPackage Repository
     * @return  void
     *-----------------------------------------------------------------------*/
    public function __construct(ManageUserRepository $manageUserRepository, CountryRepository $countryRepository, CreditWalletRepository $creditWalletRepository, MediaEngine $mediaEngine, UserRepository $userRepository, CreditPackageRepository $creditPackageRepository)
    {
        $this->manageUserRepository = $manageUserRepository;
        $this->countryRepository = $countryRepository;
        $this->creditWalletRepository = $creditWalletRepository;
        $this->mediaEngine = $mediaEngine;
        $this->userRepository = $userRepository;
        $this->creditPackageRepository = $creditPackageRepository;
    }

    /**
     * Prepare User Data table list.
     *
     * @param  int  $status

    * @return  void
    *
     *---------------------------------------------------------------- */
    public function prepareUsersDataTableList($status)
    {
        //prepare user data by status
        $userCollection = $this->manageUserRepository->fetchUsersDataTableSource($status);
        $requireColumns = [
            '_id',
            '_uid',
            'first_name',
            'last_name',
            'full_name',
            'created_at' => function ($key) {
                return formatDate($key['created_at']);
            },
            'status',
            'email',
            'username',
            'dob' => function ($key) {
                //check is not empty
                if (!__isEmpty($key['dob'])) {
                    return __tr($key['dob']);
                }

                return '-';
            },
            'gender',
            'formattedGender' => function ($key) {
                //check is not empty
                if (!__isEmpty($key['gender'])) {
                    return configItem('user_settings.gender', $key['gender']);
                }

                return '-';
            },
            'registered_via' => function ($key) {
                if (!__isEmpty($key['registered_via'])) {
                    return $key['registered_via'];
                }

                return '-';
            },
            'profile_picture' => function ($key) {
                if (isset($key['profile_picture']) and !__isEmpty($key['profile_picture'])) {
                    $imagePath = getPathByKey('profile_photo', ['{_uid}' => $key['_uid']]);

                    return getMediaUrl($imagePath, $key['profile_picture']);
                }

                return noThumbImageURL();
            },
            'profile_url' => function ($key) {
                return route('dashboard_page');
            },
            'user_roles__id',

            'mobile_number' => function ($key) {
                $withoutCodeMblNo = explode('-', ($key['mobile_number']));
                $mobileNumber = isset($withoutCodeMblNo[1]) ? $withoutCodeMblNo[1] : '';
                return $mobileNumber;
            },
            'country_code' => function ($key) {
                $withoutCodeMblNo = explode('-', ($key['mobile_number']));
                $countryCode = explode('0', $withoutCodeMblNo[0]);
                $countryPhnCode = isset($countryCode[1]) ? ($countryCode[1]) : '';
                return $countryPhnCode;

            },
            'preview_url' => function ($key) {
                return route('manage.user.write.update', [
                    'userUid' => $key['_uid'],
                ]);
            },


        ];

        return $this->dataTableResponse($userCollection, $requireColumns);
    }

    /**
     * Prepare User List.
     *
     * @param  array  $inputData
     *
     *---------------------------------------------------------------- */
    public function processAddUser($inputData)
    {
        $transactionResponse = $this->manageUserRepository->processTransaction(function () use ($inputData) {
            // Store user
            $newUser = $this->manageUserRepository->storeUser($inputData);
            // Check if user not stored successfully
            if (!$newUser) {
                return $this->manageUserRepository->transactionResponse(2, ['show_message' => true], __tr('User not added.'));
            }
            $userAuthorityData = [
                'user_id' => $newUser->_id,
                'user_roles__id' => 2,
            ];
            // Add user authority
            if ($this->manageUserRepository->storeUserAuthority($userAuthorityData)) {
                return $this->manageUserRepository->transactionResponse(1, ['show_message' => true], __tr('User added successfully.'));
            }
            // Send failed server error message
            return $this->manageUserRepository->transactionResponse(2, ['show_message' => true], __tr('Something went wrong on server.'));
        });

        return $this->engineReaction($transactionResponse);
    }


    /**
     * Process User Update.
     *
     * @param  string  $userUid
     * @param  array  $inputData
     *
     *---------------------------------------------------------------- */
    public function processUserUpdate($userUid, $inputData)
    {
        //fetch user by id for update
        $userDetails = $this->manageUserRepository->fetchUser($userUid);
        // check if user details exists
        if (__isEmpty($userDetails)) {
            return $this->engineReaction(18, ['show_message' => true], __tr('User does not exists.'));
        }
        $mobileNumber = '0' . $inputData['country_code'] . '-' . $inputData['mobile_number'];

        // Prepare Update User data
        $updateData = [
            'first_name' => $inputData['first_name'],
            'last_name' => $inputData['last_name'],
            'email' => $inputData['email'],
            'username' => $inputData['username'],
            'designation' => array_get($inputData, 'designation'),
            'mobile_number' => $mobileNumber,
            'status' => array_get($inputData, 'status', 2),
        ];

        // check if user updated
        if ($this->manageUserRepository->updateUser($userDetails, $updateData)) {
            // Adding activity log for update user
            activityLog($userDetails->first_name . ' ' . $userDetails->last_name . ' user info updated.');

            return $this->engineReaction(1, ['show_message' => true], __tr('User updated successfully.'));
        }

        return $this->engineReaction(14, ['show_message' => true], __tr('Nothing updated.'));
    }

    /**
     * Process Soft Delete User.
     *
     * @param  string  $userUid
     *
     *---------------------------------------------------------------- */
    public function processSoftDeleteUser($userUid)
    {
        //fetch user by id for delete
        $userDetails = $this->manageUserRepository->fetchUser($userUid);
        // check if user details exists
        if (__isEmpty($userDetails)) {
            return $this->engineReaction(18, ['show_message' => true], __tr('User does not exists.'));
        }
        // Prepare Update User data
        $updateData = [
            'status' => 5,
        ];

        // check if user soft deleted
        if ($this->manageUserRepository->updateUser($userDetails, $updateData)) {
            // Add activity log for user soft deleted
            activityLog($userDetails->first_name . ' ' . $userDetails->last_name . ' user soft deleted.');

            return $this->engineReaction(1, ['userUid' => $userDetails->_uid, 'show_message' => true], __tr('User soft deleted successfully.'));
        }

        return $this->engineReaction(2, ['show_message' => true], __tr('Something went wrong on server.'));
    }

    /**
     * Process Soft Delete User.
     *
     * @param  string  $userUid
     *
     *---------------------------------------------------------------- */
    public function processPermanentDeleteUser($userUid)
    {
        //fetch user by id for Permanent delete
        $userDetails = $this->manageUserRepository->fetchUser($userUid);
        // check if user details exists
        if (__isEmpty($userDetails)) {
            return $this->engineReaction(18, ['show_message' => true], __tr('User does not exists.'));
        }
        // check if user soft deleted first
        if ($userDetails->status != 5) {
            return $this->engineReaction(2, ['show_message' => true], __tr('To delete user permanently you have to soft delete first.'));
        }
        // check if user deleted
        if ($this->manageUserRepository->deleteUser($userDetails)) {
            // Add activity log for user permanent deleted
            activityLog($userDetails->first_name . ' ' . $userDetails->last_name . ' user permanent deleted.');

            return $this->engineReaction(1, ['userUid' => $userDetails->_uid, 'show_message' => true], __tr('User permanent deleted successfully.'));
        }

        return $this->engineReaction(2, ['show_message' => true], __tr('Something went wrong on server.'));
    }

    /**
     * Process Restore User.
     *
     * @param  string  $userUid
     *
     *---------------------------------------------------------------- */
    public function processUserRestore($userUid)
    {
        //fetch user by id for restore
        $userDetails = $this->manageUserRepository->fetchUser($userUid);
        // check if user details exists
        if (__isEmpty($userDetails)) {
            return $this->engineReaction(18, ['show_message' => true], __tr('User does not exists.'));
        }
        // Prepare Update User data
        $updateData = [
            'status' => 1,
        ];

        // check if restore deleted
        if ($this->manageUserRepository->updateUser($userDetails, $updateData)) {
            // Add activity log for user restored
            activityLog($userDetails->first_name . ' ' . $userDetails->last_name . ' user restored.');

            return $this->engineReaction(1, ['userUid' => $userDetails->_uid, 'show_message' => true], __tr('User restore successfully.'));
        }

        return $this->engineReaction(2, ['show_message' => true], __tr('Something went wrong on server.'));
    }

    /**
     * Process Block User.
     *
     * @param  string  $userUid
     *
     *---------------------------------------------------------------- */
    public function processBlockUser($userUid)
    {
        //fetch user by id for restore
        $userDetails = $this->manageUserRepository->fetchUser($userUid);
        // check if user details exists
        if (__isEmpty($userDetails)) {
            return $this->engineReaction(18, ['show_message' => true], __tr('User does not exists.'));
        }
        // Prepare Update User data
        $updateData = [
            'status' => 3,
            // Blocked
        ];

        // check if user blocked
        if ($this->manageUserRepository->updateUser($userDetails, $updateData)) {
            // Add activity log for user blocked
            activityLog($userDetails->first_name . ' ' . $userDetails->last_name . ' user blocked.');

            return $this->engineReaction(1, ['userUid' => $userDetails->_uid, 'show_message' => true], __tr('User blocked successfully.'));
        }

        return $this->engineReaction(2, ['show_message' => true], __tr('Something went wrong on server.'));
    }

    /**
     * Process Unblock User.
     *
     * @param  string  $userUid
     *
     *---------------------------------------------------------------- */
    public function processUnblockUser($userUid)
    {
        //fetch user by id for UnblockUser
        $userDetails = $this->manageUserRepository->fetchUser($userUid);
        // check if user details exists
        if (__isEmpty($userDetails)) {
            return $this->engineReaction(18, ['show_message' => true], __tr('User does not exists.'));
        }
        // Prepare Update User data
        $updateData = [
            'status' => 1,
            // Active
        ];

        // check if user soft deleted
        if ($this->manageUserRepository->updateUser($userDetails, $updateData)) {
            // Add activity log for user unblocked
            activityLog($userDetails->first_name . ' ' . $userDetails->last_name . ' user unblocked.');

            return $this->engineReaction(1, ['userUid' => $userDetails->_uid, 'show_message' => true], __tr('User unblocked successfully.'));
        }

        return $this->engineReaction(2, ['show_message' => true], __tr('Something went wrong on server.'));
    }

    /**
     * Prepare User edit data.
     *
     * @param  array  $userUid
     *
     *---------------------------------------------------------------- */
    public function prepareUserDetails($userUid)
    {
        //fetch user by id for UnblockUser
        $user = $this->manageUserRepository->fetchUser($userUid);
        // check if user details exists
        if (__isEmpty($user)) {
            return $this->engineReaction(18, ['show_message' => true], __tr('User does not exists.'));
        }

        $userDetails = [
            'full_name' => $user->first_name . ' ' . $user->last_name,
            'email' => $user->email,
            'username' => $user->username,
            'designation' => $user->designation,
            'mobile_number' => $user->mobile_number,
        ];

        return $this->engineReaction(1, [
            'userDetails' => $userDetails,
        ]);
    }

    /**
     * get manage  user transaction list data.
     *
     * @param $userUid
     * @return object
     *---------------------------------------------------------------- */
    public function getUserTransactionList($userUid)
    {
        //fetch user by id
        $user = $this->manageUserRepository->fetchUser($userUid);
        //if user not exist
        if (__isEmpty($user)) {
            return $this->engineReaction(2, null, __tr('User does not exist.'));
        }
        //get transaction list data
        $transactionCollection = $this->creditWalletRepository->fetchUserTransactionListData($user->_id);

        $requireColumns = [
            '_id',
            '_uid',
            'created_at' => function ($key) {
                return formatDate($key['created_at']);
            },
            'updated_at' => function ($key) {
                return formatDate($key['updated_at']);
            },
            'status',
            'formattedStatus' => function ($key) {
                return configItem('payments.status_codes', $key['status']);
            },
            'amount',
            'formattedAmount' => function ($key) {
                return priceFormat($key['amount'], true, true);
            },
            'method',
            'currency_code',
            'is_test',
            'formattedIsTestMode' => function ($key) {
                return configItem('payments.payment_checkout_modes', $key['is_test']);
            },
            'credit_type',
            'formattedCreditType' => function ($key) {
                return configItem('payments.credit_type', $key['credit_type']);
            },
            '__data',
            'packageName' => function ($key) {
                //check is not Empty
                if (!__isEmpty($key['__data']) and !__isEmpty($key['__data']['packageName'])) {
                    return $key['__data']['packageName'];
                }

                return 'N/A';
            },
        ];

        return $this->dataTableResponse($transactionCollection, $requireColumns);
    }

    /**
     * Add Allocated Credits For User
     *
     * @param  array  $inputData
     * @return  json object
     */
    public function allocatedCreditsForUser($inputData)
    {
        $transactionResponse = $this->creditWalletRepository->processTransaction(function () use ($inputData) {
            $userDetails = $this->userRepository->fetchByUid($inputData['userId']);

            $storeCreditWalletTransaction = [
                'status' => 1,
                'users__id' => $userDetails->_id,
                'credits' => $inputData['allocate_credits'],
                'credit_type' => 1, // allocate bonus credits
                'description' => 'admin_credit'
            ];

            if ($this->creditWalletRepository->storeWalletTransaction($storeCreditWalletTransaction)) {
                return $this->creditWalletRepository->transactionResponse(1, [
                    'show_message' => true,
                ], __tr('Credits store successfully.'));
            }

            return $this->creditWalletRepository->transactionResponse(2, ['show_message' => true], __tr('Fail to store credits.'));
        });

        return $this->engineReaction($transactionResponse);
    }
}
