<?php
/**
* ManageUserRepository.php - Repository file
*
* This file is part of the User component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\User\Repositories;

use App\Yantrana\Base\BaseRepository;
use App\Yantrana\Components\User\Models\CreditWalletTransaction;
use App\Yantrana\Components\User\Models\User;
use App\Yantrana\Components\User\Models\UserAuthorityModel;
use App\Yantrana\Components\User\Models\UserProfile;
use DB;

class ManageUserRepository extends BaseRepository
{
    /**
     * Fetch User
     *
     * @param    int || string $status
     * @return    eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchUser($userId)
    {
        if (is_numeric($userId)) {
            return User::where('_id', $userId)->first();
        }

        return User::where('_uid', $userId)->first();
    }

    /**
     * Fetch List of users
     *
     * @param    int || int $status
     * @return    eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchUsersDataTableSource($status)
    {
        $dataTableConfig = [
            'searchable' => [
                'first_name',
                'last_name',
                'mobile_number',
                'username',
                'email',
                'full_name' => DB::raw("CONCAT(COALESCE(first_name, ''), ' ', COALESCE(last_name, ''))"),
                'user_profiles.dob',
                'user_profiles.gender',
            ],
        ];

        return User::leftJoin('user_profiles', 'users._id', '=', 'user_profiles.users__id')
            ->leftJoin('user_authorities', 'users._id', '=', 'user_authorities.users__id')
            ->where('users.status', $status)
            ->select(
                __nestedKeyValues([
                    'users' => [
                        '_id',
                        '_uid',
                        'first_name',
                        'last_name',
                        'created_at',
                        'status',
                        'email',
                        'username',
                        'registered_via',
                        'mobile_number',
                        DB::raw("CONCAT(COALESCE(first_name, ''), ' ', COALESCE(last_name, '')) AS full_name"),
                    ],
                    'user_profiles' => [
                        'profile_picture',
                        'gender',
                        'dob',
                    ],
                    'user_authorities' => [
                        '_id AS user_authority_id',
                        'user_roles__id',
                    ],
                ])
            )
            ->dataTables($dataTableConfig)
            ->toArray();
    }

    /**
     * Fetch List of users
     *
     * @param    int || int $status
     * @return    eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchList($status)
    {
        return User::where('status', $status)->get();
    }

    /**
     * Store User.
     *
     * @param  array  $storeData
     *
     *-----------------------------------------------------------------------*/
    public function storeUser($storeData)
    {
        $keyValues = [
            'email',
            'password' => bcrypt($storeData['password']),
            'status' => array_get($storeData, 'status', 2),
            'first_name',
            'last_name',
            'username',
            'designation',
            'mobile_number',
        ];
        // Get Instance of user model
        $userModel = new User();
        // Store New User
        if ($userModel->assignInputsAndSave($storeData, $keyValues)) {
            activityLog($userModel->first_name . ' ' . $userModel->last_name . ' user created.');

            return $userModel;
        }

        return false;
    }

    /**
     * Store User Authority.
     *
     * @param  array  $userAuthorityData
     *
     *-----------------------------------------------------------------------*/
    public function storeUserAuthority($userAuthorityData)
    {
        $keyValues = [
            'status' => 1,
            'users__id' => $userAuthorityData['user_id'],
            'user_roles__id' => $userAuthorityData['user_roles__id'],
        ];
        // Get Instance of user authority model
        $userAuthorityModel = new UserAuthorityModel();
        // Store New User Authority
        if ($userAuthorityModel->assignInputsAndSave($userAuthorityData, $keyValues)) {
            return $userAuthorityModel;
        }

        return false;
    }

    /**
     * Update User.
     *
     * @param  object  $user
     * @return bool
     *---------------------------------------------------------------- */
    public function updateUser($user, $updateData)
    {
        // Check if information updated
        if ($user->modelUpdate($updateData)) {
            return true;
        }

        return false;
    }

    /**
     * Delete User
     *
     * @param  object  $user
     * @return bool
     *---------------------------------------------------------------- */
    public function deleteUser($user)
    {
        // Check if information deleted
        if ($user->delete()) {
            activityLog($user->first_name . ' ' . $user->last_name . ' user deleted.');

            return true;
        }

        return false;
    }

    /**
     * get User Profile.
     *
     * @param int userID
     *
     *-----------------------------------------------------------------------*/
    public function fetchUserProfile($userID)
    {
        return UserProfile::where('users__id', '=', $userID)->first();
    }

    /**
     * Store user profile
     *
     * @param  array  $inputData
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function storeUserProfile($inputData)
    {
        $keyValues = [
            'users__id' => $inputData['users__id'],
            'countries__id' => array_get($inputData, 'countries__id'),
            'gender' => array_get($inputData, 'gender'),
            'dob' => array_get($inputData, 'dob'),
        ];

        $userProfile = new UserProfile();
        // check if user profile stored successfully
        if ($userProfile->assignInputsAndSave($inputData, $keyValues)) {
            return $userProfile;
        }

        return false;
    }

    /**
     * Update User Profile
     *
     * @param  object  $userProfile
     * @param  array  $updateData
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function updateUserProfile($userProfile, $updateData)
    {
        if ($userProfile->modelUpdate($updateData)) {
            return true;
        }

        return false;
    }

}
