<?php

/**
 * UserRepository.php - Repository file
 *
 * This file is part of the User component.
 *-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\User\Repositories;

use App\Yantrana\Base\BaseRepository;
use App\Yantrana\Components\User\Blueprints\UserRepositoryBlueprint;
use App\Yantrana\Components\User\Models\CreditWalletTransaction;
use App\Yantrana\Components\User\Models\LoginAttempt;
use App\Yantrana\Components\User\Models\PasswordReset;
use App\Yantrana\Components\User\Models\SocialAccess;
use App\Yantrana\Components\User\Models\User as UserModel;
use App\Yantrana\Components\User\Models\UserAuthorityModel;
use App\Yantrana\Components\User\Models\UserProfile;
use App\Yantrana\Support\Utils;
use DB;
use Request;
use Str;
use YesSecurity;

class UserRepository extends BaseRepository implements UserRepositoryBlueprint
{
    /**
     * primary model instance
     * eg. YourModelModel::class;
     *
     * @var object
     */
    protected $primaryModel = UserModel::class;
    /**
     * Constructor.
     *
     * @param  UserModel  $user - User Model
     *-----------------------------------------------------------------------*/
    public function __construct(UserModel $user = null)
    {
        //if it is not null
        if ($user) {
            $user = new UserModel();
        }
        $this->user = $user;
    }

    /**
     * Fetch user by id.
     *
     * @param  number  $userID
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchByID($userID)
    {
        return UserModel::find($userID);
    }

    /**
     * Fetch user with profile by id.
     *
     * @param  number  $userID
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchWithProfile($userId)
    {
        return UserModel::where('users._id', $userId)
            ->select(
                __nestedKeyValues([
                    'users' => ['*'],
                ])
            )
            ->first();
    }

    /**
     * Fetch user with profile by id.
     *
     * @param  number  $userID
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchUsersWithProfiles($userIds)
    {
        return UserModel::whereIn('users._id', $userIds)
            ->join('user_authorities', 'users._id', '=', 'user_authorities.users__id')
            ->select(
                \__nestedKeyValues([
                    'users' => [
                        '_id AS user_id',
                        '_uid AS user_uid',
                        'first_name',
                        'last_name',
                        'username',
                        DB::raw("CONCAT(COALESCE(first_name, ''), ' ', COALESCE(last_name, '')) AS full_name"),
                    ],
                    'user_authorities' => [
                        'users__id AS user_authority_user_id',
                        'updated_at',
                    ],
                ])
            )
            ->get();
    }

    /**
     * fetch user data.
     *
     * @param  int  $idOrUid
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetch($idOrUid)
    {
        //check is numeric
        if (is_numeric($idOrUid)) {
            return UserModel::leftJoin('user_authorities', 'users._id', '=', 'user_authorities.users__id')->select(__nestedKeyValues([
                'users.*',
                'user_authorities.user_roles__id as user_role_id',
                'user_authorities' => [
                    'updated_at as userAuthorityUpdatedAt',
                ],
            ]))->where('users._id', $idOrUid)->first();
        } else {
            return UserModel::leftJoin('user_authorities', 'users._id', '=', 'user_authorities.users__id')->select(__nestedKeyValues([
                'users.*',
                'user_authorities.user_roles__id as user_role_id',
                'user_authorities' => [
                    'updated_at as userAuthorityUpdatedAt',
                ],
            ]))->where('users._uid', $idOrUid)->first();
        }
    }

    /**
     * Fetch the record of User
     *
     * @param  int || string $email
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchByEmail($email)
    {
        return UserModel::leftJoin('user_authorities', 'users._id', '=', 'user_authorities.users__id')
            ->select(
                __nestedKeyValues([
                    'users.*',
                    'user_authorities' => [
                        '_id as user_authority_id',
                        'updated_at as user_authority_updated_at',
                    ],
                ])
            )->where('email', $email)->first();
    }

    /**
     * Fetch the record of User
     *
     * @param  int || string $username
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchByName($username)
    {
        return UserModel::where('username', $username)->first();
    }

    /**
     * Fetch the record of User
     *
     * @param  int || string $username
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchByUsername($username, $checkUserStatusForAdmin = false)
    {
        return UserModel::leftJoin('user_authorities', 'users._id', '=', 'user_authorities.users__id')
            ->select(
                __nestedKeyValues([
                    'users.*',
                    'user_authorities' => [
                        '_id as userAuthorityId',
                        'updated_at as userAuthorityUpdatedAt',
                    ],
                ])
            )
            ->where('users.username', $username)
            ->where(function ($query) use ($checkUserStatusForAdmin) {
                if ($checkUserStatusForAdmin and !isAdmin()) {
                    $query->where('users.status', '=', 1);
                } elseif (!$checkUserStatusForAdmin) {
                    $query->where('users.status', '=', 1);
                }
            })
            ->first();
    }

    /**
     * Fetch the record of User
     *
     * @param  int || string $email
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchByEmailOrUsername($emailOrUsername)
    {
        $query = UserModel::leftJoin('user_authorities', 'users._id', '=', 'user_authorities.users__id')
        ->select(
            __nestedKeyValues([
                'users.*',
                'user_authorities' => [
                    '_id as user_authority_id',
                    'updated_at as user_authority_updated_at',
                ],
            ])
        );

        $query->where(function ($whereQuery) use ($emailOrUsername) {
            if (getAppSettings('allow_user_login_with_mobile_number')) {
                $whereQuery->where(
                    str_contains($emailOrUsername, '@') ? 'email' : (is_numeric(substr($emailOrUsername, -10)) ? 'mobile_number' : 'username'),
                    $emailOrUsername
                );
            } else {
                $whereQuery->where(
                    str_contains($emailOrUsername, '@') ? 'email' : 'username',
                    $emailOrUsername
                );
            }


        });
        return $query->first();
    }

    /**
     * Clear login attempts.
     *---------------------------------------------------------------- */
    public function clearLoginAttempts()
    {
        LoginAttempt::where('ip_address', Request::getClientIp())->delete();
    }

    /**
     * Fetch user authority
     *
     * @param int || string $value
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchUserAuthority($userId)
    {
        return UserAuthorityModel::where('users__id', $userId)->first();
    }

    /**
     * Fetch active user using email address & return response.
     *
     * @param  string  $email
     * @param  bool  $selectRecord
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchActiveUserByEmail($email)
    {
        return UserModel::where([
            'status' => 1,      // active status
            'email' => $email,
        ])->first();
    }

    /**
     * Fetch active user using mobile number & return response.
     *
     * @param  int  $emailOrUsernameOrNumber
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchActiveUserByEmailOrUsernameOrNumber($emailOrUsernameOrNumber)
    {
        $query = UserModel::leftJoin('user_authorities', 'users._id', '=', 'user_authorities.users__id')
        ->select(
            __nestedKeyValues([
                'users.*',
                'user_authorities' => [
                    '_id as user_authority_id',
                    'updated_at as user_authority_updated_at',
                ],
            ])
        );

        $query->where(function ($whereQuery) use ($emailOrUsernameOrNumber) {
            $whereQuery->where(
                str_contains($emailOrUsernameOrNumber, '@') ? 'email' : (is_numeric(substr($emailOrUsernameOrNumber, -10)) ? 'mobile_number' : 'username'),
                $emailOrUsernameOrNumber
            );
        });
        return $query->first();
    }

    /**
     * Check the account already exists
     *
     * @param  string  $accountId
     * @return Eloquent collection object
     *-----------------------------------------------------------------------*/
    public function checkAccountId($accountId)
    {
        return  SocialAccess::where('account_id', $accountId)
            ->first();
    }

    /**
     * Prepare data for store new social user
     *
     * @param  object  $input
     * @return bool|object
     *---------------------------------------------------------------- */
    public function storeSocialUser($input)
    {
        $keyValues = [
            'email' => strtolower($input['email']),
            'password' => 'NO_PASSWORD',
            'status' => 1, // Active
            'first_name' => $input['fname'],
            'last_name' => $input['lname'],
            'username' => $input['username'],
            'designation' => 'User',
            'registered_via' => $input['provider'],
        ];

        $newUser = new UserModel();

        // Check if new User added
        if ($newUser->assignInputsAndSave($input, $keyValues)) {
            return $newUser;
        }

        return false;   // on failed
    }

    /**
     * Fetch password reminder count.
     *
     * @param  string  $reminderToken
     * @param  string  $email
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchPasswordReminderCount($reminderToken, $email = null)
    {
        return PasswordReset::where(function ($query) use ($reminderToken, $email) {
            $query->where('token', $reminderToken);

            if (!__isEmpty($email)) {
                $query->where('email', $email);
            }
        })->count();
    }

    /**
     * Update login attempts.
     *---------------------------------------------------------------- */
    public function updateLoginAttempts()
    {
        $ipAddress = Request::getClientIp();
        $keyValues = ['attempts', 'ip_address', 'created_at'];
        $loginAttempt = LoginAttempt::where('ip_address', $ipAddress)
            ->first();

        // Check if login attempt record exist for this ip address
        if (!empty($loginAttempt)) {
            $storeData = ['attempts' => $loginAttempt->attempts + 1];
            // Store New User
            if ($loginAttempt->assignInputsAndSave($storeData, $keyValues)) {
                return true;
            } else {
                return false;
            }
        } else {
            $newLoginAttempt = new LoginAttempt();
            $storeData = [
                'ip_address' => $ipAddress,
                'attempts' => 1,
                'created_at' => getCurrentDateTime(),
            ];
            // Store New User
            if ($newLoginAttempt->assignInputsAndSave($storeData, $keyValues)) {
                return true;
            } else {
                return false;
            }
        }
    }

    /**
     * Store User.
     *
     * @param  array  $storeData
     *
     *-----------------------------------------------------------------------*/
    public function storeUser($storeData)
    {
        $mobileNumber = '0' . $storeData['country_code'] . '-' . $storeData['mobile_number'];
        $keyValues = [
            'email',
            'password' => bcrypt($storeData['password']),
            'status' => $storeData['status'],
            'first_name',
            'last_name',
            'username' => Str::slug($storeData['username']),
            'mobile_number' => $mobileNumber,
        ];

        //check if mobile app request then set otp else set token
        if (isMobileAppRequest()) {
            $keyValues['remember_token'] = Utils::generateStrongPassword(4, false, 'ud');
        } else {
            $keyValues['remember_token'] = YesSecurity::generateUid();
        }

        // Get Instance of user model
        $userModel = new UserModel();
        // Store New User
        if ($userModel->assignInputsAndSave($storeData, $keyValues)) {
            return $userModel;
        }

        return false;
    }

    /**
     * Update password.
     *
     * @param  object  $user
     * @param  string  $newPassword
     * @return bool
     *---------------------------------------------------------------- */
    public function updatePassword($user, $newPassword)
    {
        $user->password = bcrypt($newPassword);

        if ($user->save()) {
            return true;
        }

        return false;
    }

    /**
     * Delete old password reminder.
     *
     * @param  string  $email
     * @return bool
     *---------------------------------------------------------------- */
    public function deleteOldPasswordReminder($email)
    {
        $expiryTime = time() - config('__tech.account.password_reminder_expiry')
            * 60 * 60;

        return PasswordReset::where('email', $email)
            ->orWhere(
                DB::raw('UNIX_TIMESTAMP(created_at)'),
                '<',
                $expiryTime
            )
            ->delete();
    }

    /**
     * Delete old password reminder.
     *
     * @param  string  $email
     * @return bool
     *---------------------------------------------------------------- */
    public function appDeleteOldPasswordReminder($email)
    {
        $expiryTime = time() - config('__tech.account.app_password_reminder_expiry')
            * 60 * 60;

        return PasswordReset::where('email', $email)
            ->orWhere(
                DB::raw('UNIX_TIMESTAMP(created_at)'),
                '<',
                $expiryTime
            )
            ->delete();
    }

    /**
     * Store password reminder & return response.
     *
     * @param  string  $email
     * @param  string  $token
     * @return bool
     *---------------------------------------------------------------- */
    public function storePasswordReminder($storeData)
    {
        $passwordReminder = new PasswordReset();

        $keyValues = ['email', 'token', 'created_at'];

        // Store New User
        if ($passwordReminder->assignInputsAndSave($storeData, $keyValues)) {
            return true;
        }

        return false;
    }

    /**
     * Store User Authority.
     *
     * @param  array  $userAuthorityData
     *
     *-----------------------------------------------------------------------*/
    public function storeUserAuthority($userAuthorityData)
    {
        $keyValues = [
            'status' => 1,
            'users__id' => $userAuthorityData['user_id'],
            'user_roles__id' => $userAuthorityData['user_roles__id'],
        ];
        // Get Instance of user authority model
        $userAuthorityModel = new UserAuthorityModel();
        // Store New User Authority
        if ($userAuthorityModel->assignInputsAndSave($userAuthorityData, $keyValues)) {
            return $userAuthorityModel;
        }

        return false;
    }

    /**
     * Fetch never activated user.
     *
     * @param  string  $userUid
     * @param  string  $activationKey
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchNeverActivatedUser($userUid)
    {
        return UserModel::where([
            '_uid' => $userUid,
            'status' => 4,  // never activated
        ])
            ->first();
    }

    /**
     * Fetch never activated user.
     *
     * @param  string  $userUid
     * @param  string  $activationKey
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchNeverActivatedUserForApp($email)
    {
        return UserModel::where([
            'email' => $email,
            'status' => 4,  // never activated
        ])->first();
    }

    /**
     * Fetch never activated user.
     *
     * @param  number  $userID
     * @param  string  $activationKey
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchPasswordReset($activationKey)
    {
        $passwordReminder = new PasswordReset();

        return $passwordReminder->where([
            'token' => $activationKey,
        ])
            ->first();
    }

    /**
     * Activate user by updating its status information.
     *
     * @param  object  $user
     * @return bool|object
     *---------------------------------------------------------------- */
    public function updateUser($user, $updateData)
    {
        // Check if information updated
        if ($user->modelUpdate($updateData)) {
            return $user;
        }

        return false;
    }

    /**
     * Update User Authority by updating its status information.
     *
     * @param  object  $user
     * @return bool
     *---------------------------------------------------------------- */
    public function updateUserAuthority($userAuthority, $updateData)
    {
        // Check if information updated
        if ($userAuthority->modelUpdate($updateData)) {
            return true;
        }

        return false;
    }

    /**
     * Reset password.
     *
     * @param  object  $user
     * @param  string  $newPassword
     * @return bool
     *---------------------------------------------------------------- */
    public function resetPassword($user, $newPassword)
    {
        $user->password = bcrypt($newPassword);

        if ($user->save()) {  // Check for if user password reset
            $this->deleteOldPasswordReminder($user->email);

            return true;
        }

        return false;
    }

    /**
     * Store Credit Wallet.
     *
     * @param  array  $storeData
     *
     *-----------------------------------------------------------------------*/
    public function storeCreditWalletTransaction($storeData)
    {
        //wallet transaction store data
        $keyValues = [
            'status',
            'users__id',
            'credits',
            'credit_type',
        ];

        $creditWalletTransaction = new CreditWalletTransaction();
        // Check if new User added
        if ($creditWalletTransaction->assignInputsAndSave($storeData, $keyValues)) {
            return $creditWalletTransaction->_id;
        }

        return false;
    }

    /**
     * delete block user data.
     *
     * @param  array  $storeData
     *
     *-----------------------------------------------------------------------*/
    public function deleteBlockUser($blockUserData)
    {
        // Check if block user deleted
        if ($blockUserData->delete()) {
            return  true;
        }

        return false;
    }


    /**
     * Fetch all admin ids
     *
     *
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchAdminIds()
    {
        return UserModel::leftJoin('user_authorities', 'users._id', '=', 'user_authorities.users__id')
            ->where('user_authorities.user_roles__id', 1) // admin role id
            ->get()->pluck('_id')->toArray();
    }


    /**
     * Store user profile
     *
     * @param  array  $inputData
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function storeUserProfile($inputData)
    {
        $keyValues = [
            'users__id' => $inputData['users__id'],
            'gender' => array_get($inputData, 'gender'),
            'dob' => array_get($inputData, 'dob'),
            'status' => array_get($inputData, 'status'),
        ];

        $userProfile = new UserProfile();
        // check if user profile stored successfully
        if ($userProfile->assignInputsAndSave($inputData, $keyValues)) {
            return $userProfile;
        }

        return false;
    }

    /**
     * Update User Profile
     *
     * @param  object  $userProfile
     * @param  array  $updateData
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function updateProfile($userProfile, $updateData)
    {
        if ($userProfile->modelUpdate($updateData)) {
            return true;
        }

        return false;
    }

    /**
     * Delete User
     *
     * @param  object  $user
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function deleteUser($user)
    {
        if ($user->delete()) {
            return true;
        }

        return false;
    }

    /**
     * fetch By Uid
     *
     * @param  mix  $userUid
     * @return  eloquent collection object
     */
    public function fetchByUid($userUid)
    {
        return $this->user::where('_uid', $userUid)->first();
    }
}
