<?php

/**
 * UserEngine.php - Main component file
 *
 * This file is part of the User component.
 *-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\User;

use Auth;
use Hash;
use Session;
use YesSecurity;
use YesTokenAuth;
use Carbon\Carbon;
use Tzsk\Sms\Facades\Sms;
use App\Yantrana\Support\Utils;
use App\Yantrana\Base\BaseEngine;
use App\Yantrana\Base\BaseMailer;
use Illuminate\Support\Facades\URL;
use Illuminate\Support\Facades\Http;
use App\Yantrana\Support\CommonTrait;
use Illuminate\Http\Client\RequestException;
use App\Yantrana\Components\Media\MediaEngine;
use App\Yantrana\Components\User\CreditWalletEngine;
use App\Yantrana\Components\User\Repositories\UserRepository;
use App\Yantrana\Support\Country\Repositories\CountryRepository;
use App\Yantrana\Components\User\Repositories\ReferralRepository;
use App\Yantrana\Components\User\Repositories\LoginLogsRepository;
use App\Yantrana\Components\User\Repositories\CreditWalletRepository;
use App\Yantrana\Components\UserSetting\Repositories\UserSettingRepository;

class UserEngine extends BaseEngine
{
    /**
     * @var CommonTrait - Common Trait
     */
    use CommonTrait;

    /**
     * @var UserRepository - User Repository
     */
    protected $userRepository;

    /**
     * @var BaseMailer - Base Mailer
     */
    protected $baseMailer;

    /**
     * @var  UserSettingRepository - UserSetting Repository
     */
    protected $userSettingRepository;

    /**
     * @var  CreditWalletRepository - CreditWallet Repository
     */
    protected $creditWalletRepository;

    /**
     * @var  CountryRepository - Country Repository
     */
    protected $countryRepository;

    /**
     * @var  MediaEngine - Media Engine
     */
    protected $mediaEngine;

    /**
     * @var  LoginLogsRepository - LoginLogs Repository
     */
    protected $loginLogsRepository;

    /**
     * @var  CreditWalletEngine - CreditWallet Engine
     */
    protected $creditWalletEngine;

    /**
     * @var  ReferralRepository - Referral Repository
     */
    protected $referralRepository;

    /**
     * Constructor.
     *
     * @param  CreditWalletRepository  $creditWalletRepository - CreditWallet Repository
     * @param  UserRepository  $userRepository  - User Repository
     * @param  BaseMailer  $baseMailer  - Base Mailer
     * @param  UserSettingRepository  $userSettingRepository - UserSetting Repository
     * @param  CountryRepository  $countryRepository - Country Repository
     * @param  LoginLogsRepository  $loginLogsRepository - LoginLogs Repository
     * @param  CreditWalletEngine  $creditWalletEngine - CreditWallet Engine
     * @param  ReferralRepository  $referralRepository - Referral Repository
     *
     *-----------------------------------------------------------------------*/
    public function __construct(
        BaseMailer $baseMailer,
        UserRepository $userRepository,
        UserSettingRepository $userSettingRepository,
        CreditWalletRepository $creditWalletRepository,
        CountryRepository $countryRepository,
        MediaEngine $mediaEngine,
        LoginLogsRepository $loginLogsRepository,
        CreditWalletEngine $creditWalletEngine,
        ReferralRepository $referralRepository
    ) {
        $this->baseMailer = $baseMailer;
        $this->userRepository = $userRepository;
        $this->userSettingRepository = $userSettingRepository;
        $this->creditWalletRepository = $creditWalletRepository;
        $this->countryRepository = $countryRepository;
        $this->mediaEngine = $mediaEngine;
        $this->loginLogsRepository = $loginLogsRepository;
        $this->creditWalletEngine = $creditWalletEngine;
        $this->referralRepository = $referralRepository;
    }

    /**
     * Process user login request using user repository & return
     * engine reaction.
     *
     * @param  array  $input
     * @return array
     *---------------------------------------------------------------- */
    public function processLogin($input)
    {
        //check recaptcha if allow recaptcha
        if (getAppSettings('allow_recaptcha') && !$this->checkRecaptcha($input)) {
            return $this->engineReaction(2, ['show_message' => true], __tr('Invalid Re-captcha.'));
        }

        //check is email or username
        $user = $this->userRepository->fetchByEmailOrUsername($input['email_or_username']);

        // Check if empty then return error message
        if (__isEmpty($user)) {
            return $this->engineReaction(2, ['show_message' => true], __tr('You are not a member of the system, Please go and register first , then you can proceed for login.'));
        }

        //collect login credentials
        $loginCredentials = [
            'email' => $user->email,
            'password' => $input['password'],
        ];

        //check user status not equal to 1
        if ($user->status != 1) {
            return $this->engineReaction(2, ['show_message' => true], __tr('Your account currently __status__, Please contact to administrator.', ['__status__' => configItem('status_codes', $user->status)]));
        }

        //get remember me data
        $remember_me = (isset($input['remember_me']) and $input['remember_me'] == 'on') ? true : false;

        // Process for login attempt
        if (Auth::attempt($loginCredentials, $remember_me)) {
            // Clear login attempts of ip address
            $this->userRepository->clearLoginAttempts();
            if (!$this->loginLogsRepository->fetchIt(['user_id' => $user->_id])) {
                $this->loginLogsRepository->createLoginLog($user);
            }
            //if mobile request
            if (isMobileAppRequest()) {
                //issue new token
                $authToken = YesTokenAuth::issueToken([
                    'aud' => $user->_id,
                    'uaid' => $user->user_authority_id,
                ]);

                return $this->engineReaction(1, [
                    'auth_info' => getUserAuthInfo(1),
                    'access_token' => $authToken,
                ], 'Welcome, you are logged in successfully.');
            }

            return $this->engineReaction(1, [
                'auth_info' => getUserAuthInfo(1),
                'intendedUrl' => Session::get('intendedUrl'),
                'show_message' => true,
            ], __tr('Welcome, you are logged in successfully.'));
        }

        // Store every login attempt.
        $this->userRepository->updateLoginAttempts();

        return $this->engineReaction(2, ['show_message' => true], __tr('Authentication failed, please check your credentials & try again.'));
    }

    /**
     * Get Auth Info
     *
     * @return void
     */
    public function authInfo()
    {
        //authorised user info
        return $this->engineReaction(1, [
            'auth_info' => getUserAuthInfo(1),
            'show_message' => false,
        ]);
    }

    /**
     * Process logout request
     *
     * @return array
     *---------------------------------------------------------------- */
    public function processLogout()
    {
        //check session
        if (Session::has('intendedUrl')) {
            Session::forget('intendedUrl');
        }
        //check session
        if (isset($_SESSION['CURRENT_LOCALE'])) {
            $_SESSION['CURRENT_LOCALE'] = null;
        }
        //get user id
        $userId = Auth::user()->_id;

        //fetch user authority
        $userAuthority = $this->userRepository->fetchUserAuthority($userId);

        //update data
        $updateData = [
            'updated_at' => Carbon::now()->subMinutes(2)->toDateTimeString(),
        ];

        // Check for if new email activation store
        if ((!__isEmpty($userAuthority)) and $this->userRepository->updateUserAuthority($userAuthority, $updateData)) {
            Auth::logout();
        } else {
            Auth::logout();
        }

        return $this->engineReaction(2, null, __tr('User logout failed.'));
    }

    /**
     * Prepare Signup view data
     *-----------------------------------------------------------------------*/
    public function prepareSignupViewData()
    {
        //fetch refferrals cookies data
        $referralCookieData = request()->cookie('user_referred_by');
        $refUserFullName = null;
        $refUsername = null;
        //check if referral cockies data available
        if($referralCookieData) {
            //fetch reffered by user name
            $referredByUsername = $this->userRepository->fetchByUsername($referralCookieData);
            if(!__isEmpty($referredByUsername)) {
                $referredByUserData = $referredByUsername->only([
                    'first_name',
                    'last_name',
                    'username'
                ]);
                //concatinate users full name
                $refUserFullName = $referredByUserData['first_name'] . " " . $referredByUserData['last_name'];
                //concatinate username
                $refUsername = $referredByUserData['first_name'] . " " . $referredByUserData['last_name'];
            }
        }
        return $this->engineSuccessResponse([
            'genders' => configItem('user_settings.gender'),
            'referredByUsername' => $refUsername,
            'referredByFullName' => $refUserFullName
        ]);
    }

    /**
     * User Sign prepare
     *-----------------------------------------------------------------------*/
    public function prepareSignupData()
    {
        //fetch all genders
        $allGenders = configItem('user_settings.gender');
        $genders = [];
        foreach ($allGenders as $key => $value) {
            $genders[] = [
                'id' => $key,
                'value' => $value,
            ];
        }

        return $this->engineReaction(1, [
            'privacy_policy_url' => getAppSettings('privacy_policy_url'),
            'genders' => $genders,
            'country_phone_codes' => getCountryPhoneCodes(),
        ]);
    }

    /**
     * User Sign Process.
     *
     * @param  array  $inputData
     *
     *-----------------------------------------------------------------------*/
    public function userSignUpProcess($inputData)
    {
        // Verify recaptcha
        if (getAppSettings('allow_recaptcha') && !$this->checkRecaptcha($inputData)) {
            return $this->engineReaction(2, ['show_message' => true], __tr('Invalid Re-captcha.'));
        }

        $transactionResponse = $this->userRepository->processTransaction(function () use ($inputData) {
            //check if activation required for new user
            $activationRequiredForNewUser = getAppSettings('activation_required_for_new_user');
            $inputData['status'] = 1; // Active
            // check if activation is required for new user
            if ($activationRequiredForNewUser) {
                $inputData['status'] = 4; // Never Activated
            }
            // Store user
            $newUser = $this->userRepository->storeUser($inputData);
            // Check if user not stored successfully
            if (!$newUser) {
                return $this->userRepository->transactionResponse(2, ['show_message' => true], __tr('Failed to signup'));
            }
            //create array for authorize user
            $userAuthorityData = [
                'user_id' => $newUser->_id,
                'user_roles__id' => 2,
            ];
            // Add user authority
            if ($this->userRepository->storeUserAuthority($userAuthorityData)) {
                $profileData = [
                    'users__id' => $newUser->_id,
                    'gender' => $inputData['gender'],
                    'dob' => $inputData['dob'],
                    'status' => 1,
                ];

                //store profile
                if ($this->userRepository->storeUserProfile($profileData)) {

                    //check enable bonus credits for new user
                    if (getAppSettings('enable_bonus_credits')) {
                        //store user credit transaction data
                        $this->userRepository->storeCreditWalletTransaction([
                            'status' => 1,
                            'users__id' => $newUser->_id,
                            'credits' => getAppSettings('number_of_credits'),
                            'credit_type' => 6, // Welcome Bonus
                        ]);
                    }

                    if(isset($inputData['user_referred_by'])) {
                        $referredByUser = $this->userRepository->fetchByName($inputData['user_referred_by']);
                        if(!__isEmpty($referredByUser)) {
                            // User not found
                            $referralData = [
                                'users__id' => $newUser->_id,
                                'by_users__id' => $referredByUser->_id,
                            ];
                            //store referral information
                            $this->referralRepository->storeUserReferral($referralData);
                        }
                    }
                    // give referral bonus if applicable on signup
                    if(!$activationRequiredForNewUser and (getAppSettings('give_bonus', 'select_type') == 1)) {
                        storeReferralWalletTransaction($newUser->_id);
                    }
                    //check activation required for new users
                    if ($activationRequiredForNewUser) {
                        $emailData = [
                            'fullName' => $newUser->first_name,
                            'email' => $newUser->email,
                            'expirationTime' => configItem('account.expiry'),
                            'activation_url' => URL::temporarySignedRoute('user.account.activation', Carbon::now()->addHours(configItem('account.expiry')), ['userUid' => $newUser->_uid]),
                        ];
                        // check if email send to member
                        if ($this->baseMailer->notifyToUser(__tr('Your account registered successfully.'), 'account.activation', $emailData, $newUser->email)) {
                            return $this->userRepository->transactionResponse(1, [
                                'show_message' => true,
                                'activation_required' => true,
                            ], __tr('Your account created successfully, to activate your account please check your email.'));
                        }
                        //if mail not send
                        return $this->userRepository->transactionResponse(2, ['show_message' => true], __tr('Failed to send activation email, please try again later.'));
                    } else {
                        $this->userRepository->transactionResponse(1, ['show_message' => true], __tr('Your account created successfully.'));
                        //show login credentials
                        $loginCredentials = [
                            'email' => $inputData['email'],
                            'password' => $inputData['password'],
                        ];
                        $user = $this->userRepository->fetchByEmailOrUsername($inputData['email']);
                        //get remember me data
                        $remember_me = (isset($input['remember_me']) and $input['remember_me'] == 'on') ? true : false;
                        //check login credentials and remember me checked
                        if (Auth::attempt($loginCredentials, $remember_me)) {
                            //if mobile request
                            if (isMobileAppRequest()) {
                                //issue new token
                                $authToken = YesTokenAuth::issueToken([
                                    'aud' => $user->_id,
                                    'uaid' => $user->user_authority_id,
                                ]);

                                return $this->userRepository->transactionResponse(1, [
                                    'activation_required' => false,
                                    'auth_info' => getUserAuthInfo(1),
                                    'access_token' => $authToken,
                                ], 'Welcome, you are logged in successfully.');
                            }
                            //if send welcome mail to user is active
                            if (getAppSettings('send_welcome_email_to_newly_registered_users')) {
                                $this->userWelcomeNotifyMail($newUser);
                            }
                            return $this->userRepository->transactionResponse(1, [
                                'auth_info' => getUserAuthInfo(1),
                                'intendedUrl' => Session::get('intendedUrl'),
                                'show_message' => true,
                            ], __tr('Welcome, you are logged in successfully.'));
                        }
                        // Store every login attempt.
                        $this->userRepository->updateLoginAttempts();

                        return $this->userRepository->transactionResponse(2, ['show_message' => true], __tr('Authentication failed, please check your credentials & try again.'));
                    }
                }
            }
            // Send failed server error message
            return $this->userRepository->transactionResponse(2, ['show_message' => true], __tr('Something went wrong on server, please contact to administrator.'));
        });

        return $this->engineReaction($transactionResponse);
    }

    /**
     * Process user update password request.
     *
     * @param  array  $inputData
     * @return array
     *---------------------------------------------------------------- */
    public function processUpdatePassword($inputData)
    {
        $user = Auth::user();
        // Check if logged in user password matched with entered password
        if (!Hash::check($inputData['current_password'], $user->password)) {
            return $this->engineReaction(3, ['show_message' => true], __tr('Current password is incorrect.'));
        }

        // Check if user password updated
        if ($this->userRepository->updatePassword($user, $inputData['new_password'])) {
            return $this->engineReaction(1, ['show_message' => true], __tr('Password updated successfully'));
        }

        return $this->engineReaction(14, ['show_message' => true], __tr('Password not updated.'));
    }

    /**
     * Send new email activation reminder.
     *
     * @param  array  $inputData
     * @return array
     *---------------------------------------------------------------- */
    public function processChangeEmail($inputData)
    {
        $user = Auth::user();
        // Check if user entered correct password or not
        if (!Hash::check($inputData['current_password'], $user->password)) {
            return $this->engineReaction(3, ['show_message' => true], __tr('Please check your password.'));
        }
        //get data
        $activationRequired = getAppSettings('activation_required_for_change_email');

        //check activation required or not
        if ($activationRequired) {
            $emailData = [
                'full_name' => $user->first_name . ' ' . $user->last_name,
                'newEmail' => $inputData['new_email'],
                'expirationTime' => configItem('account.change_email_expiry'),
                'activation_url' => URL::temporarySignedRoute('user.new_email.activation', Carbon::now()->addHours(configItem('account.change_email_expiry')), ['userUid' => $user->_uid, 'newEmail' => $inputData['new_email']]),

            ];
            // check if email send to member
            if ($this->baseMailer->notifyToUser(__tr('New Email Activation.'), 'account.new-email-activation', $emailData, $inputData['new_email'])) {
                return $this->engineReaction(1, ['show_message' => true, 'activationRequired' => true], __tr('New email activation link has been sent to your new email address, please check your email.'));
            } else {
                return $this->engineReaction(2, ['show_message' => true], __tr('Failed to send confirmation email.'));
            }
        } else {
            $updateData = [
                'email' => $inputData['new_email'],
            ];
            // Check for if new email activation store
            if ($this->userRepository->updateUser($user, $updateData)) {
                return $this->engineReaction(1, [
                    'show_message' => true,
                    'activationRequired' => false,
                    'newEmail' => $inputData['new_email'],
                ], __tr('Update email successfully.'));
            }
        }
        //error response
        return $this->engineReaction(2, ['show_message' => true], __tr('Email not updated.'));
    }

    /**
     * Activate new email.
     *
     * @param  number  $userID
     * @return array
     *---------------------------------------------------------------- */
    public function processNewEmailActivation($userUid, $newEmail)
    {
        $user = $this->userRepository->fetch($userUid);
        // Check if user record exist
        if (__isEmpty($user)) {
            return $this->engineReaction(2, null, __tr('User data not exists.'));
        }
        $updateData = [
            'email' => $newEmail,
        ];

        // Check for if new email activation store
        if ($this->userRepository->updateUser($user, $updateData)) {
            return $this->engineReaction(1, ['show_message' => true], __tr('Update email successfully.'));
        }
        //error response
        return $this->engineReaction(2, ['show_message' => true], __tr('Email not updated.'));
    }

    /**
     * Process forgot password request based on passed email address &
     * send password reminder on enter email address.
     *
     * @param  string  $email
     * @return array
     *---------------------------------------------------------------- */
    public function sendPasswordReminder($inputData)
    {
        // Verify recaptcha
        if (getAppSettings('allow_recaptcha') && !$this->checkRecaptcha($inputData)) {
            return $this->engineReaction(2, ['show_message' => true], __tr('Invalid Re-captcha.'));
        }
        $email = $inputData['email'];
        $user = $this->userRepository->fetchActiveUserByEmail($email);

        // Check if user record exist
        if (__isEmpty($user)) {
            return $this->engineReaction(2, ['show_message' => true], __tr('Invalid Request.'));
        }

        // Delete old password reminder for this user
        $this->userRepository->deleteOldPasswordReminder($email);

        $token = YesSecurity::generateUid();
        $createdAt = getCurrentDateTime();

        $storeData = [
            'email' => $email,
            'token' => $token,
            'created_at' => $createdAt,
        ];

        // Check for if password reminder added
        if (!$this->userRepository->storePasswordReminder($storeData)) {
            return $this->engineReaction(2, ['show_message' => true], __tr('Invalid Request.'));
        }

        //message data
        $emailData = [
            'full_name' => $user->first_name . ' ' . $user->last_name,
            'email' => $user->email,
            'expirationTime' => config('__tech.account.password_reminder_expiry'),
            'email' => $user->email,
            'email' => $user->email,
            'email' => $user->email,
            'tokenUrl' => route('user.reset_password', ['reminderToken' => $token]),
        ];

        // if reminder mail has been sent
        if ($this->baseMailer->notifyToUser(__tr('Forgot Password.'), 'account.password-reminder', $emailData, $user->email)) {
            return $this->engineReaction(1, ['show_message' => true], __tr('We have e-mailed your password reset link.')); // success reaction
        }

        return $this->engineReaction(2, ['show_message' => true], __tr('Something went wrong on server')); // error reaction
    }

    /**
     * Process reset password request.
     *
     * @param  array  $input
     * @param  string  $reminderToken
     * @return array
     *---------------------------------------------------------------- */
    public function processResetPassword($input, $reminderToken)
    {
        $email = $input['email'];

        //check if mobile app request then change request Url
        $token = $reminderToken;

        //get password reminder count
        $count = $this->userRepository->fetchPasswordReminderCount($token, $email);

        // Check if reminder count not exist on 0
        if (!$count > 0) {
            return  $this->engineReaction(18, ['show_message' => true], __tr('Invalid Request.'));
        }

        //fetch active user by email
        $user = $this->userRepository->fetchActiveUserByEmail($email);

        // Check if user record exist
        if (__isEmpty($user)) {
            return  $this->engineReaction(18, ['show_message' => true], __tr('Invalid Request.'));
        }

        // Check if user password updated
        if ($this->userRepository->resetPassword($user, $input['password'])) {
            return  $this->engineReaction(1, ['show_message' => true], __tr('Password reset successfully.'));
        }

        //failed response
        return  $this->engineReaction(2, ['show_message' => true], __tr('Password not updated.'));
    }

    /**
     * Process Account Activation.
     *
     * @param  string  $userUid
     *
     *-----------------------------------------------------------------------*/
    public function processAccountActivation($userUid)
    {
        //fetch never activated user
        $neverActivatedUser = $this->userRepository->fetchNeverActivatedUser($userUid);

        // Check if never activated user exist or not
        if (__isEmpty($neverActivatedUser)) {
            return $this->engineReaction(18, null, __tr('Account Activation link invalid.'));
        }

        $updateData = [
            'status' => 1, // Active
        ];
        // Check if user activated successfully
        if ($this->userRepository->updateUser($neverActivatedUser, $updateData)) {
            // give referral bonus if applicable on signup
            if((getAppSettings('give_bonus', 'select_type') == 1)) {
                storeReferralWalletTransaction($neverActivatedUser->_id);
            }
            if (getAppSettings('send_welcome_email_to_newly_registered_users')) {
                $this->userWelcomeNotifyMail($neverActivatedUser);
            }
            return $this->engineReaction(1, null, __tr('Your account has been activated successfully. Login with your email ID and password.'));
        }

        return  $this->engineReaction(2, null, __tr('Account Activation link invalid.'));
    }

    /**
     * Prepare User Profile Data.
     *
     * @param  string  $userName
     *
     *-----------------------------------------------------------------------*/
    public function prepareUserProfile($userName)
    {
        // fetch User by username
        $user = $this->userRepository->fetchByUsername($userName, true);

        // check if user exists
        if (__isEmpty($user)) {
            return $this->engineReaction(18, [], __tr('User does not exists.'));
        }
        $userId = $user->_id;
        $userUid = $user->_uid;
        $loggedInUserId = getUserID();
        $isOwnProfile = ($userId == $loggedInUserId) ? true : false;
        $explodeMobileNumber = explodeMobileNumber($user->mobile_number);

        // Prepare user data
        $userData = [
            'userId' => $userId,
            'userUId' => $userUid,
            'fullName' => $user->first_name . ' ' . $user->last_name,
            'first_name' => $user->first_name,
            'last_name' => $user->last_name,
            'mobile_number' => $explodeMobileNumber['mobile_number'],
            'userName' => $user->username,
            'country_code' => $explodeMobileNumber['country_code'],
        ];

        $userProfileData = [];

        // fetch User details
        $userProfile = $this->userSettingRepository->fetchUserProfile($userId);
        $userSettingConfig = configItem('user_settings');
        $profilePictureFolderPath = getPathByKey('profile_photo', ['{_uid}' => $userUid]);
        $profilePictureUrl = noThumbImageURL();
        $coverPictureFolderPath = getPathByKey('cover_photo', ['{_uid}' => $userUid]);
        $coverPictureUrl = noThumbCoverImageURL();
        // Check if user profile exists
        if (!__isEmpty($userProfile)) {
            if (!__isEmpty($userProfile->profile_picture)) {
                $profilePictureUrl = getMediaUrl($profilePictureFolderPath, $userProfile->profile_picture);
            }
            if (!__isEmpty($userProfile->cover_picture)) {
                $coverPictureUrl = getMediaUrl($coverPictureFolderPath, $userProfile->cover_picture);
            }
        }
        // Set cover and profile picture url
        $userData['profilePicture'] = $profilePictureUrl;
        $userData['coverPicture'] = $coverPictureUrl;
        $userData['userAge'] = isset($userProfile->dob) ? Carbon::parse($userProfile->dob)->age : null;

        // check if user profile exists
        if (!\__isEmpty($userProfile)) {
            // Get country name
            $countryName = '';
            if (!__isEmpty($userProfile->countries__id)) {
                $country = $this->countryRepository->fetchById($userProfile->countries__id, ['name']);
                $countryName = $country->name;
            }

            $userProfileData = [
                'mobile_number' => $user->mobile_number,
                'gender' => $userProfile->gender,
                'gender_text' => array_get($userSettingConfig, 'gender.' . $userProfile->gender),
                'country' => $userProfile->countries__id,
                'country_name' => $countryName,
                'dob' => $userProfile->dob,
                'birthday' => (!\__isEmpty($userProfile->dob))
                    ? formatDate($userProfile->dob)
                    : '',
                'formatted_education' => array_get($userSettingConfig, 'educations.' . $userProfile->education),
            ];
        }

        return $this->engineReaction(1, [
            'isOwnProfile' => $isOwnProfile,
            'userData' => $userData,
            'countries' => $this->countryRepository->fetchAll()->toArray(),
            'genders' => $userSettingConfig['gender'],
            'userProfileData' => $userProfileData,
        ]);
    }


    /**
     * Prepare profile visitors Data.
     *
     *-----------------------------------------------------------------------*/
    public function prepareUserData($username)
    {
        //get user by username
        $user = $this->userRepository->fetchByName($username);
        //chech if empty
        if (__isEmpty($user)) {
            return $this->engineReaction(2, []);
        }
        //concatinate users full name
        $userFullName = $user->first_name . ' ' . $user->last_name;
        return $this->engineReaction(1, [
            'userFullName' => $userFullName,
        ]);
    }

    /**
     * Prepare User Array Data.
     *
     *-----------------------------------------------------------------------*/
    public function prepareUserArray($userCollection)
    {
        $userData = [];
        //check if not empty collection
        if (!__isEmpty($userCollection)) {
            foreach ($userCollection as $key => $user) {
                $userImageUrl = '';
                //check is not empty
                if (!__isEmpty($user->profile_picture)) {
                    $profileImageFolderPath = getPathByKey('profile_photo', ['{_uid}' => $user->userUId]);
                    $userImageUrl = getMediaUrl($profileImageFolderPath, $user->profile_picture);
                } else {
                    $userImageUrl = noThumbImageURL();
                }

                $userAge = isset($user->dob) ? Carbon::parse($user->dob)->age : '';
                $gender = isset($user->gender) ? configItem('user_settings.gender', $user->gender) : '';

                $userData[] = [
                    '_id' => $user->userId,
                    '_uid' => $user->userUId,
                    'status' => $user->status,
                    'created_at' => formatDiffForHumans($user->created_at),
                    'updated_at' => formatDiffForHumans($user->updated_at),
                    'userFullName' => $user->userFullName,
                    'username' => $user->username,
                    'userImageUrl' => $userImageUrl,
                    // 'userCoverUrl' => $userCoverUrl,
                    'profilePicture' => $user->profile_picture,
                    'gender' => $gender,
                    'dob' => $user->dob,
                    'userAge' => $userAge,
                    'countryName' => $user->countryName,
                    'detailString' => implode(', ', array_filter([__tr($userAge), $gender])),
                ];
            }
        }

        return $userData;
    }


    /**
     * Process user contact request.
     *
     * @param  array  $inputData
     * @return array
     *---------------------------------------------------------------- */
    public function processContact($inputData)
    {
        //contact email data
        $emailData = [
            'userName' => $inputData['fullName'],
            'senderEmail' => $inputData['email'],
            'toEmail' => getAppSettings('contact_email'),
            'subject' => $inputData['subject'],
            'messageText' => $inputData['message'],
        ];
        // Verify recaptcha
        if (getAppSettings('allow_recaptcha') && !$this->checkRecaptcha($inputData)) {
            return $this->engineReaction(2, ['show_message' => true], __tr('Invalid Re-captcha.'));
        }

        // check if email send to member
        if ($this->baseMailer->notifyAdmin($inputData['subject'], 'contact', $emailData, 2)) {
            //success response
            return $this->engineReaction(1, ['show_message' => true], __tr('Mail has been sent successfully, We will contact soon.'));
        }
        //error response
        return $this->engineReaction(2, ['show_message' => true], __tr('Failed to send mail.'));
    }

    public function checkRecaptcha($inputData)
    {
        //check recaptcha response
        if(!isset($inputData['g-recaptcha-response'])) {
            return false;
        }
        $recaptcha_token = $inputData['g-recaptcha-response'];
        try {
            // Make a POST request to the reCAPTCHA verification endpoint
            $response = Http::asForm()->post('https://www.google.com/recaptcha/api/siteverify', [
                'secret' => getAppSettings('recaptcha_secret_key'),
                'response' => $recaptcha_token, // The token generated by the reCAPTCHA client-side library
                'remoteip' => request()->ip(), // The IP address of the user submitting the reCAPTCHA
            ]);
            //json response
            $responseData = $response->json();

            // Check if the verification was successful
            if (isset($responseData['success']) && $responseData['success'] == 1) {
                return true;
            } else {
                return false;
            }
        } catch (RequestException $e) {
            // An error occurred while making the request
            // Handle the error here
            return false;
        }
    }


    /**
     * Process delete account.
     *
     * @param  array  $inputData
     * @return array
     *---------------------------------------------------------------- */
    public function processDeleteAccount($inputData)
    {
        // Check if user exists
        if (isAdmin()) {
            return $this->engineReaction(18, ['show_message' => true], __tr('Admin can not be deleted.'));
        }
        //fetch user by id
        $user = $this->userRepository->fetchByID(getUserID());

        // Check if user exists
        if (__isEmpty($user)) {
            return $this->engineReaction(18, ['show_message' => true], __tr('User does not exists.'));
        }
        //check password
        if (!Hash::check($inputData['password'], $user->password)) {
            return $this->engineReaction(3, ['show_message' => true], __tr('Current password is incorrect.'));
        }

        // Delete all media of user
        $deletedMedia = $this->mediaEngine->deleteUserAccount();

        // Delete account successfully
        if ($this->userRepository->deleteUser($user)) {
            // log debug entry
            activityLog($user->username . ' ' . 'deleted himself.');
            // Process Logout user
            $this->processLogout();

            return $this->engineReaction(1, ['show_message' => true], __tr('Your account has been deleted successfully.'));
        }

        return $this->engineReaction(2, ['show_message' => true], __tr('Account not deleted.'));
    }

    /**
     * Process delete account.
     *
     * @param  array  $inputData
     * @return array
     *---------------------------------------------------------------- */
    public function resendActivationMail($inputData)
    {
        //fetch user email
        $user = $this->userRepository->fetchByEmail($inputData['email']);

        // Check if user exists
        if (__isEmpty($user)) {
            return $this->engineReaction(18, ['show_message' => true], __tr('You are not a member of this system.'));
        }

        // Check if user exists
        if ($user->status == 1) {
            return $this->engineReaction(18, ['show_message' => true], __tr('Account already activated.'));
        }

        $transactionResponse = $this->userRepository->processTransaction(function () use ($user) {
            if ($updatedUser = $this->userRepository->updateUser($user, [
                'remember_token' => Utils::generateStrongPassword(4, false, 'ud'),
            ])) {
                $emailData = [
                    'fullName' => $user->first_name,
                    'email' => $user->email,
                    'expirationTime' => configItem('otp_expiry'),
                    'otp' => $updatedUser->remember_token,
                ];

                // check if email send to member
                if ($this->baseMailer->notifyToUser(__tr('Activation mail sent.'), 'account.activation-for-app', $emailData, $user->email)) {
                    return $this->userRepository->transactionResponse(1, [
                        'show_message' => true,
                        'activation_mail_sent' => true,
                    ], __tr('Activation mail sent successfully, to activate your account please check your email.'));
                }

                return $this->userRepository->transactionResponse(2, ['show_message' => true], __tr('Failed to send activation mail'));
            }
        });

        return $this->engineReaction($transactionResponse);
    }

    /**
     * Process delete account.
     *
     * @param  array  $inputData
     * @return array
     *---------------------------------------------------------------- */
    public function requestNewPassword($inputData)
    {
        //fetch user email which requested for password change
        $user = $this->userRepository->fetchByEmail($inputData['email']);

        // Check if user exists
        if (__isEmpty($user)) {
            return $this->engineReaction(18, ['show_message' => true], __tr('You are not a member of this system.'));
        }

        // Check if user exists
        if ($user->status != 1) {
            return $this->engineReaction(18, [
                'show_message' => true,
            ], __tr('Your account might be Inactive, Blocked or Not Activated.'));
        }

        $transactionResponse = $this->userRepository->processTransaction(function () use ($inputData, $user) {
            // Delete old password reminder for this user
            $this->userRepository->appDeleteOldPasswordReminder($inputData['email']);

            $currentDateTime = Carbon::now();
            $token = Utils::generateStrongPassword(4, false, 'ud');
            $createdAt = $currentDateTime->addSeconds(configItem('otp_expiry'));

            $storeData = [
                'email' => $inputData['email'],
                'token' => $token,
                'created_at' => $createdAt,
            ];

            // Check for if password reminder added
            if (!$this->userRepository->storePasswordReminder($storeData)) {
                return $this->userRepository->transactionResponse(2, ['show_message' => true], __tr('Invalid Request.'));
            }

            $otpExpiry = configItem('otp_expiry');
            $differenceSeconds = Carbon::now()->diffInSeconds($createdAt, false);
            $newExpiryTime = 0;
            if ($differenceSeconds > 0 and $differenceSeconds < $otpExpiry) {
                $newExpiryTime = $differenceSeconds;
            }

            $emailData = [
                'fullName' => $user->first_name . ' ' . $user->last_name,
                'email' => $user->email,
                'expirationTime' => config('__tech.account.app_password_reminder_expiry'),
                'otp' => $token,
            ];

            // check if email send to member
            if ($this->baseMailer->notifyToUser(__tr('OTP Verification'), 'account.forgot-password-for-app', $emailData, $user->email)) {
                return $this->userRepository->transactionResponse(1, [
                    'show_message' => true,
                    'mail_sent' => true,
                    'newExpiryTime' => $newExpiryTime,
                ], __tr('OTP sent successfully, to reset password use OTP sent to your email.'));
            }

            return $this->userRepository->transactionResponse(2, ['show_message' => true], __tr('Failed to send OTP'));
        });

        return $this->engineReaction($transactionResponse);
    }

    /**
     * Process Forgot Password resend otp request
     *
     * @param $userEmail array - userEmail data
     * @return json object
     */
    public function processForgotPasswordResendOtp($userEmail)
    {
        $transactionResponse = $this->userRepository->processTransaction(function () use ($userEmail) {
            $user = $this->userRepository->fetchActiveUserByEmail($userEmail);

            // Check if empty then return error message
            if (__isEmpty($user)) {
                return $this->userRepository->transactionResponse(2, null, 'You are not a member of the system, Please go and register first , then you can proceed for login.');
            }

            // Delete old password reminder for this user
            $this->userRepository->appDeleteOldPasswordReminder($user->email);

            //check if mobile app request then change request Url
            $currentDateTime = Carbon::now();
            $token = Utils::generateStrongPassword(4, false, 'ud');
            $createdAt = $currentDateTime->addSeconds(configItem('otp_expiry'));

            $storeData = [
                'email' => $user->email,
                'token' => $token,
                'created_at' => $createdAt,
            ];

            // Check for if password reminder added
            if (!$this->userRepository->storePasswordReminder($storeData)) {
                return $this->userRepository->transactionResponse(2, ['show_message' => true], __tr('Invalid Request.'));
            }

            $emailData = [
                'fullName' => $user->first_name,
                'email' => $user->email,
                'expirationTime' => config('__tech.account.app_password_reminder_expiry'),
                'otp' => $token,
            ];

            $otpExpiry = configItem('otp_expiry');
            $differenceSeconds = Carbon::now()->diffInSeconds($createdAt, false);
            $newExpiryTime = 0;
            if ($differenceSeconds > 0 and $differenceSeconds < $otpExpiry) {
                $newExpiryTime = $differenceSeconds;
            }

            // check if email send to member
            if ($this->baseMailer->notifyToUser(__tr('OTP Verification'), 'account.forgot-password-for-app', $emailData, $user->email)) {
                return $this->userRepository->transactionResponse(1, [
                    'show_message' => true,
                    'mail_sent' => true,
                    'newExpiryTime' => $newExpiryTime,
                ], __tr('OTP sent successfully, to reset password use OTP sent to your email.'));
            }

            return $this->userRepository->transactionResponse(2, null, 'Invalid Request'); // error reaction
        });

        return $this->engineReaction($transactionResponse);
    }


    /**
     * Process reset password request.
     *
     * @param  array  $input
     * @param  string  $reminderToken
     * @return array
     *---------------------------------------------------------------- */
    public function changeEmailProcess($input)
    {
        $email = $input['current_email'];

        //fetch active user by email
        $user = $this->userRepository->fetchActiveUserByEmail($email);

        // Check if user record exist
        if (__isEmpty($user)) {
            return  $this->engineReaction(18, ['show_message' => true], __tr('Invalid Request.'));
        }

        // Check if user entered correct password or not
        if (!Hash::check($input['current_password'], $user->password)) {
            return $this->engineReaction(3, [], __tr('Authentication Failed. Please Check Your Password.'));
        }

        // Check if user password updated
        if ($this->userRepository->updateUser($user, ['email' => $input['new_email']])) {
            return  $this->engineReaction(1, [
                'show_message' => true,
            ], __tr('Email updated successfully.'));
        }

        //failed response
        return  $this->engineReaction(2, ['show_message' => true], __tr('Email not updated.'));
    }

    /**
     * prepare user profile
     *
     * @return array
     *----------------------------------------------------------------*/
    public function userWelcomeNotifyMail($newUser)
    {
        //array of user detail data
        $emailData = [
            'fullName' => $newUser->first_name,
            'email' => $newUser->email,
        ];
        //welcome email subject
        $welcomeEmailSubject = __tr("Welcome To __siteName__", [
            '__siteName__' => getAppSettings('name')
        ]);

        if ($this->baseMailer->notifyToUser($welcomeEmailSubject, 'account.welcome', $emailData, $newUser->email)) {
            return true;
        }
    }
    /**
     * process To Send Otp
     *
     * @return json object
     *----------------------------------------------------------------*/
    public function processSendOtp($inputData)
    {
        // Verify recaptcha
        if (getAppSettings('allow_recaptcha') && !$this->checkRecaptcha($inputData)) {
            return $this->engineReaction(2, ['show_message' => true], __tr('Invalid Re-captcha.'));
        }

        $emailOrMobile = $inputData['emailOrMobile'];
        //check email or mobile
        if (str_contains($emailOrMobile, '@')) {
            return $this->sendOtpEmail($inputData);
        } else {
            if (getAppSettings('use_enable_sms_settings') == false) {
                return $this->engineReaction(2, ['show_message' => true], __tr('Mobile OTP is not available.'));
            }
        }
        //check active user with ByEmailOrUsernameOrNumber
        if (!$this->userRepository->fetchActiveUserByEmailOrUsernameOrNumber($emailOrMobile)) {
            return $this->engineReaction(2, ['show_message' => true], __tr('Please use valid user credentials.'));
        }

        // Generate a random 6-digit OTP
        $otp = rand(100000, 999999);

        // Send the OTP to the user's phone number
        $message = __tr('Your OTP for __siteName__ is __loginOtp__', [
            '__siteName__' => getAppSettings('name'),
            '__loginOtp__' => $otp
        ]); // Replace with your actual OTP message
        $explodeNumber = explode('-', $emailOrMobile);
        $countryCode = str_replace('0', '+', $explodeNumber[0]);
        $phone_numbers = str_replace($explodeNumber[0] . '-', $countryCode, $emailOrMobile);

        try {
            $this->smsProviders();
            Sms::via(getAppSettings('sms_driver'))->send($message, function ($sms) use ($phone_numbers) {
                $sms->to($phone_numbers); # The numbers to send to.
            });

            // Store the otp in the session for later validation
            Session::put(['sms_otp' => $otp, 'emailOrMobile' => $inputData['emailOrMobile']]);

            //Send response back to controller
            return $this->engineReaction(1, ['show_message' => true], __tr('OTP has been sent successfully.'));
        } catch (\Throwable $th) {
            return $this->engineReaction(2, ['show_message' => true], $th->getMessage());
        }
    }

    /**
     * Verify Otp Process
     *
     * @param   array  $inputData
     *
     * @return  json object
     */
    public function verifyOtpProcess($inputData)
    {
        // Verify recaptcha
        if (getAppSettings('allow_recaptcha') && !$this->checkRecaptcha($inputData)) {
            return $this->engineReaction(2, ['show_message' => true], __tr('Invalid Re-captcha.'));
        }
        //if sms otp selected
        if (Session::has('sms_otp')) {
            $verification = Session::get('sms_otp');//otp on sms
            $emailOrMobile = Session::get('emailOrMobile'); //otp on mail

            if ($verification != $inputData['otp']) {
                return $this->engineReaction(2, ['show_message' => true], __tr('Invalid OTP.'));
            }

            // OTP is valid, handle the successful verification
            if ($verification == $inputData['otp']) {

                Session::forget('sms_otp');

                $user = $this->userRepository->fetchByEmailOrUsername($emailOrMobile);

                //check user is empty
                if (__isEmpty($user)) {
                    return $this->engineReaction(2, null, __tr('User not exists.'));
                }

                //if user not active then show message
                if ($user->status != 1) {
                    return $this->engineReaction(2, null, __tr('Your account currently __status__, Please contact to administrator.', ['__status__' => configItem('status_codes', $user->status)]));
                }

                // Get logged in if credentials valid
                if (Auth::loginUsingId($user->_id)) {
                    //success response
                    return $this->engineReaction(1, [
                        'show_message' => true,
                        'auth_info' => getUserAuthInfo(1),
                    ], __tr('Welcome, you are logged in successfully.'));
                }
            }
            return $this->engineReaction(1, ['show_message' => true], __tr('Verification fail.'));
        }

        return $this->engineReaction(2, ['show_message' => true], __tr('Something went wrong.'));
    }

    /**
     * Send Otp to Email
     *
     * @param   array  $inputData
     *
     * @return  json object
     */
    public function sendOtpEmail($inputData)
    {
        $transactionResponse = $this->userRepository->processTransaction(function () use ($inputData) {
            $otp = rand(100000, 999999);
            $email = $inputData['emailOrMobile'];

            $emailData = [
                'email' => $email,
                'otp' => $otp
            ];

            // check if email send to member
            if ($this->baseMailer->notifyToUser(__tr('Your OTP Login Code for __siteName__.', [
                '__siteName__' => getAppSettings('name')
            ]), 'account.login-with-otp', $emailData, $inputData['emailOrMobile'])) {

                // Store the otp in the session for later validation
                Session::put(['sms_otp' => $otp, 'emailOrMobile' => $email]);

                return $this->userRepository->transactionResponse(1, [
                    'show_message' => true,
                ], __tr('OTP Send to Your Registered Email Address.'));
            }

            return $this->userRepository->transactionResponse(2, ['show_message' => true], __tr('Failed to send activation mail'));
        });

        return $this->engineReaction($transactionResponse);
    }

    /**
     * SMS Providers Settings
     *
     * @return
     */
    public function smsProviders()
    {
        //return sms data
        return config([
            'sms.drivers.twilio.sid' => getAppSettings('sms_twilio_sid'),
            'sms.drivers.twilio.token' => getAppSettings('sms_twilio_token'),
            'sms.drivers.twilio.from' => getAppSettings('sms_twilio_from'),

            'sms.drivers.textlocal.from' => getAppSettings('sms_textlocal_from'),
            'sms.drivers.textlocal.username' => getAppSettings('sms_textlocal_username'),
            'sms.drivers.textlocal.hash' => getAppSettings('sms_textlocal_hash'),

            'sms.drivers.sms77.apiKey' => getAppSettings('sms_sms77_apiKey'),
            'sms.drivers.sms77.flash' => getAppSettings('sms_sms77_flash'),
            'sms.drivers.sms77.from' => getAppSettings('sms_sms77_from'),
        ]);
    }
}
