<?php
/**
* UserSettingRepository.php - Repository file
*
* This file is part of the UserSetting component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\UserSetting\Repositories;

use App\Yantrana\Base\BaseRepository;
use App\Yantrana\Components\User\Models\User;
use App\Yantrana\Components\User\Models\UserProfile;
use App\Yantrana\Components\UserSetting\Interfaces\UserSettingRepositoryInterface;
use App\Yantrana\Components\UserSetting\Models\UserSettingModel;
use App\Yantrana\Support\CommonTrait;
use Carbon\Carbon;
use DB;

class UserSettingRepository extends BaseRepository implements UserSettingRepositoryInterface
{
    /**
     * @var CommonTrait - Common Trait
     */
    use CommonTrait;

    /**
     * Fetch All Record from Cache
     *
     * @param  array  $names
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchUserSettingByName($name)
    {
        return UserSettingModel::whereIn('key_name', $name)
            ->select('_id', 'key_name', 'value', 'data_type', 'users__id')
            ->where('users__id', getUserID())
            ->get();
    }

    /**
     * Store or update user setting data
     *
     * @param  array  $inputData
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function storeOrUpdate($inputData)
    {
        // Check if data updated or inserted
        if (UserSettingModel::bunchInsertUpdate($inputData, '_id')) {
            return true;
        }

        return false;
    }

    /**
     * Delete user setting by keys
     *
     * @param  array  $userSettingKeyNames
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function deleteUserSetting($userSettingKeyNames)
    {
        if (UserSettingModel::whereIn('_id', $userSettingKeyNames)->delete()) {
            return true;
        }

        return false;
    }

    /**
     * Fetch the record of UserSetting
     *
     * @param  int  $userId
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchUserDetails($userId)
    {
        return User::where('users._id', $userId)
            ->leftJoin('user_profiles', 'users._id', '=', 'user_profiles.users__id')
            ->select(
                \__nestedKeyValues([
                    'users' => [
                        '_id',
                        'username',
                        'email',
                        'first_name',
                        'last_name',
                        'designation',
                        'mobile_number',
                    ],
                    'user_profiles' => [
                        '_id AS user_profile_id',
                        'users__id',
                        'countries__id',
                        'profile_picture',
                        'gender',
                        'dob',
                    ],
                ])
            )
            ->first();
    }

    /**
     * Fetch User profile
     *
     * @param  int  $userId
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function fetchUserProfile($userId)
    {
        return UserProfile::where('users__id', $userId)->first();
    }

    /**
     * Update User
     *
     * @param  object  $user
     * @param  array  $updateData
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function updateUser($user, $updateData)
    {
        if ($user->modelUpdate($updateData)) {
            return true;
        }

        return false;
    }

    /**
     * Store user profile
     *
     * @param  array  $inputData
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function storeUserProfile($inputData)
    {
        $keyValues = [
            'users__id' => $inputData['user_id'],
            'countries__id',
            'gender' => array_get($inputData, 'gender'),
            'dob' => array_get($inputData, 'dob'),
        ];
        $userProfile = new UserProfile();
        // check if user profile stored successfully
        if ($userProfile->assignInputsAndSave($inputData, $keyValues)) {
            return $userProfile;
        }

        return false;
    }

    /**
     * Update User Profile
     *
     * @param  object  $userProfile
     * @param  array  $updateData
     * @return eloquent collection object
     *---------------------------------------------------------------- */
    public function updateUserProfile($userProfile, $updateData)
    {
        if ($userProfile->modelUpdate($updateData)) {
            return true;
        }

        return false;
    }

    /**
     * Delete photo.
     *
     * @param  object  $photo
     * @return bool|object
     *---------------------------------------------------------------- */
    public function deletePhoto($userPhoto)
    {
        // Check if page deleted
        if ($userPhoto->delete()) {
            return  $userPhoto;
        }

        return false;
    }
}
